@@ -1,28 +1,28 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
- * 
+ * Copyright (C) 2004, 2005, 2006 Andreas Huggel <ahuggel@gmx.net>
+ *
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA.
  */
 /*!
   @file    actions.hpp
   @brief   Implements base class Task, TaskFactory and the various supported
            actions (derived from Task).
-  @version $Rev: 575 $
+  @version $Rev: 771 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    11-Dec-03, ahu: created
@@ -58,7 +58,8 @@ namespace Exiv2 {
 namespace Action {
 
     //! Enumerates all tasks
-    enum TaskType { none, adjust, print, rename, erase, extract, insert, modify };
+    enum TaskType { none, adjust, print, rename, erase, extract, insert,
+                    modify, fixiso };
 
 // *****************************************************************************
 // class definitions
@@ -107,6 +108,8 @@ namespace Action {
           this method.
         */
         static TaskFactory& instance();
+        //! Destructor
+        void cleanup();
 
         /*!
           @brief  Create a task.
@@ -159,23 +162,21 @@ namespace Action {
 
         //! Print the Jpeg comment
         int printComment();
-        //! Print uninterpreted Iptc information 
+        //! Print uninterpreted Iptc information
         int printIptc();
         //! Print Exif summary information
-        int printSummary(); 
-        //! Print the interpreted value for each Exif tag
-        int printInterpreted();
-        //! Print uninterpreted Exif information
-        int printValues();
-        //! Print Exif information in hexdump format
-        int printHexdump();
+        int printSummary();
+        //! Print the list of Exif data in user defined format
+        int printList();
+        //! Print the label for a summary line
+        void printLabel(const std::string& label) const;
         /*!
           @brief Print one summary line with a label (if provided) and requested
                  data. A line break is printed only if a label is provided.
           @return 1 if a line was written, 0 if the key was not found.
          */
         int printTag(const Exiv2::ExifData& exifData,
-                     const std::string& key, 
+                     const std::string& key,
                      const std::string& label ="") const;
 
     private:
@@ -186,7 +187,7 @@ namespace Action {
     }; // class Print
 
     /*!
-      @brief %Rename a file to its metadate creation timestamp, 
+      @brief %Rename a file to its metadate creation timestamp,
              in the specified format.
      */
     class Rename : public Task {
@@ -211,7 +212,7 @@ namespace Action {
     private:
         virtual Adjust* clone_() const;
         int adjustDateTime(Exiv2::ExifData& exifData,
-                           const std::string& key, 
+                           const std::string& key,
                            const std::string& path) const;
 
         long adjustment_;
@@ -228,9 +229,9 @@ namespace Action {
         AutoPtr clone() const;
 
         /*!
-          @brief Delete the thumbnail image, incl IFD1 metadata from the file. 
+          @brief Delete the thumbnail image, incl IFD1 metadata from the file.
          */
-        int eraseThumbnail(Exiv2::Image* image) const; 
+        int eraseThumbnail(Exiv2::Image* image) const;
         /*!
           @brief Erase the complete Exif data block from the file.
          */
@@ -266,7 +267,7 @@ namespace Action {
                  "-thumb" and the appropriate suffix (".jpg" or ".tif"), depending
                  on the format of the Exif thumbnail image.
          */
-        int writeThumbnail() const; 
+        int writeThumbnail() const;
 
     private:
         virtual Extract* clone_() const;
@@ -297,7 +298,7 @@ namespace Action {
     }; // class Insert
 
     /*!
-      @brief %Modify the Exif data according to the commands in the 
+      @brief %Modify the Exif data according to the commands in the
              modification table.
      */
     class Modify : public Task {
@@ -306,23 +307,40 @@ namespace Action {
         virtual int run(const std::string& path);
         typedef std::auto_ptr<Modify> AutoPtr;
         AutoPtr clone() const;
-        Modify() {} 
+        Modify() {}
 
     private:
         virtual Modify* clone_() const;
-        //! Copy contructor needed because of AutoPtr memeber 
-        Modify(const Modify& src) {} 
+        //! Copy contructor needed because of AutoPtr member
+        Modify(const Modify& /*src*/) {}
 
-        //! Add a metadatum according to \em modifyCmd 
+        //! Add a metadatum according to \em modifyCmd
         void addMetadatum(const ModifyCmd& modifyCmd);
-        //! Set a metadatum according to \em modifyCmd 
+        //! Set a metadatum according to \em modifyCmd
         void setMetadatum(const ModifyCmd& modifyCmd);
-        //! Delete a metadatum according to \em modifyCmd 
+        //! Delete a metadatum according to \em modifyCmd
         void delMetadatum(const ModifyCmd& modifyCmd);
 
-        Exiv2::Image::AutoPtr image_;        //!< Image to modify 
+        Exiv2::Image::AutoPtr image_;        //!< Image to modify
     }; // class Modify
 
-}                                       // namespace Action 
+    /*!
+      @brief %Copy ISO settings from any of the Nikon makernotes to the
+             regular Exif tag, Exif.Photo.ISOSpeedRatings.
+     */
+    class FixIso : public Task {
+    public:
+        virtual ~FixIso() {}
+        virtual int run(const std::string& path);
+        typedef std::auto_ptr<FixIso> AutoPtr;
+        AutoPtr clone() const;
+
+    private:
+        virtual FixIso* clone_() const;
+        std::string path_;
+
+    }; // class FixIso
+
+}                                       // namespace Action
 
 #endif                                  // #ifndef ACTIONS_HPP_
