@@ -1,32 +1,32 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
- * 
+ * Copyright (C) 2004, 2005, 2006 Andreas Huggel <ahuggel@gmx.net>
+ *
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA.
  */
 /*
   File:      basicio.cpp
-  Version:   $Rev: 566 $
+  Version:   $Rev: 810 $
   Author(s): Brad Schick (brad) <brad@robotbattle.com>
   History:   04-Dec-04, brad: created
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: basicio.cpp 566 2005-04-26 15:27:41Z ahuggel $");
+EXIV2_RCSID("@(#) $Id: basicio.cpp 810 2006-06-02 12:59:04Z ahuggel $");
 
 // *****************************************************************************
 // included header files
@@ -44,7 +44,8 @@ EXIV2_RCSID("@(#) $Id: basicio.cpp 566 2005-04-26 15:27:41Z ahuggel $");
 // + standard includes
 #include <string>
 #include <cassert>
-#include <cstdio>                       // for remove()
+#include <cstdio>                       // for remove(), rename()
+#include <cstdlib>                      // for alloc(), realloc(), free()
 #include <sys/types.h>                  // for stat()
 #include <sys/stat.h>                   // for stat()
 #ifdef EXV_HAVE_PROCESS_H
@@ -62,11 +63,11 @@ EXIV2_RCSID("@(#) $Id: basicio.cpp 566 2005-04-26 15:27:41Z ahuggel $");
 // class member definitions
 namespace Exiv2 {
 
-    FileIo::FileIo(const std::string& path) : 
-        path_(path), fp_(0), opMode_(opSeek)
+    FileIo::FileIo(const std::string& path)
+        : path_(path), fp_(0), opMode_(opSeek)
     {
     }
-        
+
     FileIo::~FileIo()
     {
         close();
@@ -75,10 +76,10 @@ namespace Exiv2 {
     BasicIo::AutoPtr FileIo::temporary() const
     {
         BasicIo::AutoPtr basicIo;
-        
+
         struct stat buf;
         int ret = stat(path_.c_str(), &buf);
-        
+
         // If file is > 1MB then use a file, otherwise use memory buffer
         if (ret != 0 || buf.st_size > 1048576) {
             pid_t pid = getpid();
@@ -110,13 +111,13 @@ namespace Exiv2 {
         case opRead:
             // Flush if current mode allows reading, else reopen (in mode "r+b"
             // as in this case we know that we can write to the file)
-            if (   openMode_[0] == 'r' 
-                || openMode_.substr(0, 2) == "w+" 
+            if (   openMode_[0] == 'r'
+                || openMode_.substr(0, 2) == "w+"
                 || openMode_.substr(0, 2) == "a+") reopen = false;
             break;
         case opWrite:
             // Flush if current mode allows writing, else reopen
-            if (   openMode_.substr(0, 2) == "r+" 
+            if (   openMode_.substr(0, 2) == "r+"
                 || openMode_[0] == 'w'
                 || openMode_[0] == 'a') reopen = false;
             break;
@@ -176,22 +177,22 @@ namespace Exiv2 {
     {
         const bool wasOpen = (fp_ != 0);
         const std::string lastMode(openMode_);
-        
+
         FileIo *fileIo = dynamic_cast<FileIo*>(&src);
         if (fileIo) {
             // Optimization if this is another instance of FileIo
             close();
             fileIo->close();
             // MSVCRT rename that does not overwrite existing files
-            if (remove(path_.c_str()) != 0) {
-                throw Error(2, path_, strError(), "::remove");
+            if (fileExists(path_) && std::remove(path_.c_str()) != 0) {
+                throw Error(2, path_, strError(), "std::remove");
             }
-            if (rename(fileIo->path_.c_str(), path_.c_str()) == -1) {
+            if (std::rename(fileIo->path_.c_str(), path_.c_str()) == -1) {
                 throw Error(17, fileIo->path_, path_, strError());
             }
-            remove(fileIo->path_.c_str());
+            std::remove(fileIo->path_.c_str());
         }
-        else{
+        else {
             // Generic handling, reopen both to reset to start
             if (open("w+b") != 0) {
                 throw Error(10, path_, "w+b", strError());
@@ -219,26 +220,22 @@ namespace Exiv2 {
         if (switchMode(opWrite) != 0) return EOF;
         return putc(data, fp_);
     }
-    
+
     int FileIo::seek(long offset, Position pos)
     {
         assert(fp_ != 0);
-        int fileSeek;
-        if (pos == BasicIo::cur) {
-            fileSeek = SEEK_CUR;
-        }
-        else if (pos == BasicIo::beg) {
-            fileSeek = SEEK_SET;
-        }
-        else {
-            assert(pos == BasicIo::end);
-            fileSeek = SEEK_END;
+
+        int fileSeek = 0;
+        switch (pos) {
+        case BasicIo::cur: fileSeek = SEEK_CUR; break;
+        case BasicIo::beg: fileSeek = SEEK_SET; break;
+        case BasicIo::end: fileSeek = SEEK_END; break;
         }
 
         if (switchMode(opSeek) != 0) return 1;
         return fseek(fp_, offset, fileSeek);
     }
-        
+
     long FileIo::tell() const
     {
         assert(fp_ != 0);
@@ -258,9 +255,9 @@ namespace Exiv2 {
 
         struct stat buf;
         int ret = stat(path_.c_str(), &buf);
-        
+
         if (ret != 0) return -1;
-        return buf.st_size; 
+        return buf.st_size;
     }
 
     int FileIo::open()
@@ -286,7 +283,7 @@ namespace Exiv2 {
     {
         return fp_ != 0;
     }
-    
+
     int FileIo::close()
     {
         if (fp_ != 0) {
@@ -295,7 +292,7 @@ namespace Exiv2 {
         }
         return 0;
     }
-        
+
     DataBuf FileIo::read(long rcount)
     {
         assert(fp_ != 0);
@@ -323,7 +320,7 @@ namespace Exiv2 {
     {
         return fp_ != 0 ? ferror(fp_) : 0;
     }
-    
+
     bool FileIo::eof() const
     {
         assert(fp_ != 0);
@@ -335,32 +332,68 @@ namespace Exiv2 {
         return path_;
     }
 
-    MemIo::MemIo(const byte* data, long size)
+    MemIo::MemIo()
+        : data_(0),
+          idx_(0),
+          size_(0),
+          sizeAlloced_(0),
+          isMalloced_(false),
+          eof_(false)
     {
-        // If copying data is too slow it might be worth
-        // creating a readonly MemIo variant
-        data_.reserve(size);
-        data_.assign(data, data+size);
-        idx_ = 0;
+    }
+
+    MemIo::MemIo(const byte* data, long size)
+        : data_(const_cast<byte*>(data)),
+          idx_(0),
+          size_(size),
+          sizeAlloced_(0),
+          isMalloced_(false),
+          eof_(false)
+    {
+    }
+
+    MemIo::~MemIo()
+    {
+        if (isMalloced_) {
+            std::free(data_);
+        }
     }
 
     BasicIo::AutoPtr MemIo::temporary() const
     {
         return BasicIo::AutoPtr(new MemIo);
     }
-        
-    void MemIo::checkSize(long wcount)
+
+    void MemIo::reserve(long wcount)
     {
-        ByteVector::size_type need = wcount + idx_;
-        if (need > data_.size()) {
-            data_.resize(need);
+        long need = wcount + idx_;
+
+        if (!isMalloced_) {
+            // Minimum size for 1st block is 32kB
+            long size  = std::max(32768 * (1 + need / 32768), size_);
+            byte* data = (byte*)std::malloc(size);
+            std::memcpy(data, data_, size_);
+            data_ = data;
+            sizeAlloced_ = size;
+            isMalloced_ = true;
+        }
+
+        if (need > size_) {
+            if (need > sizeAlloced_) {
+                // Allocate in blocks of 32kB
+                long want = 32768 * (1 + need / 32768);
+                data_ = (byte*)std::realloc(data_, want);
+                sizeAlloced_ = want;
+                isMalloced_ = true;
+            }
+            size_ = need;
         }
     }
 
     long MemIo::write(const byte* data, long wcount)
     {
-        checkSize(wcount);
-        // According to Josuttis 6.2.3 this is safe
+        reserve(wcount);
+        assert(isMalloced_);
         memcpy(&data_[idx_], data, wcount);
         idx_ += wcount;
         return wcount;
@@ -371,27 +404,35 @@ namespace Exiv2 {
         MemIo *memIo = dynamic_cast<MemIo*>(&src);
         if (memIo) {
             // Optimization if this is another instance of MemIo
-            data_.swap(memIo->data_);
+            if (true == isMalloced_) {
+                std::free(data_);
+            }
             idx_ = 0;
+            data_ = memIo->data_;
+            size_ = memIo->size_;
+            isMalloced_ = memIo->isMalloced_;
+            memIo->idx_ = 0;
+            memIo->data_ = 0;
+            memIo->size_ = 0;
+            memIo->isMalloced_ = false;
         }
-        else{
+        else {
             // Generic reopen to reset position to start
-            data_.clear();
-            idx_ = 0;
             if (src.open() != 0) {
                 throw Error(9, src.path(), strError());
             }
+            idx_ = 0;
             write(src);
-            src.close();    
+            src.close();
         }
         if (error() || src.error()) throw Error(19, strError());
     }
 
     long MemIo::write(BasicIo& src)
     {
-        if (static_cast<BasicIo*>(this)==&src) return 0;
+        if (static_cast<BasicIo*>(this) == &src) return 0;
         if (!src.isopen()) return 0;
-        
+
         byte buf[4096];
         long readCount = 0;
         long writeTotal = 0;
@@ -405,44 +446,42 @@ namespace Exiv2 {
 
     int MemIo::putb(byte data)
     {
-        checkSize(1);
+        reserve(1);
+        assert(isMalloced_);
         data_[idx_++] = data;
         return data;
     }
-    
+
     int MemIo::seek(long offset, Position pos)
     {
-        ByteVector::size_type newIdx;
-        
-        if (pos == BasicIo::cur ) {
-            newIdx = idx_ + offset;
-        }
-        else if (pos == BasicIo::beg) {
-            newIdx = offset;
-        }
-        else {
-            assert(pos == BasicIo::end);
-            newIdx = data_.size() + offset;
+        long newIdx = 0;
+
+        switch (pos) {
+        case BasicIo::cur: newIdx = idx_ + offset; break;
+        case BasicIo::beg: newIdx = offset; break;
+        case BasicIo::end: newIdx = size_ + offset; break;
         }
 
-        if (newIdx < 0 || newIdx > data_.size()) return 1;
+        if (newIdx < 0 || newIdx > size_) return 1;
         idx_ = newIdx;
+        eof_ = false;
         return 0;
     }
 
     long MemIo::tell() const
     {
-        return (long)idx_;
+        return idx_;
     }
 
     long MemIo::size() const
     {
-        return (long)data_.size();
+        return size_;
     }
-    
+
     int MemIo::open()
     {
         idx_ = 0;
+        eof_ = false;
         return 0;
     }
 
@@ -450,12 +489,12 @@ namespace Exiv2 {
     {
         return true;
     }
-    
+
     int MemIo::close()
     {
         return 0;
     }
-        
+
     DataBuf MemIo::read(long rcount)
     {
         DataBuf buf(rcount);
@@ -466,19 +505,20 @@ namespace Exiv2 {
 
     long MemIo::read(byte* buf, long rcount)
     {
-        long avail = (long)(data_.size() - idx_);
+        long avail = size_ - idx_;
         long allow = std::min(rcount, avail);
-        
-        // According to Josuttis 6.2.3 this is safe
         memcpy(buf, &data_[idx_], allow);
         idx_ += allow;
+        if (rcount > avail) eof_ = true;
         return allow;
     }
 
     int MemIo::getb()
     {
-        if (idx_ == data_.size())
+        if (idx_ == size_) {
+            eof_ = true;
             return EOF;
+        }
         return data_[idx_++];
     }
 
@@ -486,10 +526,10 @@ namespace Exiv2 {
     {
         return 0;
     }
-    
+
     bool MemIo::eof() const
     {
-        return idx_ == data_.size();
+        return eof_;
     }
 
     std::string MemIo::path() const
