@@ -1,26 +1,26 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
- * 
+ * Copyright (C) 2004, 2005, 2006 Andreas Huggel <ahuggel@gmx.net>
+ *
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA.
  */
 /*
   File:      canonmn.cpp
-  Version:   $Rev: 569 $
+  Version:   $Rev: 754 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   18-Feb-04, ahu: created
              07-Mar-04, ahu: isolated as a separate component
@@ -30,7 +30,7 @@
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: canonmn.cpp 569 2005-05-28 05:48:43Z ahuggel $");
+EXIV2_RCSID("@(#) $Id: canonmn.cpp 754 2006-05-05 17:38:12Z ahuggel $");
 
 // *****************************************************************************
 // included header files
@@ -49,30 +49,6 @@ EXIV2_RCSID("@(#) $Id: canonmn.cpp 569 2005-05-28 05:48:43Z ahuggel $");
 #include <cstring>
 #include <cmath>
 
-// Define DEBUG_MAKERNOTE to output debug information to std::cerr
-#undef DEBUG_MAKERNOTE
-
-// *****************************************************************************
-// local declarations
-namespace {
-    /* 
-       @brief Convert Canon hex-based EV (modulo 0x20) to real number
-              Ported from Phil Harvey's Image::ExifTool::Canon::CanonEv 
-              by Will Stokes
-
-       0x00 -> 0
-       0x0c -> 0.33333
-       0x10 -> 0.5
-       0x14 -> 0.66666
-       0x20 -> 1  
-       ..
-       160 -> 5
-       128 -> 4
-       143 -> 4.46875
-     */
-    float canonEv(long val);
-}
-
 // *****************************************************************************
 // class member definitions
 namespace Exiv2 {
@@ -80,8 +56,8 @@ namespace Exiv2 {
     //! @cond IGNORE
     CanonMakerNote::RegisterMn::RegisterMn()
     {
-        MakerNoteFactory::registerMakerNote("Canon", "*", createCanonMakerNote); 
-        
+        MakerNoteFactory::registerMakerNote("Canon", "*", createCanonMakerNote);
+
         MakerNoteFactory::registerMakerNote(
             canonIfdId, MakerNote::AutoPtr(new CanonMakerNote));
         MakerNoteFactory::registerMakerNote(
@@ -90,7 +66,7 @@ namespace Exiv2 {
             canonCs2IfdId, MakerNote::AutoPtr(new CanonMakerNote));
         MakerNoteFactory::registerMakerNote(
             canonCfIfdId, MakerNote::AutoPtr(new CanonMakerNote));
-        
+
         ExifTags::registerMakerTagInfo(canonIfdId, tagInfo_);
         ExifTags::registerMakerTagInfo(canonCs1IfdId, tagInfoCs1_);
         ExifTags::registerMakerTagInfo(canonCs2IfdId, tagInfoCs2_);
@@ -100,126 +76,132 @@ namespace Exiv2 {
 
     // Canon MakerNote Tag Info
     const TagInfo CanonMakerNote::tagInfo_[] = {
-        TagInfo(0x0000, "0x0000", "Unknown", canonIfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0001, "CameraSettings1", "Various camera settings (1)", canonIfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0002, "0x0002", "Unknown", canonIfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0003, "0x0003", "Unknown", canonIfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0004, "CameraSettings2", "Various camera settings (2)", canonIfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0006, "ImageType", "Image type", canonIfdId, makerTags, asciiString, printValue),
-        TagInfo(0x0007, "FirmwareVersion", "Firmware version", canonIfdId, makerTags, asciiString, printValue),
-        TagInfo(0x0008, "ImageNumber", "Image number", canonIfdId, makerTags, unsignedLong, print0x0008),
-        TagInfo(0x0009, "OwnerName", "Owner Name", canonIfdId, makerTags, asciiString, printValue),
-        TagInfo(0x000c, "SerialNumber", "Camera serial number", canonIfdId, makerTags, unsignedLong, print0x000c),
-        TagInfo(0x000d, "0x000d", "Unknown", canonIfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x000f, "CustomFunctions", "Custom Functions", canonIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0000, "0x0000", "0x0000", "Unknown", canonIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0001, "CameraSettings1", "CameraSettings1", "Various camera settings (1)", canonIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0002, "0x0002", "0x0002", "Unknown", canonIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0003, "0x0003", "0x0003", "Unknown", canonIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0004, "CameraSettings2", "CameraSettings2", "Various camera settings (2)", canonIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0006, "ImageType", "ImageType", "Image type", canonIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x0007, "FirmwareVersion", "Firmware Version", "Firmware version", canonIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x0008, "ImageNumber", "ImageNumber", "Image number", canonIfdId, makerTags, unsignedLong, print0x0008),
+        TagInfo(0x0009, "OwnerName", "OwnerName", "Owner Name", canonIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x000c, "SerialNumber", "SerialNumber", "Camera serial number", canonIfdId, makerTags, unsignedLong, print0x000c),
+        TagInfo(0x000d, "0x000d", "0x000d", "Unknown", canonIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x000f, "CustomFunctions", "CustomFunctions", "Custom Functions", canonIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0012, "PictureInfo", "PictureInfo", "Picture info", canonIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x00a9, "WhiteBalanceTable", "WhiteBalanceTable", "White balance table", canonIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x00b5, "0x00b5", "0x00b5", "Unknown", canonIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x00c0, "0x00c0", "0x00c0", "Unknown", canonIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x00c1, "0x00c1", "0x00c1", "Unknown", canonIfdId, makerTags, unsignedShort, printValue),
         // End of list marker
-        TagInfo(0xffff, "(UnknownCanonMakerNoteTag)", "Unknown CanonMakerNote tag", canonIfdId, makerTags, invalidTypeId, printValue)
+        TagInfo(0xffff, "(UnknownCanonMakerNoteTag)", "(UnknownCanonMakerNoteTag)", "Unknown CanonMakerNote tag", canonIfdId, makerTags, invalidTypeId, printValue)
     };
 
     // Canon Camera Settings 1 Tag Info
     const TagInfo CanonMakerNote::tagInfoCs1_[] = {
-        TagInfo(0x0001, "Macro", "Macro mode", canonCs1IfdId, makerTags, unsignedShort, printCs10x0001),
-        TagInfo(0x0002, "Selftimer", "Self timer", canonCs1IfdId, makerTags, unsignedShort, printCs10x0002),
-        TagInfo(0x0003, "Quality", "Quality", canonCs1IfdId, makerTags, unsignedShort, printCs10x0003),
-        TagInfo(0x0004, "FlashMode", "Flash mode setting", canonCs1IfdId, makerTags, unsignedShort, printCs10x0004),
-        TagInfo(0x0005, "DriveMode", "Drive mode setting", canonCs1IfdId, makerTags, unsignedShort, printCs10x0005),
-        TagInfo(0x0006, "0x0006", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0007, "FocusMode", "Focus mode setting", canonCs1IfdId, makerTags, unsignedShort, printCs10x0007),
-        TagInfo(0x0008, "0x0008", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0009, "0x0009", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x000a, "ImageSize", "Image size", canonCs1IfdId, makerTags, unsignedShort, printCs10x000a),
-        TagInfo(0x000b, "EasyMode", "Easy shooting mode", canonCs1IfdId, makerTags, unsignedShort, printCs10x000b),
-        TagInfo(0x000c, "DigitalZoom", "Digital zoom", canonCs1IfdId, makerTags, unsignedShort, printCs10x000c),
-        TagInfo(0x000d, "Contrast", "Contrast setting", canonCs1IfdId, makerTags, unsignedShort, printCs1Lnh),
-        TagInfo(0x000e, "Saturation", "Saturation setting", canonCs1IfdId, makerTags, unsignedShort, printCs1Lnh),
-        TagInfo(0x000f, "Sharpness", "Sharpness setting", canonCs1IfdId, makerTags, unsignedShort, printCs1Lnh),
-        TagInfo(0x0010, "ISOSpeed", "ISO speed setting", canonCs1IfdId, makerTags, unsignedShort, printCs10x0010),
-        TagInfo(0x0011, "MeteringMode", "Metering mode setting", canonCs1IfdId, makerTags, unsignedShort, printCs10x0011),
-        TagInfo(0x0012, "FocusType", "Focus type setting", canonCs1IfdId, makerTags, unsignedShort, printCs10x0012),
-        TagInfo(0x0013, "AFPoint", "AF point selected", canonCs1IfdId, makerTags, unsignedShort, printCs10x0013),
-        TagInfo(0x0014, "ExposureProgram", "Exposure mode setting", canonCs1IfdId, makerTags, unsignedShort, printCs10x0014),
-        TagInfo(0x0015, "0x0015", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0016, "0x0016", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0017, "Lens", "'long' and 'short' focal length of lens (in 'focal units') and 'focal units' per mm", canonCs1IfdId, makerTags, unsignedShort, printCs1Lens),
-        TagInfo(0x0018, "0x0018", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0019, "0x0019", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x001a, "0x001a", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x001b, "0x001b", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x001c, "FlashActivity", "Flash activity", canonCs1IfdId, makerTags, unsignedShort, printCs10x001c),
-        TagInfo(0x001d, "FlashDetails", "Flash details", canonCs1IfdId, makerTags, unsignedShort, printCs10x001d),
-        TagInfo(0x001e, "0x001e", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x001f, "0x001f", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0020, "FocusMode", "Focus mode setting", canonCs1IfdId, makerTags, unsignedShort, printCs10x0020),
-        TagInfo(0x0021, "0x0021", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0022, "0x0022", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0023, "0x0023", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0024, "0x0024", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0025, "0x0025", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0026, "0x0026", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0027, "0x0027", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0001, "Macro", "Macro", "Macro mode", canonCs1IfdId, makerTags, unsignedShort, printCs10x0001),
+        TagInfo(0x0002, "Selftimer", "Selftimer", "Self timer", canonCs1IfdId, makerTags, unsignedShort, printCs10x0002),
+        TagInfo(0x0003, "Quality", "Quality", "Quality", canonCs1IfdId, makerTags, unsignedShort, printCs10x0003),
+        TagInfo(0x0004, "FlashMode", "Flash Mode", "Flash mode setting", canonCs1IfdId, makerTags, unsignedShort, printCs10x0004),
+        TagInfo(0x0005, "DriveMode", "Drive Mode", "Drive mode setting", canonCs1IfdId, makerTags, unsignedShort, printCs10x0005),
+        TagInfo(0x0006, "0x0006", "0x0006", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0007, "FocusMode", "Focus Mode", "Focus mode setting", canonCs1IfdId, makerTags, unsignedShort, printCs10x0007),
+        TagInfo(0x0008, "0x0008", "0x0008", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0009, "0x0009", "0x0009", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x000a, "ImageSize", "Image Size", "Image size", canonCs1IfdId, makerTags, unsignedShort, printCs10x000a),
+        TagInfo(0x000b, "EasyMode", "Easy Mode", "Easy shooting mode", canonCs1IfdId, makerTags, unsignedShort, printCs10x000b),
+        TagInfo(0x000c, "DigitalZoom", "Digital Zoom", "Digital zoom", canonCs1IfdId, makerTags, unsignedShort, printCs10x000c),
+        TagInfo(0x000d, "Contrast", "Contrast", "Contrast setting", canonCs1IfdId, makerTags, unsignedShort, printCs1Lnh),
+        TagInfo(0x000e, "Saturation", "Saturation", "Saturation setting", canonCs1IfdId, makerTags, unsignedShort, printCs1Lnh),
+        TagInfo(0x000f, "Sharpness", "Sharpness", "Sharpness setting", canonCs1IfdId, makerTags, unsignedShort, printCs1Lnh),
+        TagInfo(0x0010, "ISOSpeed", "ISO Speed Mode", "ISO speed setting", canonCs1IfdId, makerTags, unsignedShort, printCs10x0010),
+        TagInfo(0x0011, "MeteringMode", "Metering Mode", "Metering mode setting", canonCs1IfdId, makerTags, unsignedShort, printCs10x0011),
+        TagInfo(0x0012, "FocusType", "Focus Type", "Focus type setting", canonCs1IfdId, makerTags, unsignedShort, printCs10x0012),
+        TagInfo(0x0013, "AFPoint", "AFPoint", "AF point selected", canonCs1IfdId, makerTags, unsignedShort, printCs10x0013),
+        TagInfo(0x0014, "ExposureProgram", "Exposure Program", "Exposure mode setting", canonCs1IfdId, makerTags, unsignedShort, printCs10x0014),
+        TagInfo(0x0015, "0x0015", "0x0015", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0016, "0x0016", "0x0016", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0017, "Lens", "Lens", "'long' and 'short' focal length of lens (in 'focal units') and 'focal units' per mm", canonCs1IfdId, makerTags, unsignedShort, printCs1Lens),
+        TagInfo(0x0018, "0x0018", "0x0018", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0019, "0x0019", "0x0019", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x001a, "0x001a", "0x001a", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x001b, "0x001b", "0x001b", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x001c, "FlashActivity", "FlashActivity", "Flash activity", canonCs1IfdId, makerTags, unsignedShort, printCs10x001c),
+        TagInfo(0x001d, "FlashDetails", "FlashDetails", "Flash details", canonCs1IfdId, makerTags, unsignedShort, printCs10x001d),
+        TagInfo(0x001e, "0x001e", "0x001e", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x001f, "0x001f", "0x001f", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0020, "FocusMode", "FocusMode", "Focus mode setting", canonCs1IfdId, makerTags, unsignedShort, printCs10x0020),
+        TagInfo(0x0021, "0x0021", "0x0021", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0022, "0x0022", "0x0022", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0023, "0x0023", "0x0023", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0024, "0x0024", "0x0024", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0025, "0x0025", "0x0025", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0026, "0x0026", "0x0026", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0027, "0x0027", "0x0027", "Unknown", canonCs1IfdId, makerTags, unsignedShort, printValue),
         // End of list marker
-        TagInfo(0xffff, "(UnknownCanonCs1Tag)", "Unknown Canon Camera Settings 1 tag", canonCs1IfdId, makerTags, invalidTypeId, printValue)
+        TagInfo(0xffff, "(UnknownCanonCs1Tag)", "(UnknownCanonCs1Tag)", "Unknown Canon Camera Settings 1 tag", canonCs1IfdId, makerTags, invalidTypeId, printValue)
     };
 
     // Canon Camera Settings 2 Tag Info
     const TagInfo CanonMakerNote::tagInfoCs2_[] = {
-        TagInfo(0x0001, "0x0001", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0002, "ISOSpeed", "ISO speed used", canonCs2IfdId, makerTags, unsignedShort, printCs20x0002),
-        TagInfo(0x0003, "0x0003", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0004, "0x0004", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0005, "0x0005", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0006, "0x0006", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0007, "WhiteBalance", "White balance setting", canonCs2IfdId, makerTags, unsignedShort, printCs20x0007),
-        TagInfo(0x0008, "0x0008", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0009, "Sequence", "Sequence number (if in a continuous burst)", canonCs2IfdId, makerTags, unsignedShort, printCs20x0009),
-        TagInfo(0x000a, "0x000a", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x000b, "0x000b", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x000c, "0x000c", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x000d, "0x000d", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x000e, "AFPointUsed", "AF point used", canonCs2IfdId, makerTags, unsignedShort, printCs20x000e),
-        TagInfo(0x000f, "FlashBias", "Flash bias", canonCs2IfdId, makerTags, unsignedShort, printCs20x000f),
-        TagInfo(0x0010, "0x0010", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0011, "0x0011", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0012, "0x0012", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0013, "SubjectDistance", "Subject distance (units are not clear)", canonCs2IfdId, makerTags, unsignedShort, printCs20x0013),
-        TagInfo(0x0014, "0x0014", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0015, "0x0015", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0016, "0x0016", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0017, "0x0017", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0018, "0x0018", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0019, "0x0019", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x001a, "0x001a", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0001, "0x0001", "0x0001", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0002, "ISOSpeed", "ISO Speed Used", "ISO speed used", canonCs2IfdId, makerTags, unsignedShort, printCs20x0002),
+        TagInfo(0x0003, "0x0003", "0x0003", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0004, "TargetAperture", "TargetAperture", "Target Aperture", canonCs2IfdId, makerTags, unsignedShort, printCs20x0015),
+        TagInfo(0x0005, "TargetShutterSpeed", "TargetShutterSpeed", "Target shutter speed", canonCs2IfdId, makerTags, unsignedShort, printCs20x0016),
+        TagInfo(0x0006, "0x0006", "0x0006", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0007, "WhiteBalance", "WhiteBalance", "White balance setting", canonCs2IfdId, makerTags, unsignedShort, printCs20x0007),
+        TagInfo(0x0008, "0x0008", "0x0008", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0009, "Sequence", "Sequence", "Sequence number (if in a continuous burst)", canonCs2IfdId, makerTags, unsignedShort, printCs20x0009),
+        TagInfo(0x000a, "0x000a", "0x000a", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x000b, "0x000b", "0x000b", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x000c, "0x000c", "0x000c", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x000d, "0x000d", "0x000d", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x000e, "AFPointUsed", "AFPointUsed", "AF point used", canonCs2IfdId, makerTags, unsignedShort, printCs20x000e),
+        TagInfo(0x000f, "FlashBias", "FlashBias", "Flash bias", canonCs2IfdId, makerTags, unsignedShort, printCs20x000f),
+        TagInfo(0x0010, "0x0010", "0x0010", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0011, "0x0011", "0x0011", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0012, "0x0012", "0x0012", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0013, "SubjectDistance", "SubjectDistance", "Subject distance (units are not clear)", canonCs2IfdId, makerTags, unsignedShort, printCs20x0013),
+        TagInfo(0x0014, "0x0014", "0x0014", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0015, "ApertureValue", "ApertureValue", "Aperture", canonCs2IfdId, makerTags, unsignedShort, printCs20x0015),
+        TagInfo(0x0016, "ShutterSpeedValue", "ShutterSpeedValue", "Shutter speed", canonCs2IfdId, makerTags, unsignedShort, printCs20x0016),
+        TagInfo(0x0017, "0x0017", "0x0017", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0018, "0x0018", "0x0018", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0019, "0x0019", "0x0019", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x001a, "0x001a", "0x001a", "Unknown", canonCs2IfdId, makerTags, unsignedShort, printValue),
         // End of list marker
-        TagInfo(0xffff, "(UnknownCanonCs2Tag)", "Unknown Canon Camera Settings 2 tag", canonCs2IfdId, makerTags, invalidTypeId, printValue)
+        TagInfo(0xffff, "(UnknownCanonCs2Tag)", "(UnknownCanonCs2Tag)", "Unknown Canon Camera Settings 2 tag", canonCs2IfdId, makerTags, invalidTypeId, printValue)
     };
 
     // Canon Custom Function Tag Info
     const TagInfo CanonMakerNote::tagInfoCf_[] = {
-        TagInfo(0x0001, "NoiseReduction", "Long exposure noise reduction", canonCfIfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0002, "ShutterAeLock", "Shutter/AE lock buttons", canonCfIfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0003, "MirrorLockup", "Mirror lockup", canonCfIfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0004, "ExposureLevelIncrements", "Tv/Av and exposure level", canonCfIfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0005, "AFAssist", "AF assist light", canonCfIfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0006, "FlashSyncSpeedAv", "Shutter speed in Av mode", canonCfIfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0007, "AEBSequence", "AEB sequence/auto cancellation", canonCfIfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0008, "ShutterCurtainSync", "Shutter curtain sync", canonCfIfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x0009, "LensAFStopButton", "Lens AF stop button Fn. Switch", canonCfIfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x000a, "FillFlashAutoReduction", "Auto reduction of fill flash", canonCfIfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x000b, "MenuButtonReturn", "Menu button return position", canonCfIfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x000c, "SetButtonFunction", "SET button func. when shooting", canonCfIfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x000d, "SensorCleaning", "Sensor cleaning", canonCfIfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x000e, "SuperimposedDisplay", "Superimposed display", canonCfIfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x000f, "ShutterReleaseNoCFCard", "Shutter Release W/O CF Card", canonCfIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0001, "NoiseReduction", "NoiseReduction", "Long exposure noise reduction", canonCfIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0002, "ShutterAeLock", "ShutterAeLock", "Shutter/AE lock buttons", canonCfIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0003, "MirrorLockup", "MirrorLockup", "Mirror lockup", canonCfIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0004, "ExposureLevelIncrements", "ExposureLevelIncrements", "Tv/Av and exposure level", canonCfIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0005, "AFAssist", "AFAssist", "AF assist light", canonCfIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0006, "FlashSyncSpeedAv", "FlashSyncSpeedAv", "Shutter speed in Av mode", canonCfIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0007, "AEBSequence", "AEBSequence", "AEB sequence/auto cancellation", canonCfIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0008, "ShutterCurtainSync", "ShutterCurtainSync", "Shutter curtain sync", canonCfIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x0009, "LensAFStopButton", "LensAFStopButton", "Lens AF stop button Fn. Switch", canonCfIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x000a, "FillFlashAutoReduction", "FillFlashAutoReduction", "Auto reduction of fill flash", canonCfIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x000b, "MenuButtonReturn", "MenuButtonReturn", "Menu button return position", canonCfIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x000c, "SetButtonFunction", "SetButtonFunction", "SET button func. when shooting", canonCfIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x000d, "SensorCleaning", "SensorCleaning", "Sensor cleaning", canonCfIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x000e, "SuperimposedDisplay", "SuperimposedDisplay", "Superimposed display", canonCfIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x000f, "ShutterReleaseNoCFCard", "ShutterReleaseNoCFCard", "Shutter Release W/O CF Card", canonCfIfdId, makerTags, unsignedShort, printValue),
         // End of list marker
-        TagInfo(0xffff, "(UnknownCanonCfTag)", "Unknown Canon Custom Function tag", canonCfIfdId, makerTags, invalidTypeId, printValue)
+        TagInfo(0xffff, "(UnknownCanonCfTag)", "(UnknownCanonCfTag)", "Unknown Canon Custom Function tag", canonCfIfdId, makerTags, invalidTypeId, printValue)
     };
 
     int CanonMakerNote::read(const byte* buf,
-                             long len, 
-                             ByteOrder byteOrder, 
-                             long offset)
+                             long len,
+                             long start,
+                             ByteOrder byteOrder,
+                             long shift)
     {
-        int rc = IfdMakerNote::read(buf, len, byteOrder, offset);
+        int rc = IfdMakerNote::read(buf, len, start, byteOrder, shift);
         if (rc) return rc;
 
         // Decode camera settings 1 and add settings as additional entries
@@ -228,7 +210,7 @@ namespace Exiv2 {
             for (uint16_t c = 1; cs->count() > c; ++c) {
                 if (c == 23 && cs->count() > 25) {
                     // Pack related lens info into one tag
-                    addCsEntry(canonCs1IfdId, c, cs->offset() + c*2, 
+                    addCsEntry(canonCs1IfdId, c, cs->offset() + c*2,
                                cs->data() + c*2, 3);
                     c += 2;
                 }
@@ -276,9 +258,9 @@ namespace Exiv2 {
         return 0;
     }
 
-    void CanonMakerNote::addCsEntry(IfdId ifdId, 
-                                    uint16_t tag, 
-                                    long offset, 
+    void CanonMakerNote::addCsEntry(IfdId ifdId,
+                                    uint16_t tag,
+                                    long offset,
                                     const byte* data,
                                     int count)
     {
@@ -293,7 +275,7 @@ namespace Exiv2 {
     void CanonMakerNote::add(const Entry& entry)
     {
         assert(alloc_ == entry.alloc());
-        assert(   entry.ifdId() == canonIfdId 
+        assert(   entry.ifdId() == canonIfdId
                || entry.ifdId() == canonCs1IfdId
                || entry.ifdId() == canonCs2IfdId
                || entry.ifdId() == canonCfIfdId);
@@ -305,7 +287,7 @@ namespace Exiv2 {
     {
         if (byteOrder_ == invalidByteOrder) byteOrder_ = byteOrder;
 
-        assert(ifd_.alloc()); 
+        assert(ifd_.alloc());
         ifd_.clear();
 
         // Add all standard Canon entries to the IFD
@@ -340,7 +322,7 @@ namespace Exiv2 {
     void CanonMakerNote::updateBase(byte* pNewBase)
     {
         byte* pBase = ifd_.updateBase(pNewBase);
-        if (absOffset_ && !alloc_) {
+        if (absShift_ && !alloc_) {
             Entries::iterator end = entries_.end();
             for (Entries::iterator pos = entries_.begin(); pos != end; ++pos) {
                 pos->updateBase(pBase, pNewBase);
@@ -381,7 +363,7 @@ namespace Exiv2 {
         return headerSize() + ifd.size() + ifd.dataSize();
     } // CanonMakerNote::size
 
-    long CanonMakerNote::assemble(Entry& e, 
+    long CanonMakerNote::assemble(Entry& e,
                                   IfdId ifdId,
                                   uint16_t tag,
                                   ByteOrder byteOrder) const
@@ -413,7 +395,7 @@ namespace Exiv2 {
         return len;
     } // CanonMakerNote::assemble
 
-    Entries::const_iterator CanonMakerNote::findIdx(int idx) const 
+    Entries::const_iterator CanonMakerNote::findIdx(int idx) const
     {
         return std::find_if(entries_.begin(), entries_.end(),
                             FindEntryByIdx(idx));
@@ -435,9 +417,9 @@ namespace Exiv2 {
         return AutoPtr(create_(alloc));
     }
 
-    CanonMakerNote* CanonMakerNote::create_(bool alloc) const 
+    CanonMakerNote* CanonMakerNote::create_(bool alloc) const
     {
-        return new CanonMakerNote(alloc); 
+        return new CanonMakerNote(alloc);
     }
 
     CanonMakerNote::AutoPtr CanonMakerNote::clone() const
@@ -445,16 +427,17 @@ namespace Exiv2 {
         return AutoPtr(clone_());
     }
 
-    CanonMakerNote* CanonMakerNote::clone_() const 
+    CanonMakerNote* CanonMakerNote::clone_() const
     {
-        return new CanonMakerNote(*this); 
+        return new CanonMakerNote(*this);
     }
 
     std::ostream& CanonMakerNote::print0x0008(std::ostream& os,
                                               const Value& value)
     {
         std::string n = value.toString();
-        return os << n.substr(0, n.length() - 4) << "-" 
+        if (n.length() < 4) return os << "(" << n << ")";
+        return os << n.substr(0, n.length() - 4) << "-"
                   << n.substr(n.length() - 4);
     }
 
@@ -464,13 +447,13 @@ namespace Exiv2 {
         std::istringstream is(value.toString());
         uint32_t l;
         is >> l;
-        return os << std::setw(4) << std::setfill('0') << std::hex 
+        return os << std::setw(4) << std::setfill('0') << std::hex
                   << ((l & 0xffff0000) >> 16)
                   << std::setw(5) << std::setfill('0') << std::dec
                   << (l & 0x0000ffff);
     }
 
-    std::ostream& CanonMakerNote::printCs10x0001(std::ostream& os, 
+    std::ostream& CanonMakerNote::printCs10x0001(std::ostream& os,
                                                  const Value& value)
     {
         if (value.typeId() != unsignedShort) return os << value;
@@ -491,7 +474,7 @@ namespace Exiv2 {
         if (l == 0) {
             os << "Off";
         }
-        else { 
+        else {
             os << l / 10.0 << " s";
         }
         return os;
@@ -505,6 +488,7 @@ namespace Exiv2 {
         switch (l) {
         case 2: os << "Normal"; break;
         case 3: os << "Fine"; break;
+        case 4: os << "Raw"; break;
         case 5: os << "Superfine"; break;
         default: os << "(" << l << ")"; break;
         }
@@ -533,7 +517,7 @@ namespace Exiv2 {
     std::ostream& CanonMakerNote::printCs10x0005(std::ostream& os,
                                                  const Value& value)
     {
-        if (value.typeId() != unsignedShort) return os << value; 
+        if (value.typeId() != unsignedShort) return os << value;
         long l = value.toLong();
         switch (l) {
         case 0: os << "Single / timer"; break;
@@ -586,13 +570,21 @@ namespace Exiv2 {
         case  2: os << "Landscape"; break;
         case  3: os << "Fast shutter"; break;
         case  4: os << "Slow shutter"; break;
-        case  5: os << "Night"; break;
+        case  5: os << "Night Scene"; break;
         case  6: os << "B&W"; break;
         case  7: os << "Sepia"; break;
         case  8: os << "Portrait"; break;
         case  9: os << "Sports"; break;
         case 10: os << "Macro / close-up"; break;
         case 11: os << "Pan focus"; break;
+        case 18: os << "Foliage"; break;
+        case 19: os << "Indoor"; break;
+        case 20: os << "Fireworks"; break;
+        case 21: os << "Beach"; break;
+        case 23: os << "Snow"; break;
+        case 25: os << "Night SnapShot"; break;
+        case 27: os << "My Colors"; break;
+        case 28: os << "Movie"; break;
         default: os << "(" << l << ")"; break;
         }
         return os;
@@ -694,11 +686,11 @@ namespace Exiv2 {
         if (value.typeId() != unsignedShort) return os << value;
         long l = value.toLong();
         switch (l) {
-        case 0: os << "Easy shooting"; break;
-        case 1: os << "Program"; break;
-        case 2: os << "Shutter priority"; break;
-        case 3: os << "Aperture priority"; break;
-        case 4: os << "Manual"; break;
+        case 0: os << "Easy shooting (Auto)"; break;
+        case 1: os << "Program (P)"; break;
+        case 2: os << "Shutter priority (Tv)"; break;
+        case 3: os << "Aperture priority (Av)"; break;
+        case 4: os << "Manual (M)"; break;
         case 5: os << "A-DEP"; break;
         default: os << "(" << l << ")"; break;
         }
@@ -772,6 +764,7 @@ namespace Exiv2 {
         if (value.count() < 3) return os << value;
 
         float fu = value.toFloat(2);
+        if (fu == 0.0) return os << value;
         float len1 = value.toLong(0) / fu;
         float len2 = value.toLong(1) / fu;
         std::ostringstream oss;
@@ -892,24 +885,45 @@ namespace Exiv2 {
         return os;
     }
 
+    std::ostream& CanonMakerNote::printCs20x0015(std::ostream& os,
+                                                 const Value& value)
+    {
+        if (value.typeId() != unsignedShort) return os << value;
+
+        std::ostringstream oss;
+        oss.copyfmt(os);
+        os << std::setprecision(2)
+           << "F" << fnumber(canonEv(value.toLong()));
+        os.copyfmt(oss);
+
+        return os;
+    }
+
+    std::ostream& CanonMakerNote::printCs20x0016(std::ostream& os,
+                                                 const Value& value)
+    {
+        if (value.typeId() != unsignedShort) return os << value;
+
+        URational ur = exposureTime(canonEv(value.toLong()));
+        os << ur.first;
+        if (ur.second > 1) {
+            os << "/" << ur.second;
+        }
+        return os << " s";
+    }
+
 // *****************************************************************************
 // free functions
 
     MakerNote::AutoPtr createCanonMakerNote(bool alloc,
-                                            const byte* buf, 
-                                            long len, 
-                                            ByteOrder byteOrder, 
+                                            const byte* buf,
+                                            long len,
+                                            ByteOrder byteOrder,
                                             long offset)
     {
         return MakerNote::AutoPtr(new CanonMakerNote(alloc));
     }
 
-}                                       // namespace Exiv2
-
-// *****************************************************************************
-// local definitions
-namespace {
-
     float canonEv(long val)
     {
         // temporarily remove sign
@@ -931,4 +945,4 @@ namespace {
         return sign * (val + frac) / 32.0f;
     }
 
-}
+}                                       // namespace Exiv2
