@@ -1,36 +1,36 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
- * 
+ * Copyright (C) 2004, 2005, 2006 Andreas Huggel <ahuggel@gmx.net>
+ *
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA.
  */
 /*
   File:      exif.cpp
-  Version:   $Rev: 569 $
+  Version:   $Rev: 808 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   26-Jan-04, ahu: created
              11-Feb-04, ahu: isolated as a component
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: exif.cpp 569 2005-05-28 05:48:43Z ahuggel $");
+EXIV2_RCSID("@(#) $Id: exif.cpp 808 2006-06-01 15:09:39Z ahuggel $");
 
-// Define DEBUG_MAKERNOTE to output debug information to std::cerr, e.g, by 
-// calling make like this: make DEFS=-DDEBUG_MAKERNOTE exif.o 
+// Define DEBUG_MAKERNOTE to output debug information to std::cerr, e.g, by
+// calling make like this: make DEFS=-DDEBUG_MAKERNOTE exif.o
 //#define DEBUG_MAKERNOTE
 
 // *****************************************************************************
@@ -48,7 +48,7 @@ EXIV2_RCSID("@(#) $Id: exif.cpp 569 2005-05-28 05:48:43Z ahuggel $");
 #include "value.hpp"
 #include "ifd.hpp"
 #include "tags.hpp"
-#include "jpgimage.hpp"
+#include "image.hpp"
 #include "makernote.hpp"
 #include "futils.hpp"
 
@@ -79,7 +79,7 @@ namespace {
     void setOffsetTag(Exiv2::Ifd& ifd,
                       int idx,
                       uint16_t tag,
-                      uint32_t offset, 
+                      uint32_t offset,
                       Exiv2::ByteOrder byteOrder);
 
     // Read file path into a DataBuf, which is returned.
@@ -94,10 +94,10 @@ namespace Exiv2 {
     Exifdatum::Exifdatum(const Entry& e, ByteOrder byteOrder)
         : key_(ExifKey::AutoPtr(new ExifKey(e)))
     {
-        setValue(e, byteOrder);
+        setValue(e, e.byteOrder() == invalidByteOrder ? byteOrder : e.byteOrder());
     }
 
-    Exifdatum::Exifdatum(const ExifKey& key, const Value* pValue) 
+    Exifdatum::Exifdatum(const ExifKey& key, const Value* pValue)
         : key_(key.clone())
     {
         if (pValue) value_ = pValue->clone();
@@ -114,6 +114,12 @@ namespace Exiv2 {
         if (rhs.value_.get() != 0) value_ = rhs.value_->clone(); // deep copy
     }
 
+    const Value& Exifdatum::value() const
+    {
+        if (value_.get() == 0) throw Error(8);
+        return *value_;
+    }
+
     Exifdatum& Exifdatum::operator=(const Exifdatum& rhs)
     {
         if (this == &rhs) return *this;
@@ -127,41 +133,41 @@ namespace Exiv2 {
 
         return *this;
     } // Exifdatum::operator=
-    
+
     Exifdatum& Exifdatum::operator=(const std::string& value)
-    { 
-        setValue(value); 
-        return *this; 
+    {
+        setValue(value);
+        return *this;
     }
 
-    Exifdatum& Exifdatum::operator=(const uint16_t& value) 
+    Exifdatum& Exifdatum::operator=(const uint16_t& value)
     {
-        return Exiv2::setValue(*this, value); 
+        return Exiv2::setValue(*this, value);
     }
 
     Exifdatum& Exifdatum::operator=(const uint32_t& value)
     {
-        return Exiv2::setValue(*this, value); 
+        return Exiv2::setValue(*this, value);
     }
 
     Exifdatum& Exifdatum::operator=(const URational& value)
     {
-        return Exiv2::setValue(*this, value); 
+        return Exiv2::setValue(*this, value);
     }
 
     Exifdatum& Exifdatum::operator=(const int16_t& value)
     {
-        return Exiv2::setValue(*this, value); 
+        return Exiv2::setValue(*this, value);
     }
 
     Exifdatum& Exifdatum::operator=(const int32_t& value)
     {
-        return Exiv2::setValue(*this, value); 
+        return Exiv2::setValue(*this, value);
     }
 
     Exifdatum& Exifdatum::operator=(const Rational& value)
     {
-        return Exiv2::setValue(*this, value); 
+        return Exiv2::setValue(*this, value);
     }
 
     Exifdatum& Exifdatum::operator=(const Value& value)
@@ -178,7 +184,12 @@ namespace Exiv2 {
 
     void Exifdatum::setValue(const Entry& e, ByteOrder byteOrder)
     {
-        value_ = Value::create(TypeId(e.type()));
+        TypeId t = TypeId(e.type());
+        // Hack: On the fly type conversion for Exif.Photo.UserComment
+        if (e.tag() == 0x9286 && e.ifdId() == exifIfdId && e.type() == undefined) {
+            t = comment;
+        }
+        value_ = Value::create(t);
         value_->read(e.data(), e.count() * e.typeSize(), byteOrder);
         value_->setDataArea(e.dataArea(), e.sizeDataArea());
     }
@@ -192,6 +203,11 @@ namespace Exiv2 {
         value_->read(value);
     }
 
+    TiffThumbnail& TiffThumbnail::operator=(const TiffThumbnail& /*rhs*/)
+    {
+        return *this;
+    }
+
     int TiffThumbnail::setDataArea(ExifData& exifData, Ifd* pIfd1,
                                    const byte* buf, long len) const
     {
@@ -207,7 +223,7 @@ namespace Exiv2 {
         }
         DataBuf stripsBuf(totalSize);
 
-        // Copy all strips into the data buffer. For each strip remember its 
+        // Copy all strips into the data buffer. For each strip remember its
         // offset from the start of the data buffer
         ExifData::iterator stripOffsets;
         key = ExifKey("Exif.Thumbnail.StripOffsets");
@@ -232,7 +248,7 @@ namespace Exiv2 {
             currentOffset += size;
         }
 
-        // Set StripOffsets data area and relative offsets 
+        // Set StripOffsets data area and relative offsets
         stripOffsets->setDataArea(stripsBuf.pData_, stripsBuf.size_);
         stripOffsets->setValue(os.str());
 
@@ -244,7 +260,7 @@ namespace Exiv2 {
         }
 
         return 0;
-    } // TiffThumbnail::read
+    } // TiffThumbnail::setDataArea
 
     const char* TiffThumbnail::format() const
     {
@@ -276,6 +292,11 @@ namespace Exiv2 {
         return buf;
     }
 
+    JpegThumbnail& JpegThumbnail::operator=(const JpegThumbnail& /*rhs*/)
+    {
+        return *this;
+    }
+
     int JpegThumbnail::setDataArea(ExifData& exifData, Ifd* pIfd1,
                                    const byte* buf, long len) const
     {
@@ -316,16 +337,16 @@ namespace Exiv2 {
         return format->dataArea();
     }
 
-    ExifData::ExifData() 
-        : pTiffHeader_(0), 
-          pIfd0_(0), pExifIfd_(0), pIopIfd_(0), pGpsIfd_(0), pIfd1_(0), 
+    ExifData::ExifData()
+        : pTiffHeader_(0),
+          pIfd0_(0), pExifIfd_(0), pIopIfd_(0), pGpsIfd_(0), pIfd1_(0),
           pMakerNote_(0), size_(0), pData_(0), compatible_(true)
     {
     }
 
     ExifData::ExifData(const ExifData& rhs)
         : exifMetadata_(rhs.exifMetadata_), pTiffHeader_(0),
-          pIfd0_(0), pExifIfd_(0), pIopIfd_(0), pGpsIfd_(0), pIfd1_(0), 
+          pIfd0_(0), pExifIfd_(0), pIopIfd_(0), pGpsIfd_(0), pIfd1_(0),
           pMakerNote_(0), size_(0), pData_(0), compatible_(rhs.compatible_)
     {
         pData_ = new byte[rhs.size_];
@@ -444,6 +465,8 @@ namespace Exiv2 {
 
     int ExifData::load(const byte* buf, long len)
     {
+        if (!buf || len == 0) return -1;
+
         // Copy the data buffer
         delete[] pData_;
         pData_ = new byte[len];
@@ -459,113 +482,126 @@ namespace Exiv2 {
 
         // Read IFD0
         delete pIfd0_;
-        pIfd0_ = new Ifd(ifd0Id, 0, false); 
+        pIfd0_ = new Ifd(ifd0Id, 0, false);
         assert(pIfd0_ != 0);
-        rc = pIfd0_->read(pData_ + pTiffHeader_->offset(), 
-                          size_ - pTiffHeader_->offset(), 
-                          byteOrder(), 
-                          pTiffHeader_->offset());
-        if (rc) return rc;
+        rc = pIfd0_->read(pData_, size_, pTiffHeader_->offset(), byteOrder());
+        if (rc) return rc; // no point to continue if there is no IFD0
 
         delete pExifIfd_;
-        pExifIfd_ = new Ifd(exifIfdId, 0, false);
-        assert(pExifIfd_ != 0);
+        pExifIfd_ = 0;
+        std::auto_ptr<Ifd> tmpExif(new Ifd(exifIfdId, 0, false));
+        assert(tmpExif.get() != 0);
         // Find and read ExifIFD sub-IFD of IFD0
-        rc = pIfd0_->readSubIfd(*pExifIfd_, pData_, size_, byteOrder(), 0x8769);
-        if (rc) return rc;
-        // Find MakerNote in ExifIFD, create a MakerNote class 
-        Ifd::iterator pos = pExifIfd_->findTag(0x927c);
-        Ifd::iterator make = pIfd0_->findTag(0x010f);
-        Ifd::iterator model = pIfd0_->findTag(0x0110);
-        if (   pos != pExifIfd_->end() 
-            && make != pIfd0_->end() && model != pIfd0_->end()) {
-            // Todo: The conversion to string assumes that there is a \0 at the end
-            // Todo: How to avoid the cast (is that a MSVC thing?)
-            pMakerNote_ = MakerNoteFactory::create(
-                              reinterpret_cast<const char*>(make->data()),
-                              reinterpret_cast<const char*>(model->data()),
-                              false,
-                              pos->data(), 
-                              pos->size(),
-                              byteOrder(),
-                              pExifIfd_->offset() + pos->offset()).release();
+        rc = pIfd0_->readSubIfd(*tmpExif, pData_, size_, byteOrder(), 0x8769);
+        if (0 == rc) {
+            pExifIfd_ = tmpExif.release();
         }
-        // Read the MakerNote
-        if (pMakerNote_) {
-            rc = pMakerNote_->read(pos->data(), 
-                                   pos->size(),
-                                   byteOrder(),
-                                   pExifIfd_->offset() + pos->offset());
-            if (rc) {
-                // Todo: How to handle debug output like this
-                std::cerr << "Warning: Failed to read Makernote, rc = "
-                          << rc << "\n";
-                delete pMakerNote_;
-                pMakerNote_ = 0;
+        if (pExifIfd_) {
+            // Find MakerNote in ExifIFD, create a MakerNote class
+            Ifd::iterator pos = pExifIfd_->findTag(0x927c);
+            Ifd::iterator make = pIfd0_->findTag(0x010f);
+            Ifd::iterator model = pIfd0_->findTag(0x0110);
+            delete pMakerNote_;
+            pMakerNote_ = 0;
+            MakerNote::AutoPtr tmpMakerNote;
+            if (   pos  != pExifIfd_->end()
+                && make != pIfd0_->end() && model != pIfd0_->end()) {
+                // Todo: The conversion to string assumes that there is a \0 at the end
+                // Todo: How to avoid the cast (is that a MSVC thing?)
+                tmpMakerNote = MakerNoteFactory::create(
+                    reinterpret_cast<const char*>(make->data()),
+                    reinterpret_cast<const char*>(model->data()),
+                    false,
+                    pos->data(),
+                    pos->size(),
+                    byteOrder(),
+                    pExifIfd_->offset() + pos->offset());
+            }
+            // Read the MakerNote
+            if (tmpMakerNote.get() != 0) {
+                rc = tmpMakerNote->read(pData_, size_,
+                                        pExifIfd_->offset() + pos->offset(),
+                                        byteOrder());
+                if (0 == rc) {
+                    pMakerNote_ = tmpMakerNote.release();
+                }
+                else {
+#ifndef SUPPRESS_WARNINGS
+                    std::cerr << "Warning: Failed to read Makernote, rc = "
+                              << rc << "\n";
+#endif
+                }
+            }
+            // If we successfully parsed the MakerNote, delete the raw MakerNote,
+            // the parsed MakerNote is the primary MakerNote from now on
+            if (pMakerNote_) {
+                pExifIfd_->erase(pos);
             }
-        }
-        // If we successfully parsed the MakerNote, delete the raw MakerNote,
-        // the parsed MakerNote is the primary MakerNote from now on
-        if (pMakerNote_) {
-            pExifIfd_->erase(pos);
-        }
 
-        delete pIopIfd_;
-        pIopIfd_ = new Ifd(iopIfdId, 0, false);
-        assert(pIopIfd_ != 0);
-        // Find and read Interoperability IFD in ExifIFD
-        rc = pExifIfd_->readSubIfd(*pIopIfd_, pData_, size_, byteOrder(), 0xa005);
-        if (rc) return rc;
+            delete pIopIfd_;
+            pIopIfd_ = 0;
+            std::auto_ptr<Ifd> tmpIop(new Ifd(iopIfdId, 0, false));
+            assert(tmpIop.get() != 0);
+            // Find and read Interoperability IFD in ExifIFD
+            rc = pExifIfd_->readSubIfd(*tmpIop, pData_, size_, byteOrder(), 0xa005);
+            if (0 == rc) {
+                pIopIfd_ = tmpIop.release();
+            }
+        } // if (pExifIfd_)
 
         delete pGpsIfd_;
-        pGpsIfd_ = new Ifd(gpsIfdId, 0, false);
-        assert(pGpsIfd_ != 0);
+        pGpsIfd_ = 0;
+        std::auto_ptr<Ifd> tmpGps(new Ifd(gpsIfdId, 0, false));
+        assert(tmpGps.get() != 0);
         // Find and read GPSInfo sub-IFD in IFD0
-        rc = pIfd0_->readSubIfd(*pGpsIfd_, pData_, size_, byteOrder(), 0x8825);
-        if (rc) return rc;
+        rc = pIfd0_->readSubIfd(*tmpGps, pData_, size_, byteOrder(), 0x8825);
+        if (0 == rc) {
+            pGpsIfd_ = tmpGps.release();
+        }
 
         delete pIfd1_;
-        pIfd1_ = new Ifd(ifd1Id, 0, false);
-        assert(pIfd1_ != 0);
+        pIfd1_ = 0;
+        std::auto_ptr<Ifd> tmpIfd1(new Ifd(ifd1Id, 0, false));
+        assert(tmpIfd1.get() != 0);
         // Read IFD1
         if (pIfd0_->next()) {
-            rc = pIfd1_->read(pData_ + pIfd0_->next(), 
-                              size_ - pIfd0_->next(), 
-                              byteOrder(), 
-                              pIfd0_->next());
-            if (rc) return rc;
+            rc = tmpIfd1->read(pData_, size_, pIfd0_->next(), byteOrder());
+            if (0 == rc) {
+                pIfd1_ = tmpIfd1.release();
+            }
         }
-        // Find and delete ExifIFD sub-IFD of IFD1
-        pos = pIfd1_->findTag(0x8769);
-        if (pos != pIfd1_->end()) {
-            pIfd1_->erase(pos);
-            rc = 7;
-        }
-        // Find and delete GPSInfo sub-IFD in IFD1
-        pos = pIfd1_->findTag(0x8825);
-        if (pos != pIfd1_->end()) {
-            pIfd1_->erase(pos);
-            rc = 7;
+        if (pIfd1_) {
+            // Find and delete ExifIFD sub-IFD of IFD1
+            Ifd::iterator pos = pIfd1_->findTag(0x8769);
+            if (pos != pIfd1_->end()) {
+                pIfd1_->erase(pos);
+                rc = 7;
+            }
+            // Find and delete GPSInfo sub-IFD in IFD1
+            pos = pIfd1_->findTag(0x8825);
+            if (pos != pIfd1_->end()) {
+                pIfd1_->erase(pos);
+                rc = 7;
+            }
         }
         // Copy all entries from the IFDs and the MakerNote to the metadata
         exifMetadata_.clear();
         add(pIfd0_->begin(), pIfd0_->end(), byteOrder());
-        add(pExifIfd_->begin(), pExifIfd_->end(), byteOrder());
+        if (pExifIfd_) add(pExifIfd_->begin(), pExifIfd_->end(), byteOrder());
         if (pMakerNote_) {
-            add(pMakerNote_->begin(), pMakerNote_->end(), 
+            add(pMakerNote_->begin(), pMakerNote_->end(),
                 (pMakerNote_->byteOrder() == invalidByteOrder ?
                     byteOrder() : pMakerNote_->byteOrder()));
         }
-        add(pIopIfd_->begin(), pIopIfd_->end(), byteOrder()); 
-        add(pGpsIfd_->begin(), pGpsIfd_->end(), byteOrder());
-        add(pIfd1_->begin(), pIfd1_->end(), byteOrder());
+        if (pIopIfd_) add(pIopIfd_->begin(), pIopIfd_->end(), byteOrder());
+        if (pGpsIfd_) add(pGpsIfd_->begin(), pGpsIfd_->end(), byteOrder());
+        if (pIfd1_)   add(pIfd1_->begin(),   pIfd1_->end(),   byteOrder());
         // Read the thumbnail (but don't worry whether it was successful or not)
         readThumbnail();
 
         return rc;
     } // ExifData::load
 
-
     DataBuf ExifData::copy()
     {
         DataBuf buf;
@@ -602,8 +638,8 @@ namespace Exiv2 {
         if (pMakerNote_) {
             // Build MakerNote from metadata
             makerNote = pMakerNote_->create();
-            addToMakerNote(makerNote.get(), 
-                           begin(), end(), 
+            addToMakerNote(makerNote.get(),
+                           begin(), end(),
                            (pMakerNote_->byteOrder() == invalidByteOrder ?
                                byteOrder() : pMakerNote_->byteOrder()));
             // Create a placeholder MakerNote entry of the correct size and
@@ -644,21 +680,21 @@ namespace Exiv2 {
         bool addOffsetTag = false;
         long exifIfdOffset = ifd0Offset + ifd0.size() + ifd0.dataSize();
         if (exifIfd.size() > 0 || iopIfd.size() > 0) {
-            exifIfdOffset += 12; 
-            addOffsetTag = true; 
+            exifIfdOffset += 12;
+            addOffsetTag = true;
         }
         if (gpsIfd.size() > 0) {
-            exifIfdOffset += 12; 
-            addOffsetTag = true; 
+            exifIfdOffset += 12;
+            addOffsetTag = true;
         }
         if (ifd0.size() == 0 && addOffsetTag) {
-            exifIfdOffset += 6; 
+            exifIfdOffset += 6;
         }
         addOffsetTag = false;
-        long iopIfdOffset = exifIfdOffset + exifIfd.size() + exifIfd.dataSize(); 
+        long iopIfdOffset = exifIfdOffset + exifIfd.size() + exifIfd.dataSize();
         if (iopIfd.size() > 0) {
             iopIfdOffset += 12;
-            addOffsetTag = true; 
+            addOffsetTag = true;
         }
         if (exifIfd.size() == 0 && addOffsetTag) {
             iopIfdOffset += 6;
@@ -719,7 +755,7 @@ namespace Exiv2 {
         return buf;
     } // ExifData::copyFromMetadata
 
-    void ExifData::add(Entries::const_iterator begin, 
+    void ExifData::add(Entries::const_iterator begin,
                        Entries::const_iterator end,
                        ByteOrder byteOrder)
     {
@@ -739,7 +775,7 @@ namespace Exiv2 {
         if (ExifTags::isMakerIfd(exifdatum.ifdId())) {
             if (pMakerNote_ == 0) {
                 pMakerNote_ = MakerNoteFactory::create(exifdatum.ifdId()).release();
-            }            
+            }
             if (pMakerNote_ == 0) throw Error(23, exifdatum.ifdId());
         }
         // allow duplicates
@@ -794,7 +830,7 @@ namespace Exiv2 {
         (*this)["Exif.Thumbnail.JPEGInterchangeFormatLength"] = uint32_t(size);
     }
 
-    void ExifData::setJpegThumbnail(const byte* buf, long size, 
+    void ExifData::setJpegThumbnail(const byte* buf, long size,
                                     URational xres, URational yres, uint16_t unit)
     {
         setJpegThumbnail(buf, size);
@@ -809,7 +845,7 @@ namespace Exiv2 {
         setJpegThumbnail(thumb.pData_, thumb.size_);
     }
 
-    void ExifData::setJpegThumbnail(const std::string& path, 
+    void ExifData::setJpegThumbnail(const std::string& path,
                                    URational xres, URational yres, uint16_t unit)
     {
         DataBuf thumb = readFile(path); // may throw
@@ -820,8 +856,8 @@ namespace Exiv2 {
     {
         // First, determine if the thumbnail is at the end of the Exif data
         bool stp = stdThumbPosition();
-        // Delete all Exif.Thumbnail.* (IFD1) metadata 
-        ExifMetadata::iterator i = begin(); 
+        // Delete all Exif.Thumbnail.* (IFD1) metadata
+        ExifMetadata::iterator i = begin();
         while (i != end()) {
             if (i->ifdId() == ifd1Id) {
                 i = erase(i);
@@ -851,8 +887,7 @@ namespace Exiv2 {
 
     bool ExifData::stdThumbPosition() const
     {
-        if (   pIfd0_ == 0 || pExifIfd_ == 0 || pIopIfd_ == 0 
-            || pGpsIfd_ == 0 || pIfd1_ == 0) return true;
+        if (pIfd1_ == 0) return true;
 
         // Todo: There is still an invalid assumption here: The data of an IFD
         //       can be stored in multiple non-contiguous blocks. In this case,
@@ -864,20 +899,25 @@ namespace Exiv2 {
         if (thumbnail.get()) {
             long maxOffset;
             maxOffset = std::max(pIfd0_->offset(), pIfd0_->dataOffset());
-            maxOffset = std::max(maxOffset, pExifIfd_->offset());
-            maxOffset = std::max(maxOffset,   pExifIfd_->dataOffset() 
-                                            + pExifIfd_->dataSize());
+            if (pExifIfd_) {
+                maxOffset = std::max(maxOffset, pExifIfd_->offset());
+                maxOffset = std::max(maxOffset,   pExifIfd_->dataOffset()
+                                                + pExifIfd_->dataSize());
+            }
             if (pMakerNote_) {
                 maxOffset = std::max(maxOffset,   pMakerNote_->offset()
                                                 + pMakerNote_->size());
             }
-            maxOffset = std::max(maxOffset, pIopIfd_->offset());
-            maxOffset = std::max(maxOffset,   pIopIfd_->dataOffset()
-                                            + pIopIfd_->dataSize());
-            maxOffset = std::max(maxOffset, pGpsIfd_->offset());
-            maxOffset = std::max(maxOffset,   pGpsIfd_->dataOffset()
-                                            + pGpsIfd_->dataSize());
-
+            if (pIopIfd_) {
+                maxOffset = std::max(maxOffset, pIopIfd_->offset());
+                maxOffset = std::max(maxOffset,   pIopIfd_->dataOffset()
+                                                + pIopIfd_->dataSize());
+            }
+            if (pGpsIfd_) {
+                maxOffset = std::max(maxOffset, pGpsIfd_->offset());
+                maxOffset = std::max(maxOffset,   pGpsIfd_->dataOffset()
+                                                + pGpsIfd_->dataSize());
+            }
             if (   maxOffset > pIfd1_->offset()
                 || maxOffset > pIfd1_->dataOffset() && pIfd1_->dataOffset() > 0)
                 rc = false;
@@ -890,12 +930,12 @@ namespace Exiv2 {
     } // ExifData::stdThumbPosition
 
     ByteOrder ExifData::byteOrder() const
-    { 
+    {
         if (pTiffHeader_) return pTiffHeader_->byteOrder();
         return littleEndian;
     }
 
-    int ExifData::writeThumbnail(const std::string& path) const 
+    int ExifData::writeThumbnail(const std::string& path) const
     {
         Thumbnail::AutoPtr thumbnail = getThumbnail();
         if (thumbnail.get() == 0) return 8;
@@ -928,7 +968,7 @@ namespace Exiv2 {
         return thumbnail->format();
     }
 
-    const char* ExifData::thumbnailExtension() const 
+    const char* ExifData::thumbnailExtension() const
     {
         Thumbnail::AutoPtr thumbnail = getThumbnail();
         if (thumbnail.get() == 0) return "";
@@ -948,6 +988,10 @@ namespace Exiv2 {
                 thumbnail = Thumbnail::AutoPtr(new TiffThumbnail);
             }
         }
+        else {
+            pos = findKey(ExifKey("Exif.Thumbnail.JPEGInterchangeFormat"));
+            if (pos != end()) thumbnail = Thumbnail::AutoPtr(new JpegThumbnail);
+        }
         return thumbnail;
 
     } // ExifData::getThumbnail
@@ -965,27 +1009,29 @@ namespace Exiv2 {
 
     bool ExifData::updateEntries()
     {
-        if (   pIfd0_ == 0 || pExifIfd_ == 0 || pIopIfd_ == 0 
-            || pGpsIfd_ == 0 || pIfd1_ == 0) return false;
+        if (pIfd0_ == 0 || pExifIfd_ == 0 || pIopIfd_ == 0 || pGpsIfd_ == 0) {
+            return false;
+        }
         if (!this->compatible()) return false;
 
         bool compatible = true;
         compatible &= updateRange(pIfd0_->begin(), pIfd0_->end(), byteOrder());
         compatible &= updateRange(pExifIfd_->begin(), pExifIfd_->end(), byteOrder());
         if (pMakerNote_) {
-            compatible &= updateRange(pMakerNote_->begin(), 
-                                      pMakerNote_->end(), 
+            compatible &= updateRange(pMakerNote_->begin(),
+                                      pMakerNote_->end(),
                                       (pMakerNote_->byteOrder() == invalidByteOrder ?
                                           byteOrder() : pMakerNote_->byteOrder()));
         }
         compatible &= updateRange(pIopIfd_->begin(), pIopIfd_->end(), byteOrder());
         compatible &= updateRange(pGpsIfd_->begin(), pGpsIfd_->end(), byteOrder());
-        compatible &= updateRange(pIfd1_->begin(), pIfd1_->end(), byteOrder());
-
+        if (pIfd1_) {
+            compatible &= updateRange(pIfd1_->begin(), pIfd1_->end(), byteOrder());
+        }
         return compatible;
     } // ExifData::updateEntries
 
-    bool ExifData::updateRange(const Entries::iterator& begin, 
+    bool ExifData::updateRange(const Entries::iterator& begin,
                                const Entries::iterator& end,
                                ByteOrder byteOrder)
     {
@@ -1000,7 +1046,7 @@ namespace Exiv2 {
                 continue;
             }
             if (entry->count() == 0 && md->count() == 0) {
-                // Special case: don't do anything if both the entry and 
+                // Special case: don't do anything if both the entry and
                 // Exifdatum have no data. This is to preserve the original
                 // data in the offset field of an IFD entry with count 0,
                 // if the Exifdatum was not changed.
@@ -1018,13 +1064,15 @@ namespace Exiv2 {
                 // header and that is currently only done in intrusive write
                 // mode). On the other hand, it is thus now not possible to
                 // change the offsets of an entry with a data area in
-                // non-intrusive mode. This can be considered a bug. 
+                // non-intrusive mode. This can be considered a bug.
                 // Todo: Fix me!
                 if (md->sizeDataArea() == 0) {
                     DataBuf buf(md->size());
-                    md->copy(buf.pData_, byteOrder);
-                    entry->setValue(static_cast<uint16_t>(md->typeId()), 
-                                    md->count(), 
+                    // Hack: Use byte order from the entry if there is one
+                    md->copy(buf.pData_, entry->byteOrder() == invalidByteOrder
+                             ? byteOrder : entry->byteOrder());
+                    entry->setValue(static_cast<uint16_t>(md->typeId()),
+                                    md->count(),
                                     buf.pData_, md->size());
                 }
                 // Always set the data area
@@ -1050,7 +1098,7 @@ namespace Exiv2 {
             }
             // Make sure that the size of the Exifdatum fits the available size
             // of the entry
-            if (   md->size() > rc.second->size() 
+            if (   md->size() > rc.second->size()
                 || md->sizeDataArea() > rc.second->sizeDataArea()) {
                 compatible = false;
                 break;
@@ -1059,7 +1107,7 @@ namespace Exiv2 {
         return compatible;
     } // ExifData::compatible
 
-    std::pair<bool, Entries::const_iterator> 
+    std::pair<bool, Entries::const_iterator>
     ExifData::findEntry(IfdId ifdId, int idx) const
     {
         Entries::const_iterator entry;
@@ -1088,19 +1136,19 @@ namespace Exiv2 {
     {
         const Ifd* ifd = 0;
         switch (ifdId) {
-        case ifd0Id: 
+        case ifd0Id:
             ifd = pIfd0_;
             break;
-        case exifIfdId: 
+        case exifIfdId:
             ifd = pExifIfd_;
             break;
-        case iopIfdId: 
+        case iopIfdId:
             ifd = pIopIfd_;
             break;
-        case gpsIfdId: 
+        case gpsIfdId:
             ifd = pGpsIfd_;
             break;
-        case ifd1Id: 
+        case ifd1Id:
             ifd = pIfd1_;
             break;
         default:
@@ -1113,9 +1161,18 @@ namespace Exiv2 {
     // *************************************************************************
     // free functions
 
-    void addToIfd(Ifd& ifd, 
-                  ExifMetadata::const_iterator begin, 
-                  ExifMetadata::const_iterator end, 
+    IfdId hasMakerNote(const ExifData& exifData)
+    {
+        ExifData::const_iterator e = exifData.end();
+        for (ExifData::const_iterator md = exifData.begin(); md != e; ++md) {
+            if (ExifTags::isMakerIfd(md->ifdId())) return md->ifdId();
+        }
+        return ifdIdNotSet;
+    } // hasMakerNote
+
+    void addToIfd(Ifd& ifd,
+                  ExifMetadata::const_iterator begin,
+                  ExifMetadata::const_iterator end,
                   ByteOrder byteOrder)
     {
         for (ExifMetadata::const_iterator i = begin; i != end; ++i) {
@@ -1138,8 +1195,8 @@ namespace Exiv2 {
 
         DataBuf buf(md.size());
         md.copy(buf.pData_, byteOrder);
-        e.setValue(static_cast<uint16_t>(md.typeId()), md.count(), 
-                   buf.pData_, buf.size_); 
+        e.setValue(static_cast<uint16_t>(md.typeId()), md.count(),
+                   buf.pData_, buf.size_);
 
         DataBuf dataArea(md.dataArea());
         e.setDataArea(dataArea.pData_, dataArea.size_);
@@ -1149,18 +1206,26 @@ namespace Exiv2 {
 
     void addToMakerNote(MakerNote* makerNote,
                         ExifMetadata::const_iterator begin,
-                        ExifMetadata::const_iterator end, 
+                        ExifMetadata::const_iterator end,
                         ByteOrder byteOrder)
     {
         for (ExifMetadata::const_iterator i = begin; i != end; ++i) {
             if (ExifTags::isMakerIfd(i->ifdId())) {
-                addToMakerNote(makerNote, *i, byteOrder);
+                // Wild Hack... very ugly
+                ByteOrder bo = byteOrder;
+                if (   i->ifdId() == minoltaCs5DIfdId
+                    || i->ifdId() == minoltaCs7DIfdId
+                    || i->ifdId() == minoltaCsOldIfdId
+                    || i->ifdId() == minoltaCsNewIfdId) {
+                    bo = bigEndian;
+                }
+                addToMakerNote(makerNote, *i, bo);
             }
         }
     } // addToMakerNote
 
-    void addToMakerNote(MakerNote* makerNote, 
-                        const Exifdatum& md, 
+    void addToMakerNote(MakerNote* makerNote,
+                        const Exifdatum& md,
                         ByteOrder byteOrder)
     {
         Entry e;
@@ -1172,7 +1237,7 @@ namespace Exiv2 {
         DataBuf buf(md.size());
         md.copy(buf.pData_, byteOrder);
         e.setValue(static_cast<uint16_t>(md.typeId()), md.count(),
-                   buf.pData_, md.size()); 
+                   buf.pData_, md.size());
 
         DataBuf dataArea(md.dataArea());
         e.setDataArea(dataArea.pData_, dataArea.size_);
@@ -1193,7 +1258,7 @@ namespace {
     void setOffsetTag(Exiv2::Ifd& ifd,
                       int idx,
                       uint16_t tag,
-                      uint32_t offset, 
+                      uint32_t offset,
                       Exiv2::ByteOrder byteOrder)
     {
         Exiv2::Ifd::iterator pos = ifd.findTag(tag);
@@ -1224,7 +1289,7 @@ namespace {
         if (len != buf.size_) {
             throw Exiv2::Error(2, path, Exiv2::strError(), "FileIo::read");
         }
-        return buf; 
+        return buf;
     }
 
 }
