@@ -1,34 +1,34 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
- * 
+ * Copyright (C) 2004, 2005, 2006 Andreas Huggel <ahuggel@gmx.net>
+ *
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA.
  */
 /*
   Abstract:  Command line program to display and manipulate image %Exif data
 
   File:      exiv2.cpp
-  Version:   $Rev: 575 $
+  Version:   $Rev: 788 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   10-Dec-03, ahu: created
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: exiv2.cpp 575 2005-06-04 07:32:20Z ahuggel $");
+EXIV2_RCSID("@(#) $Id: exiv2.cpp 788 2006-05-21 10:12:18Z ahuggel $");
 
 // *****************************************************************************
 // included header files
@@ -56,12 +56,12 @@ namespace {
     //! List of all command identifiers and corresponding strings
     static const CmdIdAndString cmdIdAndString[] = {
         { add, "add" },
-        { set, "set" }, 
-        { del, "del" }, 
+        { set, "set" },
+        { del, "del" },
         { invalidCmdId, "invalidCmd" }          // End of list marker
     };
 
-    // Return a command Id for a command string 
+    // Return a command Id for a command string
     CmdId commandId(const std::string& cmdString);
 
     // Evaluate [-]HH[:MM[:SS]], returns true and sets time to the value
@@ -72,7 +72,7 @@ namespace {
       @brief Parse the oparg string into a bitmap of common targets.
       @param optarg Option arguments
       @param action Action being processed
-      @return A bitmap of common targets or -1 in case of a parse error 
+      @return A bitmap of common targets or -1 in case of a parse error
      */
     int parseCommonTargets(const std::string& optarg,
                            const std::string& action);
@@ -82,7 +82,7 @@ namespace {
       @param modifyCmds Reference to a structure to store the parsed commands
       @param cmdFiles Container with the file names
      */
-    bool parseCmdFiles(ModifyCmds& modifyCmds, 
+    bool parseCmdFiles(ModifyCmds& modifyCmds,
                        const Params::CmdFiles& cmdFiles);
 
     /*!
@@ -90,12 +90,12 @@ namespace {
       @param modifyCmds Reference to a structure to store the parsed commands
       @param cmdLines Container with the commands
      */
-    bool parseCmdLines(ModifyCmds& modifyCmds, 
+    bool parseCmdLines(ModifyCmds& modifyCmds,
                        const Params::CmdLines& cmdLines);
 
     /*!
       @brief Parse one line of the command file
-      @param modifyCmd Reference to a command structure to store the parsed 
+      @param modifyCmd Reference to a command structure to store the parsed
              command
       @param line Input line
       @param num Line number (used for error output)
@@ -126,7 +126,7 @@ int main(int argc, char* const argv[])
 
     // Create the required action class
     Action::TaskFactory& taskFactory = Action::TaskFactory::instance();
-    Action::Task::AutoPtr task 
+    Action::Task::AutoPtr task
         = taskFactory.create(Action::TaskType(params.action_));
     assert(task.get());
 
@@ -137,11 +137,15 @@ int main(int argc, char* const argv[])
     Params::Files::const_iterator e = params.files_.end();
     for (Params::Files::const_iterator i = params.files_.begin(); i != e; ++i) {
         if (params.verbose_) {
-            std::cout << "File " << std::setw(w) << n++ << "/" << s << ": " 
+            std::cout << "File " << std::setw(w) << n++ << "/" << s << ": "
                       << *i << std::endl;
         }
         task->run(*i);
     }
+
+    taskFactory.cleanup();
+    params.cleanup();
+
     return 0;
 } // main
 
@@ -157,18 +161,36 @@ Params& Params::instance()
     return *instance_;
 }
 
+void Params::cleanup()
+{
+    delete instance_;
+    instance_ = 0;
+}
+
 void Params::version(std::ostream& os) const
 {
-    os << EXV_PACKAGE_STRING << ", " 
-       << "Copyright (C) 2004, 2005 Andreas Huggel.\n\n"
-       << "This is free software; see the source for copying conditions.  "
-       << "There is NO \nwarranty; not even for MERCHANTABILITY or FITNESS FOR "
-       << "A PARTICULAR PURPOSE.\n";
+    os << EXV_PACKAGE_STRING << "\n"
+       << "Copyright (C) 2004, 2005, 2006 Andreas Huggel.\n"
+       << "\n"
+       << "This program is free software; you can redistribute it and/or\n"
+       << "modify it under the terms of the GNU General Public License\n"
+       << "as published by the Free Software Foundation; either version 2\n"
+       << "of the License, or (at your option) any later version.\n"
+       << "\n"
+       << "This program is distributed in the hope that it will be useful,\n"
+       << "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
+       << "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
+       << "GNU General Public License for more details.\n"
+       << "\n"
+       << "You should have received a copy of the GNU General Public\n"
+       << "License along with this program; if not, write to the Free\n"
+       << "Software Foundation, Inc., 51 Franklin Street, Fifth Floor,\n"
+       << "Boston, MA 02110-1301 USA\n";
 }
 
 void Params::usage(std::ostream& os) const
 {
-    os << "Usage: " << progname() 
+    os << "Usage: " << progname()
        << " [ options ] [ action ] file ...\n\n"
        << "Manipulate the Exif metadata of images.\n";
 }
@@ -182,45 +204,70 @@ void Params::help(std::ostream& os) const
        << "  pr | print    Print image metadata.\n"
        << "  rm | delete   Delete image metadata from the files.\n"
        << "  in | insert   Insert metadata from corresponding *.exv files.\n"
+       << "                Use option -S to change the suffix of the input files.\n"
        << "  ex | extract  Extract metadata to *.exv and thumbnail image files.\n"
-       << "  mv | rename   Rename files according to the Exif create timestamp.\n"
-       << "                The filename format can be set with -r format.\n"
-       << "  mo | modify   Apply commands to modify (add, set, delete) the Exif\n"
-       << "                and Iptc metadata of image files. Requires option -m or -M\n"
+       << "  mv | rename   Rename files and/or set file timestamps according to the\n"
+       << "                Exif create timestamp. The filename format can be set with\n"
+       << "                -r format, timestamp options are controlled with -t and -T.\n"
+       << "  mo | modify   Apply commands to modify (add, set, delete) the Exif and\n"
+       << "                Iptc metadata of image files or set the Jpeg comment.\n"
+       << "                Requires option -c, -m or -M.\n"
+       << "  fi | fixiso   Copy ISO setting from the Nikon Makernote to the regular\n"
+       << "                Exif tag.\n"
        << "\nOptions:\n"
        << "   -h      Display this help and exit.\n"
        << "   -V      Show the program version and exit.\n"
        << "   -v      Be verbose during the program run.\n"
+       << "   -b      Show large binary values.\n"
+       << "   -u      Don't show unknown tags.\n"
+       << "   -k      Preserve file timestamps (keep).\n"
+       << "   -t      Also set the file timestamp in 'rename' action (overrides -k).\n"
+       << "   -T      Only set the file timestamp in 'rename' action, do not rename\n"
+       << "           the file (overrides -k).\n"
        << "   -f      Do not prompt before overwriting existing files (force).\n"
-       << "   -F      Do not prompt before renaming existing files (Force).\n"
+       << "   -F      Do not prompt before renaming files (Force).\n"
        << "   -a time Time adjustment in the format [-]HH[:MM[:SS]]. This option\n"
-       << "           is only used with the `adjust' action.\n"
-       << "   -p mode Print mode for the `print' action. Possible modes are:\n"
+       << "           is only used with the 'adjust' action.\n"
+       << "   -p mode Print mode for the 'print' action. Possible modes are:\n"
        << "             s : print a summary of the Exif metadata (the default)\n"
-       << "             t : interpreted (translated) Exif data\n"
-       << "             v : plain Exif data values\n"
-       << "             h : hexdump of the Exif data\n"
+       << "             t : interpreted (translated) Exif data (shortcut for -Pkyct)\n"
+       << "             v : plain Exif data values (shortcut for -Pxgnycv)\n"
+       << "             h : hexdump of the Exif data (shortcut for -Pxgnycsh)\n"
        << "             i : Iptc data values\n"
        << "             c : Jpeg comment\n"
-       << "   -d tgt  Delete target(s) for the `delete' action. Possible targets are:\n"
+       << "   -P cols Print columns for the Exif taglist ('print' action). Valid are:\n"
+       << "             x : print a column with the tag value\n"
+       << "             g : group name\n"
+       << "             k : key\n"
+       << "             l : tag label\n"
+       << "             n : tag name\n"
+       << "             y : type\n"
+       << "             c : number of components (count)\n"
+       << "             s : size in bytes\n"
+       << "             v : plain data value\n"
+       << "             t : interpreted (translated) data\n"
+       << "             h : hexdump of the data\n"
+       << "   -d tgt  Delete target(s) for the 'delete' action. Possible targets are:\n"
        << "             a : all supported metadata (the default)\n"
        << "             e : Exif section\n"
        << "             t : Exif thumbnail only\n"
        << "             i : Iptc data\n"
        << "             c : Jpeg comment\n"
-       << "   -i tgt  Insert target(s) for the `insert' action. Possible targets are\n"
+       << "   -i tgt  Insert target(s) for the 'insert' action. Possible targets are\n"
        << "           the same as those for the -d option. Only Jpeg thumbnails can\n"
        << "           be inserted, they need to be named <file>-thumb.jpg\n"
-       << "   -e tgt  Extract target(s) for the `extract' action. Possible targets\n"
+       << "   -e tgt  Extract target(s) for the 'extract' action. Possible targets\n"
        << "           are the same as those for the -d option.\n"
-       << "   -r fmt  Filename format for the `rename' action. The format string\n"
-       << "           follows strftime(3). Default filename format is " 
+       << "   -r fmt  Filename format for the 'rename' action. The format string\n"
+       << "           follows strftime(3). Default filename format is "
        <<             format_ << ".\n"
+       << "   -c txt  Jpeg comment string to set in the image.\n"
        << "   -m file Command file for the modify action. The format for commands is\n"
        << "           set|add|del <key> [[<type>] <value>].\n"
        << "   -M cmd  Command line for the modify action. The format for the\n"
        << "           commands is the same as that of the lines of a command file.\n"
-       << "   -l dir  Location (directory) for files to be inserted or extracted.\n\n";
+       << "   -l dir  Location (directory) for files to be inserted from or extracted to.\n"
+       << "   -S .suf Use suffix .suf for source files for insert command.\n\n";
 } // Params::help
 
 int Params::option(int opt, const std::string& optarg, int optopt)
@@ -230,30 +277,38 @@ int Params::option(int opt, const std::string& optarg, int optopt)
     case 'h': help_ = true; break;
     case 'V': version_ = true; break;
     case 'v': verbose_ = true; break;
+    case 'k': preserve_ = true; break;
+    case 'b': binary_ = false; break;
+    case 'u': unknown_ = true; break;
     case 'f': force_ = true; fileExistsPolicy_ = overwritePolicy; break;
     case 'F': force_ = true; fileExistsPolicy_ = renamePolicy; break;
-    case 'r': rc = evalRename(optarg); break;
+    case 'r': rc = evalRename(opt, optarg); break;
+    case 't': rc = evalRename(opt, optarg); break;
+    case 'T': rc = evalRename(opt, optarg); break;
     case 'a': rc = evalAdjust(optarg); break;
     case 'p': rc = evalPrint(optarg); break;
+    case 'P': rc = evalPrintCols(optarg); break;
     case 'd': rc = evalDelete(optarg); break;
     case 'e': rc = evalExtract(optarg); break;
     case 'i': rc = evalInsert(optarg); break;
+    case 'c': rc = evalModify(opt, optarg); break;
     case 'm': rc = evalModify(opt, optarg); break;
     case 'M': rc = evalModify(opt, optarg); break;
     case 'l': directory_ = optarg; break;
+    case 'S': suffix_ = optarg; break;
     case ':':
-        std::cerr << progname() << ": Option -" << static_cast<char>(optopt) 
+        std::cerr << progname() << ": Option -" << static_cast<char>(optopt)
                   << " requires an argument\n";
         rc = 1;
         break;
     case '?':
-        std::cerr << progname() << ": Unrecognized option -" 
+        std::cerr << progname() << ": Unrecognized option -"
                   << static_cast<char>(optopt) << "\n";
         rc = 1;
         break;
     default:
-        std::cerr << progname() 
-                  << ": getopt returned unexpected character code " 
+        std::cerr << progname()
+                  << ": getopt returned unexpected character code "
                   << std::hex << opt << "\n";
         rc = 1;
         break;
@@ -261,21 +316,28 @@ int Params::option(int opt, const std::string& optarg, int optopt)
     return rc;
 } // Params::option
 
-int Params::evalRename(const std::string& optarg)
+int Params::evalRename(int opt, const std::string& optarg)
 {
     int rc = 0;
     switch (action_) {
     case Action::none:
         action_ = Action::rename;
-        format_ = optarg; 
+        switch (opt) {
+        case 'r': format_ = optarg; break;
+        case 't': timestamp_ = true; break;
+        case 'T': timestampOnly_ = true; break;
+        }
         break;
     case Action::rename:
-        std::cerr << progname() 
-                  << ": Ignoring surplus option -r \"" << optarg << "\"\n";
+        if (opt == 'r' && !format_.empty()) {
+            std::cerr << progname()
+                      << ": Ignoring surplus option -r \"" << optarg << "\"\n";
+        }
         break;
     default:
-        std::cerr << progname() 
-                  << ": Option -r is not compatible with a previous option\n";
+        std::cerr << progname()
+                  << ": Option -" << (char)opt
+                  << " is not compatible with a previous option\n";
         rc = 1;
         break;
     }
@@ -290,17 +352,17 @@ int Params::evalAdjust(const std::string& optarg)
         action_ = Action::adjust;
         adjust_ = parseTime(optarg, adjustment_);
         if (!adjust_) {
-            std::cerr << progname() << ": Error parsing -a option argument `" 
+            std::cerr << progname() << ": Error parsing -a option argument `"
                       << optarg << "'\n";
             rc = 1;
         }
         break;
     case Action::adjust:
-        std::cerr << progname() 
+        std::cerr << progname()
                   << ": Ignoring surplus option -a " << optarg << "\n";
         break;
     default:
-        std::cerr << progname() 
+        std::cerr << progname()
                   << ": Option -a is not compatible with a previous option\n";
         rc = 1;
         break;
@@ -313,12 +375,11 @@ int Params::evalPrint(const std::string& optarg)
     int rc = 0;
     switch (action_) {
     case Action::none:
-        action_ = Action::print;
         switch (optarg[0]) {
         case 's': printMode_ = pmSummary; break;
-        case 't': printMode_ = pmInterpreted; break;
-        case 'v': printMode_ = pmValues; break;
-        case 'h': printMode_ = pmHexdump; break;
+        case 't': rc = evalPrintCols("kyct"); break;
+        case 'v': rc = evalPrintCols("xgnycv"); break;
+        case 'h': rc = evalPrintCols("xgnycsh"); break;
         case 'i': printMode_ = pmIptc; break;
         case 'c': printMode_ = pmComment; break;
         default:
@@ -329,11 +390,11 @@ int Params::evalPrint(const std::string& optarg)
         }
         break;
     case Action::print:
-        std::cerr << progname() 
+        std::cerr << progname()
                   << ": Ignoring surplus option -p" << optarg << "\n";
         break;
     default:
-        std::cerr << progname() 
+        std::cerr << progname()
                   << ": Option -p is not compatible with a previous option\n";
         rc = 1;
         break;
@@ -341,6 +402,47 @@ int Params::evalPrint(const std::string& optarg)
     return rc;
 } // Params::evalPrint
 
+int Params::evalPrintCols(const std::string& optarg)
+{
+    int rc = 0;
+    switch (action_) {
+    case Action::none:
+        action_ = Action::print;
+        printMode_ = pmList;
+        for (std::size_t i = 0; i < optarg.length(); ++i) {
+            switch (optarg[i]) {
+            case 'x': printItems_ |= prTag;   break;
+            case 'g': printItems_ |= prGroup; break;
+            case 'k': printItems_ |= prKey;   break;
+            case 'l': printItems_ |= prLabel; break;
+            case 'n': printItems_ |= prName;  break;
+            case 'y': printItems_ |= prType;  break;
+            case 'c': printItems_ |= prCount; break;
+            case 's': printItems_ |= prSize;  break;
+            case 'v': printItems_ |= prValue; break;
+            case 't': printItems_ |= prTrans; break;
+            case 'h': printItems_ |= prHex;   break;
+            default:
+                std::cerr << progname() << ": Unrecognized print item `"
+                          << optarg[i] << "'\n";
+                rc = 1;
+                break;
+            }
+        }
+        break;
+    case Action::print:
+        std::cerr << progname()
+                  << ": Ignoring surplus option -P" << optarg << "\n";
+        break;
+    default:
+        std::cerr << progname()
+                  << ": Option -P is not compatible with a previous option\n";
+        rc = 1;
+        break;
+    }
+    return rc;
+} // Params::evalPrintCols
+
 int Params::evalDelete(const std::string& optarg)
 {
     int rc = 0;
@@ -355,12 +457,12 @@ int Params::evalDelete(const std::string& optarg)
             target_ |= rc;
             rc = 0;
         }
-        else {    
+        else {
             rc = 1;
         }
         break;
     default:
-        std::cerr << progname() 
+        std::cerr << progname()
                   << ": Option -d is not compatible with a previous option\n";
         rc = 1;
         break;
@@ -382,12 +484,12 @@ int Params::evalExtract(const std::string& optarg)
             target_ |= rc;
             rc = 0;
         }
-        else {    
+        else {
             rc = 1;
         }
         break;
     default:
-        std::cerr << progname() 
+        std::cerr << progname()
                   << ": Option -e is not compatible with a previous option\n";
         rc = 1;
         break;
@@ -409,12 +511,12 @@ int Params::evalInsert(const std::string& optarg)
             target_ |= rc;
             rc = 0;
         }
-        else {    
+        else {
             rc = 1;
         }
         break;
     default:
-        std::cerr << progname() 
+        std::cerr << progname()
                   << ": Option -i is not compatible with a previous option\n";
         rc = 1;
         break;
@@ -430,11 +532,12 @@ int Params::evalModify(int opt, const std::string& optarg)
         action_ = Action::modify;
         // fallthrough
     case Action::modify:
+        if (opt == 'c') jpegComment_ = optarg;
         if (opt == 'm') cmdFiles_.push_back(optarg);  // parse the files later
         if (opt == 'M') cmdLines_.push_back(optarg);  // parse the commands later
         break;
     default:
-        std::cerr << progname() 
+        std::cerr << progname()
                   << ": Option -" << (char)opt
                   << " is not compatible with a previous option\n";
         rc = 1;
@@ -513,6 +616,15 @@ int Params::nonoption(const std::string& argv)
             action = true;
             action_ = Action::modify;
         }
+        if (argv == "fi" || argv == "fixiso") {
+            if (action_ != Action::none && action_ != Action::fixiso) {
+                std::cerr << progname() << ": Action fixiso is not "
+                          << "compatible with the given options\n";
+                rc = 1;
+            }
+            action = true;
+            action_ = Action::fixiso;
+        }
         if (action_ == Action::none) {
             // if everything else fails, assume print as the default action
             action_ = Action::print;
@@ -525,7 +637,7 @@ int Params::nonoption(const std::string& argv)
 } // Params::nonoption
 
 int Params::getopt(int argc, char* const argv[])
-{ 
+{
     int rc = Util::Getopt::getopt(argc, argv, optstring_);
     // Further consistency checks
     if (help_ || version_) return 0;
@@ -535,13 +647,14 @@ int Params::getopt(int argc, char* const argv[])
         rc = 1;
     }
     if (action_ == Action::adjust && !adjust_) {
-        std::cerr << progname() 
+        std::cerr << progname()
                   << ": Adjust action requires option -a time\n";
         rc = 1;
     }
-    if (action_ == Action::modify && cmdFiles_.empty() && cmdLines_.empty()) {
-        std::cerr << progname() 
-                  << ": Modify action requires at least one -m or -M option\n";
+    if (   action_ == Action::modify
+        && cmdFiles_.empty() && cmdLines_.empty() && jpegComment_.empty()) {
+        std::cerr << progname()
+                  << ": Modify action requires at least one -c, -m or -M option\n";
         rc = 1;
     }
     if (0 == files_.size()) {
@@ -551,7 +664,7 @@ int Params::getopt(int argc, char* const argv[])
     if (rc == 0 && action_ == Action::modify) {
         // Parse command files
         if (!parseCmdFiles(modifyCmds_, cmdFiles_)) {
-            std::cerr << progname() << ": Error parsing -m option arguments\n"; 
+            std::cerr << progname() << ": Error parsing -m option arguments\n";
             rc = 1;
         }
     }
@@ -562,9 +675,26 @@ int Params::getopt(int argc, char* const argv[])
             rc = 1;
         }
     }
-    if (!directory_.empty() && !(action_ == Action::insert || action_ == Action::extract)) {
-        std::cerr << progname() << ": -l option can only be used with extract or insert actions\n"; 
-        rc = 1; 
+    if (   !directory_.empty()
+        && !(action_ == Action::insert || action_ == Action::extract)) {
+        std::cerr << progname()
+                  << ": -l option can only be used with extract or insert actions\n";
+        rc = 1;
+    }
+    if (!suffix_.empty() && !(action_ == Action::insert)) {
+        std::cerr << progname()
+                  << ": -S option can only be used with insert action\n";
+        rc = 1;
+    }
+    if (timestamp_ && !(action_ == Action::rename)) {
+        std::cerr << progname()
+                  << ": -t option can only be used with rename action\n";
+        rc = 1;
+    }
+    if (timestampOnly_ && !(action_ == Action::rename)) {
+        std::cerr << progname()
+                  << ": -T option can only be used with rename action\n";
+        rc = 1;
     }
     return rc;
 } // Params::getopt
@@ -613,7 +743,7 @@ namespace {
         return true;
     } // parseTime
 
-    int parseCommonTargets(const std::string& optarg, 
+    int parseCommonTargets(const std::string& optarg,
                            const std::string& action)
     {
         int rc = 0;
@@ -624,11 +754,11 @@ namespace {
             case 'i': target |= Params::ctIptc; break;
             case 'c': target |= Params::ctComment; break;
             case 't': target |= Params::ctThumb; break;
-            case 'a': target |=   Params::ctExif 
+            case 'a': target |=   Params::ctExif
                                 | Params::ctIptc
                                 | Params::ctComment; break;
             default:
-                std::cerr << Params::instance().progname() << ": Unrecognized " 
+                std::cerr << Params::instance().progname() << ": Unrecognized "
                           << action << " target `" << optarg[i] << "'\n";
                 rc = -1;
                 break;
@@ -637,7 +767,7 @@ namespace {
         return rc ? rc : target;
     } // parseCommonTargets
 
-    bool parseCmdFiles(ModifyCmds& modifyCmds, 
+    bool parseCmdFiles(ModifyCmds& modifyCmds,
                        const Params::CmdFiles& cmdFiles)
     {
         Params::CmdFiles::const_iterator end = cmdFiles.end();
@@ -646,7 +776,7 @@ namespace {
             try {
                 std::ifstream file(filename->c_str());
                 if (!file) {
-                    std::cerr << *filename 
+                    std::cerr << *filename
                               << ": Failed to open command file for reading\n";
                     return false;
                 }
@@ -667,7 +797,7 @@ namespace {
         return true;
     } // parseCmdFile
 
-    bool parseCmdLines(ModifyCmds& modifyCmds, 
+    bool parseCmdLines(ModifyCmds& modifyCmds,
                        const Params::CmdLines& cmdLines)
     {
         try {
@@ -696,21 +826,21 @@ namespace {
         std::string::size_type cmdStart = line.find_first_not_of(delim);
         if (cmdStart == std::string::npos || line[cmdStart] == '#') return false;
 
-        // Get command and key 
+        // Get command and key
         std::string::size_type cmdEnd = line.find_first_of(delim, cmdStart+1);
         std::string::size_type keyStart = line.find_first_not_of(delim, cmdEnd+1);
         std::string::size_type keyEnd = line.find_first_of(delim, keyStart+1);
         if (   cmdStart == std::string::npos
             || cmdEnd == std::string::npos
             || keyStart == std::string::npos) {
-            throw Exiv2::Error(1, Exiv2::toString(num) 
+            throw Exiv2::Error(1, Exiv2::toString(num)
                                + ": Invalid command line");
         }
 
         std::string cmd(line.substr(cmdStart, cmdEnd-cmdStart));
         CmdId cmdId = commandId(cmd);
         if (cmdId == invalidCmdId) {
-            throw Exiv2::Error(1, Exiv2::toString(num) 
+            throw Exiv2::Error(1, Exiv2::toString(num)
                                + ": Invalid command `" + cmd + "'");
         }
 
@@ -720,7 +850,7 @@ namespace {
         try {
             Exiv2::IptcKey iptcKey(key);
             metadataId = iptc;
-            defaultType = Exiv2::IptcDataSets::dataSetType(iptcKey.tag(), 
+            defaultType = Exiv2::IptcDataSets::dataSetType(iptcKey.tag(),
                                                            iptcKey.record());
         }
         catch (const Exiv2::AnyError&) {}
@@ -734,7 +864,7 @@ namespace {
             catch (const Exiv2::AnyError&) {}
         }
         if (metadataId == invalidMetadataId) {
-            throw Exiv2::Error(1, Exiv2::toString(num) 
+            throw Exiv2::Error(1, Exiv2::toString(num)
                                 + ": Invalid key `" + key + "'");
         }
 
@@ -743,17 +873,17 @@ namespace {
         bool explicitType = false;
         if (cmdId != del) {
             // Get type and value
-            std::string::size_type typeStart 
+            std::string::size_type typeStart
                 = line.find_first_not_of(delim, keyEnd+1);
-            std::string::size_type typeEnd 
+            std::string::size_type typeEnd
                 = line.find_first_of(delim, typeStart+1);
             std::string::size_type valStart = typeStart;
             std::string::size_type valEnd = line.find_last_not_of(delim);
 
-            if (   keyEnd == std::string::npos 
+            if (   keyEnd == std::string::npos
                 || typeStart == std::string::npos
                 || valStart == std::string::npos) {
-                throw Exiv2::Error(1, Exiv2::toString(num) 
+                throw Exiv2::Error(1, Exiv2::toString(num)
                                     + ": Invalid command line ");
             }
 
@@ -763,7 +893,7 @@ namespace {
                 if (tmpType != Exiv2::invalidTypeId) {
                     valStart = line.find_first_not_of(delim, typeEnd+1);
                     if (valStart == std::string::npos) {
-                        throw Exiv2::Error(1, Exiv2::toString(num) 
+                        throw Exiv2::Error(1, Exiv2::toString(num)
                                             + ": Invalid command line  ");
                     }
                     type = tmpType;
@@ -773,9 +903,9 @@ namespace {
 
             value = line.substr(valStart, valEnd+1-valStart);
             std::string::size_type last = value.length()-1;
-            if (  (value[0] == '"' || value[last] == '"') 
+            if (  (value[0] == '"' || value[last] == '"')
                 && value[0] != value[last]) {
-                throw Exiv2::Error(1, Exiv2::toString(num) 
+                throw Exiv2::Error(1, Exiv2::toString(num)
                                     + ": Unbalanced quotes");
             }
             if (value[0] == '"') {
