@@ -1,27 +1,27 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
- * 
+ * Copyright (C) 2004, 2005, 2006 Andreas Huggel <ahuggel@gmx.net>
+ *
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA.
  */
 /*!
   @file    exiv2.hpp
   @brief   Defines class Params, used for the command line handling of exiv2
-  @version $Rev: 567 $
+  @version $Rev: 788 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    08-Dec-03, ahu: created
@@ -46,19 +46,19 @@
 enum CmdId { invalidCmdId, add, set, del };
 //! Metadata identifiers
 enum MetadataId { invalidMetadataId, iptc, exif };
-//! Structure for one parsed modification command 
+//! Structure for one parsed modification command
 struct ModifyCmd {
     //! C'tor
     ModifyCmd() :
-        cmdId_(invalidCmdId), metadataId_(invalidMetadataId), 
+        cmdId_(invalidCmdId), metadataId_(invalidMetadataId),
         typeId_(Exiv2::invalidTypeId), explicitType_(false) {}
     CmdId cmdId_;                               //!< Command identifier
     std::string key_;                           //!< Exiv2 key string
-    MetadataId metadataId_;                     //!< Metadata identifier 
+    MetadataId metadataId_;                     //!< Metadata identifier
     Exiv2::TypeId typeId_;                      //!< Exiv2 type identifier
     //! Flag to indicate if the type was explicitely specified (true)
     bool explicitType_;
-    std::string value_;                         //!< Data 
+    std::string value_;                         //!< Data
 };
 //! Container for modification commands
 typedef std::vector<ModifyCmd> ModifyCmds;
@@ -69,7 +69,7 @@ struct CmdIdAndString {
 };
 
 /*!
-  @brief Implements the command line handling for the program. 
+  @brief Implements the command line handling for the program.
 
   Derives from Util::Getopt to use the command line argument parsing
   functionalty provided there. This class is implemented as a Singleton,
@@ -119,10 +119,27 @@ public:
       @return Reference to the global Params instance.
     */
     static Params& instance();
+    //! Destructor
+    void cleanup();
 
     //! Enumerates print modes
-    enum PrintMode { pmSummary, pmInterpreted, pmValues, pmHexdump, pmIptc, 
-                     pmComment };
+    enum PrintMode { pmSummary, pmList, pmIptc, pmComment };
+
+    //! Individual items to print
+    enum PrintItem {
+        prTag   =    1,
+        prGroup =    2,
+        prKey   =    4,
+        prName  =    8,
+        prLabel =   16,
+        prType  =   32,
+        prCount =   64,
+        prSize  =  128,
+        prValue =  256,
+        prTrans =  512,
+        prHex   = 1024
+    };
+
     //! Enumerates common targets, bitmap
     enum CommonTarget { ctExif = 1, ctIptc = 2, ctComment = 4, ctThumb = 8 };
     //! Enumerates the policies to handle existing files in rename action
@@ -131,10 +148,16 @@ public:
     bool help_;                         //!< Help option flag.
     bool version_;                      //!< Version option flag.
     bool verbose_;                      //!< Verbose (talkative) option flag.
-    bool force_;                        //!< Force overwrites flag. 
+    bool force_;                        //!< Force overwrites flag.
+    bool binary_;                       //!< Suppress long binary values.
+    bool unknown_;                      //!< Suppress unknown tags.
+    bool preserve_;                     //!< Preserve timestamps flag.
+    bool timestamp_;                    //!< Rename also sets the file timestamp.
+    bool timestampOnly_;                //!< Rename only sets the file timestamp.
     FileExistsPolicy fileExistsPolicy_; //!< What to do if file to rename exists.
     bool adjust_;                       //!< Adjustment flag.
-    PrintMode printMode_;               //!< Print mode. 
+    PrintMode printMode_;               //!< Print mode.
+    unsigned long printItems_;          //!< Print items.
     //! %Action (integer rather than TaskType to avoid dependency).
     int  action_;
     int  target_;                       //!< What common target to process.
@@ -144,7 +167,9 @@ public:
     CmdFiles cmdFiles_;                 //!< Names of the modification command files
     CmdLines cmdLines_;                 //!< Commands from the command line
     ModifyCmds modifyCmds_;             //!< Parsed modification commands
+    std::string jpegComment_;           //!< Jpeg comment to set in the image
     std::string directory_;             //!< Location for files to extract/insert
+    std::string suffix_;                //!< File extension of the file to insert
     Files files_;                       //!< List of non-option arguments.
 
 private:
@@ -152,14 +177,20 @@ private:
       @brief Default constructor. Note that optstring_ is initialized here.
              The c'tor is private to force instantiation through instance().
      */
-    Params() : optstring_(":hVvfFa:r:p:d:e:i:m:M:l:"),
-               help_(false), 
+    Params() : optstring_(":hVvfbuktTFa:r:p:P:d:e:i:c:m:M:l:S:"),
+               help_(false),
                version_(false),
-               verbose_(false), 
-               force_(false), 
+               verbose_(false),
+               force_(false),
+               binary_(true),
+               unknown_(false),
+               preserve_(false),
+               timestamp_(false),
+               timestampOnly_(false),
                fileExistsPolicy_(askPolicy),
                adjust_(false),
                printMode_(pmSummary),
+               printItems_(0),
                action_(0),
                target_(ctExif|ctIptc|ctComment),
                adjustment_(0),
@@ -171,9 +202,10 @@ private:
 
     //! @name Helpers
     //@{
-    int evalRename(const std::string& optarg);
+    int evalRename(int opt, const std::string& optarg);
     int evalAdjust(const std::string& optarg);
     int evalPrint(const std::string& optarg);
+    int evalPrintCols(const std::string& optarg);
     int evalDelete(const std::string& optarg);
     int evalExtract(const std::string& optarg);
     int evalInsert(const std::string& optarg);
