@@ -23,13 +23,13 @@
 #define EXV_PACKAGE_NAME "Exiv2"
 
 /* Define to the full name and version of this package. */
-#define EXV_PACKAGE_STRING "Exiv2 0.7"
+#define EXV_PACKAGE_STRING "Exiv2 0.10"
 
 /* Define to the one symbol short name of this package. */
 #define EXV_PACKAGE_TARNAME "exiv2"
 
 /* Define to the version of this package. */
-#define EXV_PACKAGE_VERSION "0.7"
+#define EXV_PACKAGE_VERSION "0.10"
 
 /* Define to `int' if <sys/types.h> does not define pid_t. */
 #define pid_t int
@@ -38,4 +38,13 @@
 #define EXV_SEPERATOR_STR "\\"
 #define EXV_SEPERATOR_CHR '\\'
 
+/*
+  Visual Studio C++ 2005 (8.0)
+  Disable warnings about 'deprecated' standard functions
+  See, eg. http://www.informit.com/guides/content.asp?g=cplusplus&seqNum=259
+*/
+#if _MSC_VER >= 1400
+# pragma warning(disable : 4996)
+#endif
+
 #endif /* _MSC_VER */
