@@ -1,37 +1,37 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
- * 
+ * Copyright (C) 2004, 2005, 2006 Andreas Huggel <ahuggel@gmx.net>
+ *
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA.
  */
 /*
   File:      fujimn.cpp
-  Version:   $Rev: 569 $
+  Version:   $Rev: 669 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   18-Feb-04, ahu: created
              07-Mar-04, ahu: isolated as a separate component
   Credits:   Fujifilm MakerNote implemented according to the specification
-             in "Appendix 4: Makernote of Fujifilm" of the document 
+             in "Appendix 4: Makernote of Fujifilm" of the document
              "Exif file format" by TsuruZoh Tachibanaya
              <http://park2.wakwak.com/%7Etsuruzoh/Computer/Digicams/exif-e.html>
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: fujimn.cpp 569 2005-05-28 05:48:43Z ahuggel $");
+EXIV2_RCSID("@(#) $Id: fujimn.cpp 669 2006-01-17 14:53:57Z ahuggel $");
 
 // *****************************************************************************
 // included header files
@@ -46,9 +46,6 @@ EXIV2_RCSID("@(#) $Id: fujimn.cpp 569 2005-05-28 05:48:43Z ahuggel $");
 #include <iomanip>
 #include <cassert>
 
-// Define DEBUG_MAKERNOTE to output debug information to std::cerr
-#undef DEBUG_MAKERNOTE
-
 // *****************************************************************************
 // class member definitions
 namespace Exiv2 {
@@ -66,35 +63,35 @@ namespace Exiv2 {
 
     // Fujifilm MakerNote Tag Info
     const TagInfo FujiMakerNote::tagInfo_[] = {
-        TagInfo(0x0000, "Version", "Fujifilm Makernote version", fujiIfdId, makerTags, undefined, printValue),
-        TagInfo(0x1000, "Quality", "Image quality setting", fujiIfdId, makerTags, asciiString, printValue),
-        TagInfo(0x1001, "Sharpness", "Sharpness setting", fujiIfdId, makerTags, unsignedShort, print0x1001),
-        TagInfo(0x1002, "WhiteBalance", "White balance setting", fujiIfdId, makerTags, unsignedShort, print0x1002),
-        TagInfo(0x1003, "Color", "Chroma saturation setting", fujiIfdId, makerTags, unsignedShort, print0x1003),
-        TagInfo(0x1004, "Tone", "Contrast setting", fujiIfdId, makerTags, unsignedShort, print0x1004),
-        TagInfo(0x1010, "FlashMode", "Flash firing mode setting", fujiIfdId, makerTags, unsignedShort, print0x1010),
-        TagInfo(0x1011, "FlashStrength", "Flash firing strength compensation setting", fujiIfdId, makerTags, signedRational, printValue),
-        TagInfo(0x1020, "Macro", "Macro mode setting", fujiIfdId, makerTags, unsignedShort, printOffOn),
-        TagInfo(0x1021, "FocusMode", "Focusing mode setting", fujiIfdId, makerTags, unsignedShort, print0x1021),
-        TagInfo(0x1022, "0x1022", "Unknown", fujiIfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x1030, "SlowSync", "Slow synchro mode setting", fujiIfdId, makerTags, unsignedShort, printOffOn),
-        TagInfo(0x1031, "PictureMode", "Picture mode setting", fujiIfdId, makerTags, unsignedShort, print0x1031),
-        TagInfo(0x1032, "0x1032", "Unknown", fujiIfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x1100, "Continuous", "Continuous shooting or auto bracketing setting", fujiIfdId, makerTags, unsignedShort, printOffOn),
-        TagInfo(0x1101, "0x1101", "Unknown", fujiIfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x1200, "0x1200", "Unknown", fujiIfdId, makerTags, unsignedShort, printValue),
-        TagInfo(0x1300, "BlurWarning", "Blur warning status", fujiIfdId, makerTags, unsignedShort, printOffOn),
-        TagInfo(0x1301, "FocusWarning", "Auto Focus warning status", fujiIfdId, makerTags, unsignedShort, printOffOn),
-        TagInfo(0x1302, "AeWarning", "Auto Exposure warning status", fujiIfdId, makerTags, unsignedShort, printOffOn),
+        TagInfo(0x0000, "Version", "Version", "Fujifilm Makernote version", fujiIfdId, makerTags, undefined, printValue),
+        TagInfo(0x1000, "Quality", "Quality", "Image quality setting", fujiIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x1001, "Sharpness", "Sharpness", "Sharpness setting", fujiIfdId, makerTags, unsignedShort, print0x1001),
+        TagInfo(0x1002, "WhiteBalance", "WhiteBalance", "White balance setting", fujiIfdId, makerTags, unsignedShort, print0x1002),
+        TagInfo(0x1003, "Color", "Color", "Chroma saturation setting", fujiIfdId, makerTags, unsignedShort, print0x1003),
+        TagInfo(0x1004, "Tone", "Tone", "Contrast setting", fujiIfdId, makerTags, unsignedShort, print0x1004),
+        TagInfo(0x1010, "FlashMode", "FlashMode", "Flash firing mode setting", fujiIfdId, makerTags, unsignedShort, print0x1010),
+        TagInfo(0x1011, "FlashStrength", "FlashStrength", "Flash firing strength compensation setting", fujiIfdId, makerTags, signedRational, printValue),
+        TagInfo(0x1020, "Macro", "Macro", "Macro mode setting", fujiIfdId, makerTags, unsignedShort, printOffOn),
+        TagInfo(0x1021, "FocusMode", "FocusMode", "Focusing mode setting", fujiIfdId, makerTags, unsignedShort, print0x1021),
+        TagInfo(0x1022, "0x1022", "0x1022", "Unknown", fujiIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x1030, "SlowSync", "SlowSync", "Slow synchro mode setting", fujiIfdId, makerTags, unsignedShort, printOffOn),
+        TagInfo(0x1031, "PictureMode", "PictureMode", "Picture mode setting", fujiIfdId, makerTags, unsignedShort, print0x1031),
+        TagInfo(0x1032, "0x1032", "0x1032", "Unknown", fujiIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x1100, "Continuous", "Continuous", "Continuous shooting or auto bracketing setting", fujiIfdId, makerTags, unsignedShort, printOffOn),
+        TagInfo(0x1101, "0x1101", "0x1101", "Unknown", fujiIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x1200, "0x1200", "0x1200", "Unknown", fujiIfdId, makerTags, unsignedShort, printValue),
+        TagInfo(0x1300, "BlurWarning", "BlurWarning", "Blur warning status", fujiIfdId, makerTags, unsignedShort, printOffOn),
+        TagInfo(0x1301, "FocusWarning", "FocusWarning", "Auto Focus warning status", fujiIfdId, makerTags, unsignedShort, printOffOn),
+        TagInfo(0x1302, "AeWarning", "AeWarning", "Auto Exposure warning status", fujiIfdId, makerTags, unsignedShort, printOffOn),
         // End of list marker
-        TagInfo(0xffff, "(UnknownFujiMakerNoteTag)", "Unknown FujiMakerNote tag", fujiIfdId, makerTags, invalidTypeId, printValue)
+        TagInfo(0xffff, "(UnknownFujiMakerNoteTag)", "(UnknownFujiMakerNoteTag)", "Unknown FujiMakerNote tag", fujiIfdId, makerTags, invalidTypeId, printValue)
     };
 
     FujiMakerNote::FujiMakerNote(bool alloc)
         : IfdMakerNote(fujiIfdId, alloc)
     {
         byteOrder_ = littleEndian;
-        absOffset_ = false;
+        absShift_ = false;
         byte buf[] = {
             'F', 'U', 'J', 'I', 'F', 'I', 'L', 'M', 0x0c, 0x00, 0x00, 0x00
         };
@@ -107,16 +104,16 @@ namespace Exiv2 {
     }
 
     int FujiMakerNote::readHeader(const byte* buf,
-                                  long len, 
+                                  long len,
                                   ByteOrder byteOrder)
     {
         if (len < 12) return 1;
 
         header_.alloc(12);
         memcpy(header_.pData_, buf, header_.size_);
-        // Read the offset relative to the start of the makernote from the header
-        // Note: we ignore the byteOrder paramter
-        adjOffset_ = getUShort(header_.pData_ + 8, byteOrder_);
+        // Read offset to the IFD relative to the start of the makernote
+        // from the header. Note that we ignore the byteOrder paramter
+        start_ = getUShort(header_.pData_ + 8, byteOrder_);
         return 0;
     }
 
@@ -125,7 +122,7 @@ namespace Exiv2 {
         int rc = 0;
         // Check the FUJIFILM prefix
         if (   header_.size_ < 12
-            || std::string(reinterpret_cast<char*>(header_.pData_), 8) 
+            || std::string(reinterpret_cast<char*>(header_.pData_), 8)
                     != std::string("FUJIFILM", 8)) {
             rc = 2;
         }
@@ -137,7 +134,7 @@ namespace Exiv2 {
         return AutoPtr(create_(alloc));
     }
 
-    FujiMakerNote* FujiMakerNote::create_(bool alloc) const 
+    FujiMakerNote* FujiMakerNote::create_(bool alloc) const
     {
         AutoPtr makerNote = AutoPtr(new FujiMakerNote(alloc));
         assert(makerNote.get() != 0);
@@ -150,7 +147,7 @@ namespace Exiv2 {
         return AutoPtr(clone_());
     }
 
-    FujiMakerNote* FujiMakerNote::clone_() const 
+    FujiMakerNote* FujiMakerNote::clone_() const
     {
         return new FujiMakerNote(*this);
     }
@@ -166,7 +163,7 @@ namespace Exiv2 {
         return os;
     }
 
-    std::ostream& FujiMakerNote::print0x1001(std::ostream& os, 
+    std::ostream& FujiMakerNote::print0x1001(std::ostream& os,
                                              const Value& value)
     {
         switch (value.toLong()) {
@@ -176,7 +173,7 @@ namespace Exiv2 {
         case 4: // fallthrough
         case 5: os << "Hard"; break;
         default: os << "(" << value << ")"; break;
-        } 
+        }
         return os;
     }
 
@@ -209,7 +206,7 @@ namespace Exiv2 {
         return os;
     }
 
-    std::ostream& FujiMakerNote::print0x1004(std::ostream& os, 
+    std::ostream& FujiMakerNote::print0x1004(std::ostream& os,
                                              const Value& value)
     {
         switch (value.toLong()) {
@@ -221,7 +218,7 @@ namespace Exiv2 {
         return os;
     }
 
-    std::ostream& FujiMakerNote::print0x1010(std::ostream& os, 
+    std::ostream& FujiMakerNote::print0x1010(std::ostream& os,
                                              const Value& value)
     {
         switch (value.toLong()) {
@@ -267,9 +264,9 @@ namespace Exiv2 {
 // free functions
 
     MakerNote::AutoPtr createFujiMakerNote(bool alloc,
-                                           const byte* buf, 
-                                           long len, 
-                                           ByteOrder byteOrder, 
+                                           const byte* buf,
+                                           long len,
+                                           ByteOrder byteOrder,
                                            long offset)
     {
         return MakerNote::AutoPtr(new FujiMakerNote(alloc));
