@@ -1,33 +1,33 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
- * 
+ * Copyright (C) 2004, 2005, 2006 Andreas Huggel <ahuggel@gmx.net>
+ *
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA.
  */
 /*
   File:      ifd.cpp
-  Version:   $Rev: 562 $
+  Version:   $Rev: 763 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   26-Jan-04, ahu: created
              11-Feb-04, ahu: isolated as a component
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: ifd.cpp 562 2005-04-20 18:16:29Z ahuggel $");
+EXIV2_RCSID("@(#) $Id: ifd.cpp 763 2006-05-09 19:42:23Z ahuggel $");
 
 // *****************************************************************************
 // included header files
@@ -50,9 +50,9 @@ EXIV2_RCSID("@(#) $Id: ifd.cpp 562 2005-04-20 18:16:29Z ahuggel $");
 namespace Exiv2 {
 
     Entry::Entry(bool alloc)
-        : alloc_(alloc), ifdId_(ifdIdNotSet), idx_(0), 
+        : alloc_(alloc), ifdId_(ifdIdNotSet), idx_(0),
           tag_(0), type_(0), count_(0), offset_(0), size_(0), pData_(0),
-          sizeDataArea_(0), pDataArea_(0)
+          sizeDataArea_(0), pDataArea_(0), byteOrder_(invalidByteOrder)
     {
     }
 
@@ -66,9 +66,9 @@ namespace Exiv2 {
 
     Entry::Entry(const Entry& rhs)
         : alloc_(rhs.alloc_), ifdId_(rhs.ifdId_), idx_(rhs.idx_),
-          tag_(rhs.tag_), type_(rhs.type_), 
+          tag_(rhs.tag_), type_(rhs.type_),
           count_(rhs.count_), offset_(rhs.offset_), size_(rhs.size_), pData_(0),
-          sizeDataArea_(rhs.sizeDataArea_), pDataArea_(0)
+          sizeDataArea_(rhs.sizeDataArea_), pDataArea_(0), byteOrder_(rhs.byteOrder_)
     {
         if (alloc_) {
             if (rhs.pData_) {
@@ -98,6 +98,7 @@ namespace Exiv2 {
         offset_ = rhs.offset_;
         size_ = rhs.size_;
         sizeDataArea_ = rhs.sizeDataArea_;
+        byteOrder_ = rhs.byteOrder_;
         if (alloc_) {
             delete[] pData_;
             pData_ = 0;
@@ -133,8 +134,9 @@ namespace Exiv2 {
         count_ = 1;
     }
 
-    void Entry::setValue(uint16_t type, uint32_t count, const byte* buf, long len)
+    void Entry::setValue(uint16_t type, uint32_t count, const byte* buf, long len, ByteOrder byteOrder)
     {
+        byteOrder_ = byteOrder;
         long dataSize = count * TypeInfo::typeSize(TypeId(type));
         // No minimum size requirement, but make sure the buffer can hold the data
         if (len < dataSize) throw Error(24, tag(), dataSize, len);
@@ -201,7 +203,7 @@ namespace Exiv2 {
                 break;
             }
             case unsignedLong: {
-                ul2Data(buf, getULong(buf, byteOrder) + offset, byteOrder); 
+                ul2Data(buf, getULong(buf, byteOrder) + offset, byteOrder);
                 break;
             }
             case unsignedRational: {
@@ -253,7 +255,7 @@ namespace Exiv2 {
     } // Entry::component
 
     Ifd::Ifd(IfdId ifdId)
-        : alloc_(true), ifdId_(ifdId), pBase_(0), offset_(0), 
+        : alloc_(true), ifdId_(ifdId), pBase_(0), offset_(0),
           dataOffset_(0), hasNext_(true), pNext_(0), next_(0)
     {
         pNext_ = new byte[4];
@@ -261,7 +263,7 @@ namespace Exiv2 {
     }
 
     Ifd::Ifd(IfdId ifdId, long offset)
-        : alloc_(true), ifdId_(ifdId), pBase_(0), offset_(offset), 
+        : alloc_(true), ifdId_(ifdId), pBase_(0), offset_(offset),
           dataOffset_(0), hasNext_(true), pNext_(0), next_(0)
     {
         pNext_ = new byte[4];
@@ -269,7 +271,7 @@ namespace Exiv2 {
     }
 
     Ifd::Ifd(IfdId ifdId, long offset, bool alloc, bool hasNext)
-        : alloc_(alloc), ifdId_(ifdId), pBase_(0), offset_(offset), 
+        : alloc_(alloc), ifdId_(ifdId), pBase_(0), offset_(offset),
           dataOffset_(0), hasNext_(hasNext), pNext_(0), next_(0)
     {
         if (alloc_ && hasNext_) {
@@ -286,37 +288,39 @@ namespace Exiv2 {
 
     Ifd::Ifd(const Ifd& rhs)
         : alloc_(rhs.alloc_), entries_(rhs.entries_), ifdId_(rhs.ifdId_),
-          pBase_(rhs.pBase_), offset_(rhs.offset_), dataOffset_(rhs.dataOffset_), 
+          pBase_(rhs.pBase_), offset_(rhs.offset_), dataOffset_(rhs.dataOffset_),
           hasNext_(rhs.hasNext_), pNext_(rhs.pNext_), next_(rhs.next_)
     {
         if (alloc_ && hasNext_) {
             pNext_ = new byte[4];
             memset(pNext_, 0x0, 4);
-            if (rhs.pNext_) memcpy(pNext_, rhs.pNext_, 4); 
+            if (rhs.pNext_) memcpy(pNext_, rhs.pNext_, 4);
         }
     }
 
-    int Ifd::read(const byte* buf, long len, ByteOrder byteOrder, long offset)
+    int Ifd::read(const byte* buf,
+                  long len,
+                  long start,
+                  ByteOrder byteOrder,
+                  long shift)
     {
-        // Todo: This is a hack to work around bug #424 - fix it properly!
-        if (ifdId_ == olympusIfdId) len = 65535;
-
         int rc = 0;
-        long o = 0;
+        long o = start;
         Ifd::PreEntries preEntries;
 
-        if (len < 2) rc = 6;
+        if (len < o + 2) rc = 6;
         if (rc == 0) {
-            offset_ = offset;
-            int n = getUShort(buf, byteOrder);
-            o = 2;
+            offset_ = start - shift;
+            int n = getUShort(buf + o, byteOrder);
+            o += 2;
 
             for (int i = 0; i < n; ++i) {
                 if (len < o + 12) {
-                    // Todo: How to handle debug output like this
-                    std::cerr << "Error: " << ExifTags::ifdName(ifdId_) 
+#ifndef SUPPRESS_WARNINGS
+                    std::cerr << "Error: " << ExifTags::ifdName(ifdId_)
                               << " entry " << i
                               << " lies outside of the IFD memory buffer.\n";
+#endif
                     rc = 6;
                     break;
                 }
@@ -325,7 +329,7 @@ namespace Exiv2 {
                 pe.type_ = getUShort(buf + o + 2, byteOrder);
                 pe.count_ = getULong(buf + o + 4, byteOrder);
                 pe.size_ = pe.count_ * TypeInfo::typeSize(TypeId(pe.type_));
-                pe.offsetLoc_ = o + 8;
+                pe.offsetLoc_ = o + 8 - shift;
                 pe.offset_ = pe.size_ > 4 ? getLong(buf + o + 8, byteOrder) : 0;
                 preEntries.push_back(pe);
                 o += 12;
@@ -333,10 +337,11 @@ namespace Exiv2 {
         }
         if (rc == 0 && hasNext_) {
             if (len < o + 4) {
-                // Todo: How to handle debug output like this
-                std::cerr << "Error: " << ExifTags::ifdName(ifdId_) 
+#ifndef SUPPRESS_WARNINGS
+                std::cerr << "Error: " << ExifTags::ifdName(ifdId_)
                           << " memory of the pointer to the next IFD"
                           << " lies outside of the IFD memory buffer.\n";
+#endif
                 rc = 6;
             }
             else {
@@ -350,10 +355,6 @@ namespace Exiv2 {
             }
         }
         // Set the offset of the first data entry outside of the IFD.
-        // At the same time we guess the offset of the IFD, if it was not
-        // given. The guess is based on the assumption that the smallest offset
-        // points to a data buffer directly following the IFD. Subsequently all
-        // offsets of IFD entries will need to be recalculated.
         if (rc == 0 && preEntries.size() > 0) {
             // Find the entry with the smallest offset
             Ifd::PreEntries::const_iterator i = std::min_element(
@@ -361,24 +362,33 @@ namespace Exiv2 {
             // Only do something if there is at least one entry with data
             // outside the IFD directory itself.
             if (i->size_ > 4) {
-                if (offset_ == 0) {
-                    // Set the 'guessed' IFD offset
-                    offset_ = i->offset_ 
-		      - (2 + 12 * static_cast<long>(preEntries.size()) 
-                         + (hasNext_ ? 4 : 0));
-                }
                 // Set the offset of the first data entry outside of the IFD
-                if (i->offset_ - offset_ >= len) {
-                    // Todo: How to handle debug output like this
-                    std::cerr << "Error: Offset of the 1st data entry of " 
-                              << ExifTags::ifdName(ifdId_) 
+                if (i->offset_ + shift < 0) {
+#ifndef SUPPRESS_WARNINGS
+                    std::cerr << "Error: Offset of the 1st data entry of "
+                              << ExifTags::ifdName(ifdId_)
                               << " is out of bounds:\n"
-                              << " Offset = 0x" << std::setw(8) 
-                              << std::setfill('0') << std::hex 
-                              << i->offset_ - offset_
-                              << ", exceeds buffer size by "
-                              << std::dec << i->offset_ - len
+                              << " Offset = 0x" << std::setw(8)
+                              << std::setfill('0') << std::hex
+                              << i->offset_ - offset_ // relative to start of IFD
+                              << ", is before start of buffer by "
+                              << std::dec << -1 * (i->offset_ + shift)
                               << " Bytes\n";
+#endif
+                    rc = 6;
+                }
+                else if (i->offset_ + shift + i->size_ > len) {
+#ifndef SUPPRESS_WARNINGS
+                    std::cerr << "Error: Upper boundary of the 1st data entry of "
+                              << ExifTags::ifdName(ifdId_)
+                              << " is out of bounds:\n"
+                              << " Offset = 0x" << std::setw(8)
+                              << std::setfill('0') << std::hex
+                              << i->offset_ - offset_ // relative to start of IFD
+                              << ", exceeds buffer size by "
+                              << std::dec << i->offset_ + shift + i->size_ - len
+                              << " Bytes\n";
+#endif
                     rc = 6;
                 }
                 else {
@@ -399,41 +409,52 @@ namespace Exiv2 {
                 e.setIfdId(ifdId_);
                 e.setIdx(++idx);
                 e.setTag(i->tag_);
-                long tmpOffset = 
-                    i->size_ > 4 ? i->offset_ - offset_ : i->offsetLoc_;
-                if (tmpOffset + i->size_ > len) {
-                    // Todo: How to handle debug output like this
-                    std::cerr << "Warning: Upper boundary of data for " 
-                              << ExifTags::ifdName(ifdId_) 
-                              << " entry " << static_cast<int>(i - begin) 
+                long tmpOffset = // still from the start of the TIFF header
+                    i->size_ > 4 ? i->offset_ : i->offsetLoc_;
+                if (tmpOffset + shift + i->size_ > len) {
+#ifndef SUPPRESS_WARNINGS
+                    std::cerr << "Warning: Upper boundary of data for "
+                              << ExifTags::ifdName(ifdId_)
+                              << " entry " << static_cast<int>(i - begin)
                               << " is out of bounds:\n"
-                              << " Offset = 0x" << std::setw(8) 
-                              << std::setfill('0') << std::hex 
-                              << tmpOffset
-                              << ", size = " << std::dec << i->size_ 
+                              << " Offset = 0x" << std::setw(8)
+                              << std::setfill('0') << std::hex
+                              << tmpOffset - offset_ // relative to start of IFD
+                              << ", size = " << std::dec << i->size_
                               << ", exceeds buffer size by "
-                              << tmpOffset + i->size_ - len
+                              << tmpOffset + shift + i->size_ - len
                               << " Bytes; Truncating the data.\n";
+#endif
                     // Truncate the entry
                     i->size_ = 0;
                     i->count_ = 0;
                     tmpOffset = i->offsetLoc_;
                 }
                 // Set the offset to the data, relative to start of IFD
-                e.setOffset(tmpOffset);
+                e.setOffset(tmpOffset - offset_);
                 // Set the size to at least for bytes to accomodate offset-data
-                e.setValue(i->type_, i->count_, buf + e.offset(), 
+#ifndef SUPPRESS_WARNINGS
+                if (i->type_ < 1 || i->type_ > 10 || i->type_ == 6) {
+                    std::cerr << "Warning: "
+                              << ExifTags::ifdName(ifdId_) << " tag 0x"
+                              << std::setw(4) << std::setfill('0') << std::hex
+                              << i->tag_ << " has invalid Exif type "
+                              << std::dec << i->type_
+                              << "; using 7 (undefined).\n";
+                }
+#endif
+                e.setValue(i->type_, i->count_, buf + start + e.offset(),
                            std::max(long(4), i->size_));
                 this->add(e);
             }
         }
-        if (!alloc_) pBase_ = const_cast<byte*>(buf) - offset_;
+        if (!alloc_) pBase_ = const_cast<byte*>(buf + shift);
         if (rc) this->clear();
 
         return rc;
     } // Ifd::read
 
-    Ifd::const_iterator Ifd::findIdx(int idx) const 
+    Ifd::const_iterator Ifd::findIdx(int idx) const
     {
         return std::find_if(entries_.begin(), entries_.end(),
                             FindEntryByIdx(idx));
@@ -445,7 +466,7 @@ namespace Exiv2 {
                             FindEntryByIdx(idx));
     }
 
-    Ifd::const_iterator Ifd::findTag(uint16_t tag) const 
+    Ifd::const_iterator Ifd::findTag(uint16_t tag) const
     {
         return std::find_if(entries_.begin(), entries_.end(),
                             FindEntryByTag(tag));
@@ -474,7 +495,7 @@ namespace Exiv2 {
                 rc = 6;
             }
             else {
-                rc = dest.read(buf + offset, len - offset, byteOrder, offset);
+                rc = dest.read(buf, len, offset, byteOrder);
             }
         }
         return rc;
@@ -505,7 +526,7 @@ namespace Exiv2 {
             us2Data(buf + o, i->tag(), byteOrder);
             us2Data(buf + o + 2, i->type(), byteOrder);
             ul2Data(buf + o + 4, i->count(), byteOrder);
-            if (i->sizeDataArea() > 0) { 
+            if (i->sizeDataArea() > 0) {
                 long dataAreaOffset = offset_+size()+totalDataSize+dataAreaSize;
                 i->setDataAreaOffsets(dataAreaOffset, byteOrder);
                 dataAreaSize += i->sizeDataArea();
@@ -624,7 +645,7 @@ namespace Exiv2 {
     long Ifd::size() const
     {
         if (entries_.size() == 0 && next_ == 0) return 0;
-        return static_cast<long>(2 + 12 * entries_.size() + (hasNext_ ? 4 : 0)); 
+        return static_cast<long>(2 + 12 * entries_.size() + (hasNext_ ? 4 : 0));
     }
 
     long Ifd::dataSize() const
@@ -643,9 +664,9 @@ namespace Exiv2 {
         if (entries_.size() == 0) return;
         // Print a header
         os << prefix << "IFD Offset: 0x"
-           << std::setw(8) << std::setfill('0') << std::hex << std::right 
-           << offset_ 
-           << ",   IFD Entries: " 
+           << std::setw(8) << std::setfill('0') << std::hex << std::right
+           << offset_
+           << ",   IFD Entries: "
            << std::setfill(' ') << std::dec << std::right
            << static_cast<unsigned int>(entries_.size()) << "\n"
            << prefix << "Entry     Tag  Format   (Bytes each)  Number  Offset\n"
@@ -669,10 +690,10 @@ namespace Exiv2 {
             }
             os << prefix << std::setw(5) << std::setfill(' ') << std::dec
                << std::right << static_cast<int>(i - b)
-               << "  0x" << std::setw(4) << std::setfill('0') << std::hex 
+               << "  0x" << std::setw(4) << std::setfill('0') << std::hex
                << std::right << i->tag()
-               << "  " << std::setw(17) << std::setfill(' ') 
-               << std::left << i->typeName() 
+               << "  " << std::setw(17) << std::setfill(' ')
+               << std::left << i->typeName()
                << " (" << std::dec << i->typeSize() << ")"
                << "  " << std::setw(6) << std::setfill(' ') << std::dec
                << std::right << i->count()
@@ -680,11 +701,11 @@ namespace Exiv2 {
                << "\n";
         }
         if (hasNext_) {
-            os << prefix << "Next IFD: 0x" 
+            os << prefix << "Next IFD: 0x"
                << std::setw(8) << std::setfill('0') << std::hex
                << std::right << next() << "\n";
         }
-        // Print data of IFD entries 
+        // Print data of IFD entries
         for (i = b; i != e; ++i) {
             if (i->size() > 4) {
                 os << "Data of entry " << static_cast<int>(i - b) << ":\n";
