@@ -1,26 +1,26 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
- * 
+ * Copyright (C) 2004, 2005, 2006 Andreas Huggel <ahuggel@gmx.net>
+ *
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA.
  */
 /*
   File:      image.cpp
-  Version:   $Rev: 563 $
+  Version:   $Rev: 692 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
              Brad Schick (brad) <brad@robotbattle.com>
   History:   26-Jan-04, ahu: created
@@ -30,7 +30,7 @@
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: image.cpp 563 2005-04-21 07:21:53Z ahuggel $");
+EXIV2_RCSID("@(#) $Id: image.cpp 692 2006-02-03 14:36:32Z ahuggel $");
 
 // *****************************************************************************
 // included header files
@@ -44,14 +44,10 @@ EXIV2_RCSID("@(#) $Id: image.cpp 563 2005-04-21 07:21:53Z ahuggel $");
 #include "error.hpp"
 #include "futils.hpp"
 
-// Ensure registration with factory
-#include "jpgimage.hpp"
-
 // + standard includes
 #include <cerrno>
 #include <cstdio>
 #include <cstring>
-#include <cstdio>                               // for rename, remove
 #include <cassert>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -66,50 +62,55 @@ EXIV2_RCSID("@(#) $Id: image.cpp 563 2005-04-21 07:21:53Z ahuggel $");
 // class member definitions
 namespace Exiv2 {
 
-    ImageFactory::Registry* ImageFactory::registry_ = 0;
-
-    void ImageFactory::init()
+    bool Image::supportsMetadata(MetadataId metadataId) const
     {
-        if (0 == registry_) {
-            registry_ = new Registry;
+        return (supportedMetadata_ & metadataId) != 0;
+    }
+
+    const ImageFactory::Registry* ImageFactory::find(int imageType)
+    {
+        for (unsigned int i = 0; registry_[i].imageType_ != ImageType::none; ++i) {
+            if (registry_[i].imageType_ == imageType) return &registry_[i];
         }
+        return 0;
     }
 
-    void ImageFactory::registerImage(Image::Type type, 
-                NewInstanceFct newInst, IsThisTypeFct isType)
+    void ImageFactory::registerImage(int            type,
+                                     NewInstanceFct newInst,
+                                     IsThisTypeFct  isType)
     {
-        init();
-        assert (newInst && isType);
-        (*registry_)[type] = ImageFcts(newInst, isType);
+        unsigned int i = 0;
+        for (; i < MAX_IMAGE_FORMATS; ++i) {
+            if (registry_[i].imageType_ == ImageType::none) {
+                registry_[i] = Registry(type, newInst, isType);
+                break;
+            }
+        }
+        if (i == MAX_IMAGE_FORMATS) throw Error(35);
     }
 
-    Image::Type ImageFactory::getType(const std::string& path)
+    int ImageFactory::getType(const std::string& path)
     {
         FileIo fileIo(path);
         return getType(fileIo);
     }
 
-    Image::Type ImageFactory::getType(const byte* data, long size)
+    int ImageFactory::getType(const byte* data, long size)
     {
         MemIo memIo(data, size);
         return getType(memIo);
     }
 
-    Image::Type ImageFactory::getType(BasicIo& io)
+    int ImageFactory::getType(BasicIo& io)
     {
-        if (io.open() != 0) return Image::none; 
+        if (io.open() != 0) return ImageType::none;
         IoCloser closer(io);
-        Image::Type type = Image::none;
-        Registry::const_iterator b = registry_->begin();
-        Registry::const_iterator e = registry_->end();
-        for (Registry::const_iterator i = b; i != e; ++i)
-        {
-            if (i->second.isThisType(io, false)) {
-                type = i->first;
-                break;
+        for (unsigned int i = 0; registry_[i].imageType_ != ImageType::none; ++i) {
+            if (registry_[i].isThisType_(io, false)) {
+                return registry_[i].imageType_;
             }
         }
-        return type;
+        return ImageType::none;
     } // ImageFactory::getType
 
     Image::AutoPtr ImageFactory::open(const std::string& path)
@@ -133,19 +134,15 @@ namespace Exiv2 {
         if (io->open() != 0) {
             throw Error(9, io->path(), strError());
         }
-        Image::AutoPtr image;
-        Registry::const_iterator b = registry_->begin();
-        Registry::const_iterator e = registry_->end();
-        for (Registry::const_iterator i = b; i != e; ++i) {
-            if (i->second.isThisType(*io, false)) {
-                image = i->second.newInstance(io, false);
-                break;
+        for (unsigned int i = 0; registry_[i].imageType_ != ImageType::none; ++i) {
+            if (registry_[i].isThisType_(*io, false)) {
+                return registry_[i].newInstance_(io, false);
             }
         }
-        return image;
+        return Image::AutoPtr();
     } // ImageFactory::open
 
-    Image::AutoPtr ImageFactory::create(Image::Type type, 
+    Image::AutoPtr ImageFactory::create(int type,
                                         const std::string& path)
     {
         std::auto_ptr<FileIo> fileIo(new FileIo(path));
@@ -160,7 +157,7 @@ namespace Exiv2 {
         return image;
     }
 
-    Image::AutoPtr ImageFactory::create(Image::Type type)
+    Image::AutoPtr ImageFactory::create(int type)
     {
         BasicIo::AutoPtr io(new MemIo);
         Image::AutoPtr image = create(type, io);
@@ -168,18 +165,18 @@ namespace Exiv2 {
         return image;
     }
 
-    Image::AutoPtr ImageFactory::create(Image::Type type, 
+    Image::AutoPtr ImageFactory::create(int type,
                                         BasicIo::AutoPtr io)
     {
         // BasicIo instance does not need to be open
-        Registry::const_iterator i = registry_->find(type);
-        if (i != registry_->end()) {
-            return i->second.newInstance(io, true);
+        const Registry* r = find(type);
+        if (0 != r) {
+            return r->newInstance_(io, true);
         }
         return Image::AutoPtr();
     } // ImageFactory::create
 
-    TiffHeader::TiffHeader(ByteOrder byteOrder) 
+    TiffHeader::TiffHeader(ByteOrder byteOrder)
         : byteOrder_(byteOrder), tag_(0x002a), offset_(0x00000008)
     {
     }
@@ -220,4 +217,20 @@ namespace Exiv2 {
         return size();
     } // TiffHeader::copy
 
+// *****************************************************************************
+// template, inline and free functions
+
+    void append(Blob& blob, const byte* buf, uint32_t len)
+    {
+        if (len != 0) {
+            assert(buf != 0);
+            Blob::size_type size = blob.size();
+            if (blob.capacity() - size < len) {
+                blob.reserve(size + 65536);
+            }
+            blob.resize(size + len);
+            std::memcpy(&blob[size], buf, len);
+        }
+    } // append
+
 }                                       // namespace Exiv2
