@@ -1,30 +1,30 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
- * 
+ * Copyright (C) 2004, 2005, 2006 Andreas Huggel <ahuggel@gmx.net>
+ *
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA.
  */
 /*!
   @file    image.hpp
   @brief   Class JpegImage to access JPEG images
-  @version $Rev: 563 $
+  @version $Rev: 705 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
-  @author  Brad Schick (brad) 
+  @author  Brad Schick (brad)
            <a href="mailto:brad@robotbattle.com">brad@robotbattle.com</a>
   @date    09-Jan-04, ahu: created<BR>
            11-Feb-04, ahu: isolated as a component<BR>
@@ -41,7 +41,7 @@
 
 // + standard includes
 #include <string>
-#include <map>
+#include <vector>
 
 // *****************************************************************************
 // namespace extensions
@@ -52,21 +52,30 @@ namespace Exiv2 {
     class ExifData;
     class IptcData;
 
+// *****************************************************************************
+// type definitions
+
+    //! Container for binary image
+    typedef std::vector<byte> Blob;
+
 // *****************************************************************************
 // class definitions
+
+    //! Supported image formats
+    namespace ImageType {
+        const int none = 0;         //!< Not an image
+    }
+
     /*!
       @brief Abstract base class defining the interface for an image. This is
          the top-level interface to the Exiv2 library.
 
       Most client apps will obtain an Image instance by calling a static
       ImageFactory method. The Image class can then be used to to
-      read, write, and save metadata.      
+      read, write, and save metadata.
      */
     class Image {
     public:
-        //! Supported image formats
-        enum Type { none, jpeg, exv };
-     
         //! Image auto_ptr type
         typedef std::auto_ptr<Image> AutoPtr;
 
@@ -79,13 +88,21 @@ namespace Exiv2 {
         //! @name Manipulators
         //@{
         /*!
-          @brief Read metadata from assigned image. Before this method
-              is called, the various metadata types (Iptc, Exif) will be empty.
-          @throw Error In case of failure.
+          @brief Read all metadata supported by a specific image format from the
+              image. Before this method is called, the various metadata types
+              will be empty.
+
+          This method returns success even if no metadata is found in the
+          image. Callers must therefore check the size of individual metadata
+          types before accessing the data.
+
+          @throw Error if opening or reading of the file fails or the image
+              data is not valid (does not look like data of the specific image
+              type).
          */
         virtual void readMetadata() =0;
         /*!
-          @brief Write metadata back to the image. 
+          @brief Write metadata back to the image.
 
           All existing metadata sections in the image are either created,
           replaced, or erased. If values for a given metadata type have been
@@ -93,7 +110,7 @@ namespace Exiv2 {
           replaced. If no values have been assigned to a given metadata type,
           any exists section for that metadata type will be removed from the
           image.
-          
+
           @throw Error if the operation fails
          */
         virtual void writeMetadata() =0;
@@ -147,57 +164,57 @@ namespace Exiv2 {
         //! @name Accessors
         //@{
         /*!
-          @brief Check if the Image instance is valid. Use after object 
+          @brief Check if the Image instance is valid. Use after object
                  construction.
           @return true if the Image is in a valid state.
          */
         virtual bool good() const =0;
         /*!
           @brief Returns an ExifData instance containing currently buffered
-              exif data.
+              Exif data.
 
-          The exif data may have been read from the image by
-          a previous call to readMetadata() or added directly. The exif
+          The Exif data may have been read from the image by
+          a previous call to readMetadata() or added directly. The Exif
           data in the returned instance will be written to the image when
           writeMetadata() is called.
-          
-          @return read only ExifData instance containing exif values
+
+          @return read only ExifData instance containing Exif values
          */
         virtual const ExifData& exifData() const =0;
         /*!
           @brief Returns an ExifData instance containing currently buffered
-              exif data.
+              Exif data.
 
-          The contained exif data may have been read from the image by
-          a previous call to readMetadata() or added directly. The exif
+          The contained Exif data may have been read from the image by
+          a previous call to readMetadata() or added directly. The Exif
           data in the returned instance will be written to the image when
           writeMetadata() is called.
-          
-          @return modifiable ExifData instance containing exif values
+
+          @return modifiable ExifData instance containing Exif values
          */
         virtual ExifData& exifData() =0;
         /*!
           @brief Returns an IptcData instance containing currently buffered
-              iptc data.
+              Iptc data.
 
-          The contained iptc data may have been read from the image by
-          a previous call to readMetadata() or added directly. The iptc
+          The contained Iptc data may have been read from the image by
+          a previous call to readMetadata() or added directly. The Iptc
           data in the returned instance will be written to the image when
           writeMetadata() is called.
-          
-          @return modifiable IptcData instance containing iptc values
+
+          @return modifiable IptcData instance containing Iptc values
          */
         virtual const IptcData& iptcData() const =0;
         /*!
-          @brief Returns an ExifData instance containing currently buffered
-              exif data.
+          @brief Returns an IptcData instance containing currently buffered
+              Iptc data.
 
-          The contained iptc data may have been read from the image by
-          a previous call to readMetadata() or added directly. The iptc
+          The contained Iptc data may have been read from the image by
+          a previous call to readMetadata() or added directly. The Iptc
           data in the returned instance will be written to the image when
           writeMetadata() is called.
-          
-          @return modifiable IptcData instance containing iptc values
+
+          @return modifiable IptcData instance containing Iptc values
          */
         virtual IptcData& iptcData() =0;
         /*!
@@ -206,12 +223,12 @@ namespace Exiv2 {
         virtual std::string comment() const =0;
         /*!
           @brief Return a reference to the BasicIo instance being used for Io.
-          
+
           This refence is particularly useful to reading the results of
           operations on a MemIo instance. For example after metadata has
           been modified and the writeMetadata() method has been called,
-          this method can be used to get access to the modified image. 
-          
+          this method can be used to get access to the modified image.
+
           @return BasicIo instance that can be used to read or write image
              data directly.
           @note If the returned BasicIo is used to write to the image, the
@@ -219,13 +236,18 @@ namespace Exiv2 {
              method is called.
          */
         virtual BasicIo& io() const = 0;
+        /*!
+          @brief Check if image supports a particular type of metadata.
+         */
+        bool supportsMetadata(MetadataId metadataId) const;
         //@}
 
     protected:
         //! @name Creators
         //@{
-        //! Default Constructor
-        Image() {}
+        //! Constructor taking a bitmap of the metadata types, which are supported
+        Image(uint16_t supportedMetadata)
+            : supportedMetadata_(supportedMetadata) {}
         //@}
 
     private:
@@ -235,6 +257,9 @@ namespace Exiv2 {
         //! Assignment operator
         Image& operator=(const Image& rhs);
 
+        // DATA
+        uint16_t supportedMetadata_; //!< Bitmap with all supported metadata types
+
     }; // class Image
 
     //! Type for function pointer that creates new Image instances
@@ -245,8 +270,7 @@ namespace Exiv2 {
     /*!
       @brief Returns an Image instance of the specified type.
 
-      The factory is implemented as a singleton, which can be accessed
-      through static member functions.
+      The factory is implemented as a static class.
     */
     class ImageFactory {
     public:
@@ -264,8 +288,8 @@ namespace Exiv2 {
           @param newInst Function pointer for creating image instances.
           @param isType Function pointer to test for matching image types.
         */
-        static void registerImage(Image::Type type, 
-                                  NewInstanceFct newInst, 
+        static void registerImage(int type,
+                                  NewInstanceFct newInst,
                                   IsThisTypeFct isType);
         //@}
 
@@ -274,11 +298,11 @@ namespace Exiv2 {
         /*!
           @brief Create an Image subclass of the appropriate type by reading
               the specified file. %Image type is derived from the file
-              contents. 
+              contents.
           @param  path %Image file. The contents of the file are tested to
               determine the image type. File extension is ignored.
-          @return An auto-pointer that owns an Image instance whose type 
-              matches that of the file. 
+          @return An auto-pointer that owns an Image instance whose type
+              matches that of the file.
           @throw Error If opening the file fails or it contains data of an
               unknown image type.
          */
@@ -286,11 +310,11 @@ namespace Exiv2 {
         /*!
           @brief Create an Image subclass of the appropriate type by reading
               the provided memory. %Image type is derived from the memory
-              contents. 
+              contents.
           @param data Pointer to a data buffer containing an image. The contents
               of the memory are tested to determine the image type.
           @param size Number of bytes pointed to by \em data.
-          @return An auto-pointer that owns an Image instance whose type 
+          @return An auto-pointer that owns an Image instance whose type
               matches that of the data buffer.
           @throw Error If the memory contains data of an unknown image type.
          */
@@ -299,15 +323,15 @@ namespace Exiv2 {
           @brief Create an Image subclass of the appropriate type by reading
               the provided BasicIo instance. %Image type is derived from the
               data provided by \em io. The passed in \em io instance is
-              (re)opened by this method. 
+              (re)opened by this method.
           @param io An auto-pointer that owns a BasicIo instance that provides
               image data. The contents of the image data are tested to determine
-              the type. 
+              the type.
           @note This method takes ownership of the passed
               in BasicIo instance through the auto-pointer. Callers should not
               continue to use the BasicIo instance after it is passed to this method.
               Use the Image::io() method to get a temporary reference.
-          @return An auto-pointer that owns an Image instance whose type 
+          @return An auto-pointer that owns an Image instance whose type
               matches that of the \em io data. If no image type could be
               determined, the pointer is 0.
           @throw Error If opening the BasicIo fails
@@ -319,49 +343,49 @@ namespace Exiv2 {
           @param type Type of the image to be created.
           @param path %Image file to create. File extension is ignored.
           @return An auto-pointer that owns an Image instance of the requested
-              type. 
+              type.
           @throw Error If the image type is not supported.
          */
-        static Image::AutoPtr create(Image::Type type, const std::string& path);
+        static Image::AutoPtr create(int type, const std::string& path);
         /*!
           @brief Create an Image subclass of the requested type by creating a
               new image in memory.
           @param type Type of the image to be created.
           @return An auto-pointer that owns an Image instance of the requested
-              type. 
+              type.
           @throw Error If the image type is not supported
          */
-        static Image::AutoPtr create(Image::Type type);
+        static Image::AutoPtr create(int type);
         /*!
           @brief Create an Image subclass of the requested type by writing a
               new image to a BasicIo instance. If the BasicIo instance already
               contains data, it will be overwritten.
           @param type Type of the image to be created.
           @param io An auto-pointer that owns a BasicIo instance that will
-              be written to when creating a new image. 
+              be written to when creating a new image.
           @note This method takes ownership of the passed in BasicIo instance
-              through the auto-pointer. Callers should not continue to use the 
-              BasicIo instance after it is passed to this method.  Use the 
+              through the auto-pointer. Callers should not continue to use the
+              BasicIo instance after it is passed to this method.  Use the
               Image::io() method to get a temporary reference.
           @return An auto-pointer that owns an Image instance of the requested
               type. If the image type is not supported, the pointer is 0.
          */
-        static Image::AutoPtr create(Image::Type type, BasicIo::AutoPtr io);
+        static Image::AutoPtr create(int type, BasicIo::AutoPtr io);
         /*!
-          @brief Returns the image type of the provided file. 
+          @brief Returns the image type of the provided file.
           @param path %Image file. The contents of the file are tested to
               determine the image type. File extension is ignored.
           @return %Image type or Image::none if the type is not recognized.
          */
-        static Image::Type getType(const std::string& path);
+        static int getType(const std::string& path);
         /*!
-          @brief Returns the image type of the provided data buffer. 
+          @brief Returns the image type of the provided data buffer.
           @param data Pointer to a data buffer containing an image. The contents
               of the memory are tested to determine the image type.
           @param size Number of bytes pointed to by \em data.
           @return %Image type or Image::none if the type is not recognized.
          */
-        static Image::Type getType(const byte* data, long size);
+        static int getType(const byte* data, long size);
         /*!
           @brief Returns the image type of data provided by a BasicIo instance.
               The passed in \em io instance is (re)opened by this method.
@@ -369,38 +393,48 @@ namespace Exiv2 {
               of the image data are tested to determine the type.
           @return %Image type or Image::none if the type is not recognized.
          */
-        static Image::Type getType(BasicIo& io);
+        static int getType(BasicIo& io);
         //@}
 
-
     private:
         //! @name Creators
         //@{
-        //! Prevent construction other than through instance().
+        //! Prevent construction: not implemented.
         ImageFactory();
         //! Prevent copy construction: not implemented.
         ImageFactory(const ImageFactory& rhs);
-        //! Creates the private static instance
-        static void init();
         //@}
 
-        //! Struct for storing image function pointers.
-        struct ImageFcts
+        //! Struct for storing image types and function pointers.
+        struct Registry
         {
-            NewInstanceFct newInstance;
-            IsThisTypeFct isThisType;
-            ImageFcts(NewInstanceFct newInst, IsThisTypeFct isType) 
-                : newInstance(newInst), isThisType(isType) {}
-            ImageFcts() : newInstance(0), isThisType(0) {}
+            //! Default constructor
+            Registry()
+                : imageType_(ImageType::none),
+                  newInstance_(0),
+                  isThisType_(0)
+                {}
+            //! Constructor
+            Registry(int            imageType,
+                     NewInstanceFct newInstance,
+                     IsThisTypeFct  isThisType)
+                : imageType_(imageType),
+                  newInstance_(newInstance),
+                  isThisType_(isThisType)
+                {}
+            int            imageType_;
+            NewInstanceFct newInstance_;
+            IsThisTypeFct  isThisType_;
         };
 
-        // DATA
-        //! Type used to store Image creation functions
-        typedef std::map<Image::Type, ImageFcts> Registry;
-        //! List of image types and corresponding creation functions.
-        static Registry* registry_;
-    }; // class ImageFactory
+        //! Return the registry entry for type
+        static const Registry* find(int imageType);
 
+        // DATA
+        static const unsigned int MAX_IMAGE_FORMATS = 32;
+        //! List of image types and corresponding creation functions.
+        static Registry registry_[MAX_IMAGE_FORMATS];
+    }; // class ImageFactory
 
     //! Helper class modelling the TIFF header structure.
     class TiffHeader {
@@ -408,7 +442,7 @@ namespace Exiv2 {
         //! @name Creators
         //@{
         /*!
-          @brief Default constructor. Optionally sets the byte order 
+          @brief Default constructor. Optionally sets the byte order
                  (default: little endian).
          */
         explicit TiffHeader(ByteOrder byteOrder =littleEndian);
@@ -422,7 +456,7 @@ namespace Exiv2 {
 
         //! @name Accessors
         //@{
-        /*! 
+        /*!
           @brief Write a standard TIFF header into buf as a data string, return
                  number of bytes copied.
 
@@ -443,7 +477,7 @@ namespace Exiv2 {
         uint16_t tag() const { return tag_; }
         /*!
           @brief Return the offset to IFD0 from the start of the TIFF header.
-                 The offset is 0x00000008 if IFD0 begins immediately after the 
+                 The offset is 0x00000008 if IFD0 begins immediately after the
                  TIFF header.
          */
         uint32_t offset() const { return offset_; }
@@ -454,7 +488,13 @@ namespace Exiv2 {
         uint16_t tag_;
         uint32_t offset_;
 
-    }; // class TiffHeader   
+    }; // class TiffHeader
+
+// *****************************************************************************
+// template, inline and free functions
+
+    //! Append \em len bytes pointed to by \em buf to \em blob.
+    void append(Blob& blob, const byte* buf, uint32_t len);
 
 }                                       // namespace Exiv2
 
