@@ -1,35 +1,32 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
- * 
+ * Copyright (C) 2004, 2005, 2006 Andreas Huggel <ahuggel@gmx.net>
+ *
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA.
  */
 /*
   File:      iptc.cpp
-  Version:   $Rev: 560 $
+  Version:   $Rev: 801 $
   Author(s): Brad Schick (brad) <brad@robotbattle.com>
   History:   31-July-04, brad: created
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: iptc.cpp 560 2005-04-17 11:51:32Z ahuggel $");
-
-// Define DEBUG_MAKERNOTE to output debug information to std::cerr
-#undef DEBUG_MAKERNOTE
+EXIV2_RCSID("@(#) $Id: iptc.cpp 801 2006-05-27 09:08:56Z ahuggel $");
 
 // *****************************************************************************
 // included header files
@@ -48,7 +45,7 @@ EXIV2_RCSID("@(#) $Id: iptc.cpp 560 2005-04-17 11:51:32Z ahuggel $");
 // class member definitions
 namespace Exiv2 {
 
-    Iptcdatum::Iptcdatum(const IptcKey& key, 
+    Iptcdatum::Iptcdatum(const IptcKey& key,
                          const Value* pValue)
         : key_(key.clone())
     {
@@ -66,6 +63,12 @@ namespace Exiv2 {
     {
     }
 
+    const Value& Iptcdatum::value() const
+    {
+        if (value_.get() == 0) throw Error(8);
+        return *value_;
+    }
+
     Iptcdatum& Iptcdatum::operator=(const Iptcdatum& rhs)
     {
         if (this == &rhs) return *this;
@@ -79,7 +82,7 @@ namespace Exiv2 {
 
         return *this;
     } // Iptcdatum::operator=
-    
+
     Iptcdatum& Iptcdatum::operator=(const uint16_t& value)
     {
         UShortValue::AutoPtr v = UShortValue::AutoPtr(new UShortValue);
@@ -133,17 +136,19 @@ namespace Exiv2 {
         const byte* pRead = buf;
         iptcMetadata_.clear();
 
-        int rc = 0;
         uint16_t record = 0;
         uint16_t dataSet = 0;
         uint32_t sizeData = 0;
         byte extTest = 0;
 
         while (pRead + 3 < buf + len) {
-            if (*pRead++ != marker_) return 5;
+            // First byte should be a marker. If it isn't, scan forward and skip
+            // the chunk bytes present in some images. This deviates from the
+            // standard, which advises to treat such cases as errors.
+            if (*pRead++ != marker_) continue;
             record = *pRead++;
             dataSet = *pRead++;
-            
+
             extTest = *pRead;
             if (extTest & 0x80) {
                 // extended dataset
@@ -160,27 +165,28 @@ namespace Exiv2 {
                 sizeData = getUShort(pRead, bigEndian);
                 pRead += 2;
             }
-            rc = readData(dataSet, record, pRead, sizeData);
-            if( rc ) return rc;
+            readData(dataSet, record, pRead, sizeData);
             pRead += sizeData;
         }
 
-        return rc;
+        return 0;
     } // IptcData::read
 
-    int IptcData::readData(uint16_t dataSet, uint16_t record, 
+    int IptcData::readData(uint16_t dataSet, uint16_t record,
                            const byte* data, uint32_t sizeData)
     {
         Value::AutoPtr value;
         TypeId type = IptcDataSets::dataSetType(dataSet, record);
         value = Value::create(type);
-        value->read(data, sizeData, bigEndian);
-        IptcKey key(dataSet, record);
-        add(key, value.get());
-        return 0;
+        int rc = value->read(data, sizeData, bigEndian);
+        if (0 == rc) {
+            IptcKey key(dataSet, record);
+            add(key, value.get());
+        }
+        return rc;
     }
 
-    DataBuf IptcData::copy()
+    DataBuf IptcData::copy() const
     {
         DataBuf buf(size());
         byte *pWrite = buf.pData_;
@@ -212,7 +218,7 @@ namespace Exiv2 {
         }
 
         return buf;
-    } // IptcData::updateBuffer
+    } // IptcData::copy
 
     long IptcData::size() const
     {
@@ -240,7 +246,7 @@ namespace Exiv2 {
     int IptcData::add(const Iptcdatum& iptcDatum)
     {
         if (!IptcDataSets::dataSetRepeatable(
-               iptcDatum.tag(), iptcDatum.record()) && 
+               iptcDatum.tag(), iptcDatum.record()) &&
                findId(iptcDatum.tag(), iptcDatum.record()) != end()) {
              return 6;
         }
