@@ -1,34 +1,34 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
- * 
+ * Copyright (C) 2004, 2005, 2006 Andreas Huggel <ahuggel@gmx.net>
+ *
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA.
  */
 /*
   File:      jpgimage.cpp
-  Version:   $Rev: 563 $
+  Version:   $Rev: 812 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
              Brad Schick (brad) <brad@robotbattle.com>
   History:   15-Jan-05, brad: split out from image.cpp
-             
+
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: jpgimage.cpp 563 2005-04-21 07:21:53Z ahuggel $");
+EXIV2_RCSID("@(#) $Id: jpgimage.cpp 812 2006-06-02 15:29:31Z ahuggel $");
 
 // *****************************************************************************
 // included header files
@@ -48,42 +48,137 @@ EXIV2_RCSID("@(#) $Id: jpgimage.cpp 563 2005-04-21 07:21:53Z ahuggel $");
 
 // *****************************************************************************
 // class member definitions
+
 namespace Exiv2 {
 
-    // Local functions. These could be static private functions on Image
-    // subclasses but then ImageFactory needs to be made a friend. 
-    /*!
-      @brief Create a new ExvImage instance and return an auto-pointer to it. 
-             Caller owns the returned object and the auto-pointer ensures that 
-             it will be deleted.
-     */
-    Image::AutoPtr newExvInstance(BasicIo::AutoPtr io, bool create);
-    //! Check if the file iIo is an EXV file
-    bool isExvType(BasicIo& iIo, bool advance);
-    /*!
-      @brief Create a new JpegImage instance and return an auto-pointer to it.
-             Caller owns the returned object and the auto-pointer ensures that 
-             it will be deleted.
-     */
-    Image::AutoPtr newJpegInstance(BasicIo::AutoPtr io, bool create);
-    //! Check if the file iIo is a JPEG image.
-    bool isJpegType(BasicIo& iIo, bool advance);
- 
-    const byte JpegBase::sos_    = 0xda;
-    const byte JpegBase::eoi_    = 0xd9;
-    const byte JpegBase::app0_   = 0xe0;
-    const byte JpegBase::app1_   = 0xe1;
-    const byte JpegBase::app13_  = 0xed;
-    const byte JpegBase::com_    = 0xfe;
-    const uint16_t JpegBase::iptc_ = 0x0404;
-    const char JpegBase::exifId_[] = "Exif\0\0";
-    const char JpegBase::jfifId_[] = "JFIF\0";
-    const char JpegBase::ps3Id_[]  = "Photoshop 3.0\0";
-    const char JpegBase::bimId_[]  = "8BIM";
+    const byte     JpegBase::sos_      = 0xda;
+    const byte     JpegBase::eoi_      = 0xd9;
+    const byte     JpegBase::app0_     = 0xe0;
+    const byte     JpegBase::app1_     = 0xe1;
+    const byte     JpegBase::app13_    = 0xed;
+    const byte     JpegBase::com_      = 0xfe;
+    const char     JpegBase::exifId_[] = "Exif\0\0";
+    const char     JpegBase::jfifId_[] = "JFIF\0";
 
-    JpegBase::JpegBase(BasicIo::AutoPtr io, bool create, 
-                       const byte initData[], long dataSize) 
-        : io_(io)
+    const char     Photoshop::ps3Id_[] = "Photoshop 3.0\0";
+    const char     Photoshop::bimId_[] = "8BIM";
+    const uint16_t Photoshop::iptc_    = 0x0404;
+
+    // Todo: Generalised from JpegBase::locateIptcData without really understanding
+    //       the format (in particular the header). So it remains to be confirmed
+    //       if this also makes sense for psTag != Photoshop::iptc
+    int Photoshop::locateIrb(const byte*     pPsData,
+                             long            sizePsData,
+                             uint16_t        psTag,
+                             const byte**    record,
+                             uint16_t *const sizeHdr,
+                             uint16_t *const sizeData)
+    {
+        assert(record);
+        assert(sizeHdr);
+        assert(sizeData);
+        // Used for error checking
+        long position = 0;
+
+        // Data should follow Photoshop format, if not exit
+        while (   position <= sizePsData - 14
+               && memcmp(pPsData + position, Photoshop::bimId_, 4) == 0) {
+            const byte *hrd = pPsData + position;
+            position += 4;
+            uint16_t type = getUShort(pPsData + position, bigEndian);
+            position += 2;
+
+            // Pascal string is padded to have an even size (including size byte)
+            byte psSize = pPsData[position] + 1;
+            psSize += (psSize & 1);
+            position += psSize;
+            if (position >= sizePsData) return -2;
+
+            // Data is also padded to be even
+            long dataSize = getULong(pPsData + position, bigEndian);
+            position += 4;
+            if (dataSize > sizePsData - position) return -2;
+
+            if (type == psTag) {
+                *sizeData = static_cast<uint16_t>(dataSize);
+                *sizeHdr = psSize + 10;
+                *record = hrd;
+                return 0;
+            }
+            position += dataSize + (dataSize & 1);
+        }
+        return 3;
+    } // Photoshop::locateIrb
+
+    int Photoshop::locateIptcIrb(const byte*     pPsData,
+                                 long            sizePsData,
+                                 const byte**    record,
+                                 uint16_t *const sizeHdr,
+                                 uint16_t *const sizeData)
+    {
+        return locateIrb(pPsData, sizePsData, iptc_,
+                         record, sizeHdr, sizeData);
+    }
+
+    DataBuf Photoshop::setIptcIrb(const byte*     pPsData,
+                                  long            sizePsData,
+                                  const IptcData& iptcData)
+    {
+        if (sizePsData > 0) assert(pPsData);
+#ifdef DEBUG
+        std::cerr << "IRB block at the beginning of Photoshop::setIptcIrb\n";
+        if (sizePsData == 0) std::cerr << "  None.\n";
+        else hexdump(std::cerr, pPsData, sizePsData);
+#endif
+        const byte* record    = pPsData;
+        uint16_t    sizeIptc  = 0;
+        uint16_t    sizeHdr   = 0;
+        // Safe to call with zero psData.size_
+        Photoshop::locateIptcIrb(pPsData, sizePsData,
+                                 &record, &sizeHdr, &sizeIptc);
+
+        Blob psBlob;
+        // Data is rounded to be even
+        const uint16_t sizeOldData = sizeHdr + sizeIptc + (sizeIptc & 1);
+        const uint16_t sizeFront = static_cast<uint16_t>(record - pPsData);
+        const uint16_t sizeEnd = static_cast<uint16_t>(sizePsData - sizeFront - sizeOldData);
+
+        // Write data before old record.
+        if (sizePsData > 0 && sizeFront > 0) {
+            append(psBlob, pPsData, sizeFront);
+        }
+        // Write new iptc record if we have it
+        DataBuf rawIptc(iptcData.copy());
+        if (rawIptc.size_ > 0) {
+            byte tmpBuf[12];
+            memcpy(tmpBuf, Photoshop::bimId_, 4);
+            us2Data(tmpBuf + 4, iptc_, bigEndian);
+            tmpBuf[6] = 0;
+            tmpBuf[7] = 0;
+            ul2Data(tmpBuf + 8, rawIptc.size_, bigEndian);
+            append(psBlob, tmpBuf, 12);
+            append(psBlob, rawIptc.pData_, rawIptc.size_);
+            // Data is padded to be even (but not included in size)
+            if (rawIptc.size_ & 1) psBlob.push_back(0x00);
+        }
+        // Write existing stuff after record
+        if (sizePsData > 0 && sizeEnd > 0) {
+            append(psBlob, record + sizeOldData, sizeEnd);
+        }
+        DataBuf rc;
+        if (psBlob.size() > 0) rc = DataBuf(&psBlob[0], static_cast<long>(psBlob.size()));
+#ifdef DEBUG
+        std::cerr << "IRB block at the end of Photoshop::setIptcIrb\n";
+        if (rc.size_ == 0) std::cerr << "  None.\n";
+        else hexdump(std::cerr, rc.pData_, rc.size_);
+#endif
+        return rc;
+
+    } // Photoshop::setIptcIrb
+
+    JpegBase::JpegBase(BasicIo::AutoPtr io, bool create,
+                       const byte initData[], long dataSize)
+        : Image(mdExif | mdIptc | mdComment), io_(io)
     {
         if (create) {
             initImage(initData, dataSize);
@@ -115,7 +210,7 @@ namespace Exiv2 {
         clearExifData();
         clearComment();
     }
-    
+
     void JpegBase::clearIptcData()
     {
         iptcData_.clear();
@@ -142,8 +237,8 @@ namespace Exiv2 {
     }
 
     void JpegBase::setComment(const std::string& comment)
-    { 
-        comment_ = comment; 
+    {
+        comment_ = comment;
     }
 
     void JpegBase::setMetadata(const Image& image)
@@ -160,7 +255,7 @@ namespace Exiv2 {
         while ((c=io_->getb()) != 0xff) {
             if (c == EOF) return -1;
         }
-            
+
         // Markers can start with any number of 0xff
         while ((c=io_->getb()) == 0xff) {
             if (c == EOF) return -1;
@@ -188,7 +283,7 @@ namespace Exiv2 {
         // Read section marker
         int marker = advanceToMarker();
         if (marker < 0) throw Error(15);
-        
+
         while (marker != sos_ && marker != eoi_ && search > 0) {
             // Read size and signature (ok if this hits EOF)
             bufRead = io_->read(buf.pData_, bufMinSize);
@@ -198,15 +293,15 @@ namespace Exiv2 {
             if (marker == app1_ && memcmp(buf.pData_ + 2, exifId_, 6) == 0) {
                 if (size < 8) throw Error(15);
                 // Seek to begining and read the Exif data
-                io_->seek(8-bufRead, BasicIo::cur); 
+                io_->seek(8-bufRead, BasicIo::cur);
                 long sizeExifData = size - 8;
                 DataBuf rawExif(sizeExifData);
                 io_->read(rawExif.pData_, sizeExifData);
                 if (io_->error() || io_->eof()) throw Error(14);
-                if (exifData_.load(rawExif.pData_, sizeExifData)) throw Error(15);
+                if (exifData_.load(rawExif.pData_, sizeExifData)) throw Error(36, "Exif");
                 --search;
             }
-            else if (marker == app13_ && memcmp(buf.pData_ + 2, ps3Id_, 14) == 0) {
+            else if (marker == app13_ && memcmp(buf.pData_ + 2, Photoshop::ps3Id_, 14) == 0) {
                 if (size < 16) throw Error(15);
                 // Read the rest of the APP13 segment
                 // needed if bufMinSize!=16: io_->seek(16-bufRead, BasicIo::cur);
@@ -217,10 +312,10 @@ namespace Exiv2 {
                 uint16_t sizeIptc = 0;
                 uint16_t sizeHdr = 0;
                 // Find actual Iptc data within the APP13 segment
-                if (!locateIptcData(psData.pData_, psData.size_, &record,
-                            &sizeHdr, &sizeIptc)) {
+                if (!Photoshop::locateIptcIrb(psData.pData_, psData.size_,
+                                          &record, &sizeHdr, &sizeIptc)) {
                     assert(sizeIptc);
-                    if (iptcData_.load(record + sizeHdr, sizeIptc)) throw Error(15);
+                    if (iptcData_.load(record + sizeHdr, sizeIptc)) throw Error(36, "IPTC");
                 }
                 --search;
             }
@@ -252,50 +347,6 @@ namespace Exiv2 {
         }
     } // JpegBase::readMetadata
 
-
-    // Operates on raw data (rather than file streams) to simplify reuse
-    int JpegBase::locateIptcData(const byte *pPsData, 
-                                 long sizePsData,
-                                 const byte **record, 
-                                 uint16_t *const sizeHdr,
-                                 uint16_t *const sizeIptc) const
-    {
-        assert(record);
-        assert(sizeHdr);
-        assert(sizeIptc);
-        // Used for error checking
-        long position = 0;
-
-        // Data should follow Photoshop format, if not exit
-        while (position <= (sizePsData - 14) &&
-                memcmp(pPsData + position, bimId_, 4)==0) {
-            const byte *hrd = pPsData + position;
-            position += 4;
-            uint16_t type = getUShort(pPsData+ position, bigEndian);
-            position += 2;
-           
-            // Pascal string is padded to have an even size (including size byte)
-            byte psSize = pPsData[position] + 1;
-            psSize += (psSize & 1);
-            position += psSize;
-            if (position >= sizePsData) return -2;
-
-            // Data is also padded to be even
-            long dataSize = getULong(pPsData + position, bigEndian);
-            position += 4;
-            if (dataSize > sizePsData - position) return -2;
-           
-            if (type == iptc_) {
-                *sizeIptc = static_cast<uint16_t>(dataSize);
-                *sizeHdr = psSize + 10;
-                *record = hrd;
-                return 0;
-            }
-            position += dataSize + (dataSize & 1);
-        }
-        return 3;
-    } // JpegBase::locateIptcData
-
     void JpegBase::writeMetadata()
     {
         if (io_->open() != 0) {
@@ -320,7 +371,7 @@ namespace Exiv2 {
             if (io_->error() || io_->eof()) throw Error(20);
             throw Error(22);
         }
-        
+
         const long bufMinSize = 16;
         long bufRead = 0;
         DataBuf buf(bufMinSize);
@@ -339,7 +390,7 @@ namespace Exiv2 {
         // Read section marker
         int marker = advanceToMarker();
         if (marker < 0) throw Error(22);
-        
+
         // First find segments of interest. Normally app0 is first and we want
         // to insert after it. But if app0 comes after com, app1 and app13 then
         // don't bother.
@@ -360,7 +411,7 @@ namespace Exiv2 {
                 ++search;
                 if (io_->seek(size-bufRead, BasicIo::cur)) throw Error(22);
             }
-            else if (marker == app13_ && memcmp(buf.pData_ + 2, ps3Id_, 14) == 0) {
+            else if (marker == app13_ && memcmp(buf.pData_ + 2, Photoshop::ps3Id_, 14) == 0) {
                 if (size < 16) throw Error(22);
                 skipApp13Ps3 = count;
                 ++search;
@@ -395,7 +446,7 @@ namespace Exiv2 {
         count = 0;
         marker = advanceToMarker();
         if (marker < 0) throw Error(22);
-        
+
         // To simplify this a bit, new segments are inserts at either the start
         // or right after app0. This is standard in most jpegs, but has the
         // potential to change segment ordering (which is allowed).
@@ -414,7 +465,7 @@ namespace Exiv2 {
                     // Write COM marker, size of comment, and string
                     tmpBuf[0] = 0xff;
                     tmpBuf[1] = com_;
-                    us2Data(tmpBuf + 2, 
+                    us2Data(tmpBuf + 2,
                             static_cast<uint16_t>(comment_.length()+3), bigEndian);
                     if (outIo.write(tmpBuf, 4) != 4) throw Error(21);
                     if (outIo.write((byte*)comment_.data(), (long)comment_.length())
@@ -428,67 +479,39 @@ namespace Exiv2 {
                     DataBuf rawExif(exifData_.copy());
                     tmpBuf[0] = 0xff;
                     tmpBuf[1] = app1_;
-                    us2Data(tmpBuf + 2, 
-                            static_cast<uint16_t>(rawExif.size_+8), 
+                    us2Data(tmpBuf + 2,
+                            static_cast<uint16_t>(rawExif.size_+8),
                             bigEndian);
                     memcpy(tmpBuf + 4, exifId_, 6);
                     if (outIo.write(tmpBuf, 10) != 10) throw Error(21);
-                    if (outIo.write(rawExif.pData_, rawExif.size_) 
+                    if (outIo.write(rawExif.pData_, rawExif.size_)
                         != rawExif.size_) throw Error(21);
                     if (outIo.error()) throw Error(21);
                     --search;
                 }
-                
-                const byte *record = psData.pData_;
-                uint16_t sizeIptc = 0;
-                uint16_t sizeHdr = 0;
-                // Safe to call with zero psData.size_
-                locateIptcData(psData.pData_, psData.size_, &record, &sizeHdr, &sizeIptc);
-
-                // Data is rounded to be even
-                const int sizeOldData = sizeHdr + sizeIptc + (sizeIptc & 1);
-                if (psData.size_ > sizeOldData || iptcData_.count() > 0) {
-                    // rawIptc may have size of zero.
-                    DataBuf rawIptc(iptcData_.copy());
-                    // write app13 marker, new size, and ps3Id
-                    tmpBuf[0] = 0xff;
-                    tmpBuf[1] = app13_;
-                    const int sizeNewData = rawIptc.size_ ? 
-                            rawIptc.size_ + (rawIptc.size_ & 1) + 12 : 0;
-                    us2Data(tmpBuf + 2, 
-                            static_cast<uint16_t>(psData.size_-sizeOldData+sizeNewData+16),
-                            bigEndian);
-                    memcpy(tmpBuf + 4, ps3Id_, 14);
-                    if (outIo.write(tmpBuf, 18) != 18) throw Error(21);
-                    if (outIo.error()) throw Error(21);
-
-                    const long sizeFront = (long)(record - psData.pData_);
-                    const long sizeEnd = psData.size_ - sizeFront - sizeOldData;
-                    // write data before old record.
-                    if (outIo.write(psData.pData_, sizeFront) != sizeFront) throw Error(21);
-
-                    // write new iptc record if we have it
-                    if (iptcData_.count() > 0) {
-                        memcpy(tmpBuf, bimId_, 4);
-                        us2Data(tmpBuf+4, iptc_, bigEndian);
-                        tmpBuf[6] = 0;
-                        tmpBuf[7] = 0;
-                        ul2Data(tmpBuf + 8, rawIptc.size_, bigEndian);
-                        if (outIo.write(tmpBuf, 12) != 12) throw Error(21);
-                        if (outIo.write(rawIptc.pData_, rawIptc.size_) 
-                            != rawIptc.size_) throw Error(21);
-                        // data is padded to be even (but not included in size)
-                        if (rawIptc.size_ & 1) {
-                            if (outIo.putb(0)==EOF) throw Error(21);
-                        }
+                if (psData.size_ > 0 || iptcData_.count() > 0) {
+                    // Set the new IPTC IRB, keeps existing IRBs but removes the
+                    // IPTC block if there is no new IPTC data to write
+                    DataBuf newPsData = Photoshop::setIptcIrb(psData.pData_,
+                                                              psData.size_,
+                                                              iptcData_);
+                    if (newPsData.size_ > 0) {
+                        // Write APP13 marker, new size, and ps3Id
+                        tmpBuf[0] = 0xff;
+                        tmpBuf[1] = app13_;
+                        us2Data(tmpBuf + 2, static_cast<uint16_t>(newPsData.size_ + 16), bigEndian);
+                        memcpy(tmpBuf + 4, Photoshop::ps3Id_, 14);
+                        if (outIo.write(tmpBuf, 18) != 18) throw Error(21);
                         if (outIo.error()) throw Error(21);
+
+                        // Write new Photoshop IRB data buffer
+                        if (   outIo.write(newPsData.pData_, newPsData.size_)
+                               != newPsData.size_) throw Error(21);
+                        if (outIo.error()) throw Error(21);
+                    }
+                    if (iptcData_.count() > 0) {
                         --search;
                     }
-                    
-                    // write existing stuff after record
-                    if (outIo.write(record+sizeOldData, sizeEnd) 
-                        != sizeEnd) throw Error(21);
-                    if (outIo.error()) throw Error(21);
                 }
             }
             if (marker == eoi_) {
@@ -525,7 +548,6 @@ namespace Exiv2 {
 
     } // JpegBase::doWriteMetadata
 
-
     const byte JpegImage::soi_ = 0xd8;
     const byte JpegImage::blank_[] = {
         0xFF,0xD8,0xFF,0xDB,0x00,0x84,0x00,0x10,0x0B,0x0B,0x0B,0x0C,0x0B,0x10,0x0C,0x0C,
@@ -545,19 +567,11 @@ namespace Exiv2 {
         0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xFF,0xDA,0x00,0x0C,0x03,0x01,0x00,0x02,
         0x11,0x03,0x11,0x00,0x3F,0x00,0xA0,0x00,0x0F,0xFF,0xD9 };
 
-    JpegImage::JpegImage(BasicIo::AutoPtr io, bool create) 
+    JpegImage::JpegImage(BasicIo::AutoPtr io, bool create)
         : JpegBase(io, create, blank_, sizeof(blank_))
     {
     }
 
-    //! @cond IGNORE
-    JpegImage::JpegRegister::JpegRegister()
-    {
-        ImageFactory::registerImage(
-            Image::jpeg, newJpegInstance, isJpegType);
-    }
-    //! @endcond
-  
     int JpegImage::writeHeader(BasicIo& outIo) const
     {
         // Jpeg header
@@ -596,23 +610,15 @@ namespace Exiv2 {
         if (!advance || !result ) iIo.seek(-2, BasicIo::cur);
         return result;
     }
-   
+
     const char ExvImage::exiv2Id_[] = "Exiv2";
     const byte ExvImage::blank_[] = { 0xff,0x01,'E','x','i','v','2',0xff,0xd9 };
 
-    ExvImage::ExvImage(BasicIo::AutoPtr io, bool create) 
+    ExvImage::ExvImage(BasicIo::AutoPtr io, bool create)
         : JpegBase(io, create, blank_, sizeof(blank_))
     {
     }
 
-    //! @cond IGNORE
-    ExvImage::ExvRegister::ExvRegister()
-    {
-        ImageFactory::registerImage(
-            Image::exv, newExvInstance, isExvType);
-    }
-    //! @endcond
-
     int ExvImage::writeHeader(BasicIo& outIo) const
     {
         // Exv header
@@ -650,7 +656,7 @@ namespace Exiv2 {
         iIo.read(tmpBuf, 7);
         if (iIo.error() || iIo.eof()) return false;
 
-        if (   0xff != tmpBuf[0] || 0x01 != tmpBuf[1] 
+        if (   0xff != tmpBuf[0] || 0x01 != tmpBuf[1]
             || memcmp(tmpBuf + 2, ExvImage::exiv2Id_, 5) != 0) {
             result = false;
         }
