@@ -1,30 +1,30 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
- * 
+ * Copyright (C) 2004, 2005, 2006 Andreas Huggel <ahuggel@gmx.net>
+ *
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA.
  */
 /*!
   @file    jpgimage.hpp
   @brief   Class JpegImage to access JPEG images
-  @version $Rev: 563 $
+  @version $Rev: 808 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
-  @author  Brad Schick (brad) 
+  @author  Brad Schick (brad)
            <a href="mailto:brad@robotbattle.com">brad@robotbattle.com</a>
   @date    15-Jan-05, brad: split out from image.cpp
  */
@@ -50,7 +50,69 @@ namespace Exiv2 {
 // *****************************************************************************
 // class definitions
 
-    /*! 
+    // Supported jpeg image formats
+    namespace ImageType {
+        const int jpeg = 1;         //!< Jpeg image type (see class JpegImage)
+        const int exv  = 2;         //!< Exv image type (see class ExvImage)
+    }
+
+    /*!
+      @brief Helper class, has methods to deal with %Photoshop "Information
+             Resource Blocks" (IRBs).
+     */
+    struct Photoshop {
+        // Todo: Public for now
+        static const char     ps3Id_[]; //!< %Photoshop marker
+        static const char     bimId_[]; //!< %Photoshop marker
+        static const uint16_t iptc_;    //!< %Photoshop IPTC marker
+
+        /*!
+          @brief Locates the data for a %Photoshop tag in a %Photoshop formated memory
+              buffer. Operates on raw data to simplify reuse.
+          @param pPsData Pointer to buffer containing entire payload of
+              %Photoshop formated data, e.g., from APP13 Jpeg segment.
+          @param sizePsData Size in bytes of pPsData.
+          @param psTag %Tag number of the block to look for.
+          @param record Output value that is set to the start of the
+              data block within pPsData (may not be null).
+          @param sizeHdr Output value that is set to the size of the header
+              within the data block pointed to by record (may not be null).
+          @param sizeData Output value that is set to the size of the actual
+              data within the data block pointed to by record (may not be null).
+          @return 0 if successful;<BR>
+                  3 if no data for psTag was found in pPsData;<BR>
+                 -2 if the pPsData buffer does not contain valid data.
+        */
+        static int locateIrb(const byte *pPsData,
+                             long sizePsData,
+                             uint16_t psTag,
+                             const byte **record,
+                             uint16_t *const sizeHdr,
+                             uint16_t *const sizeData);
+        /*!
+          @brief Forwards to locateIrb() with \em psTag = \em iptc_
+         */
+        static int locateIptcIrb(const byte *pPsData,
+                                 long sizePsData,
+                                 const byte **record,
+                                 uint16_t *const sizeHdr,
+                                 uint16_t *const sizeData);
+        /*!
+          @brief Set the new IPTC IRB, keeps existing IRBs but removes the
+                 IPTC block if there is no new IPTC data to write.
+
+          @param pPsData    Existing IRB buffer
+          @param sizePsData Size of the IRB buffer, may be 0
+          @param iptcData   Iptc data to embed, may be empty
+          @return A data buffer containing the new IRB buffer, may have 0 size
+        */
+        static DataBuf setIptcIrb(const byte*     pPsData,
+                                  long            sizePsData,
+                                  const IptcData& iptcData);
+
+    }; // class Photoshop
+
+    /*!
       @brief Abstract helper base class to access JPEG images.
      */
     class JpegBase : public Image {
@@ -62,36 +124,8 @@ namespace Exiv2 {
         //@}
         //! @name Manipulators
         //@{
-        /*!
-          @brief Read all metadata from the image. Before this method
-              is called, the various metadata types (Iptc, Exif) will be empty.
-              
-          This method returns success even when no metadata is found in
-          the image. Callers must therefore check the size of individual
-          metadata types before accessing the data.
-          
-          @throw Error if opening or reading of the file fails or the image
-              data is not valid (does not look like JPEG data).
-         */
         void readMetadata();
-        /*!
-          @brief Write metadata back to the image. 
-
-          All existing metadata sections in the image are either created,
-          replaced, or erased. If values for a given metadata type have been
-          assigned, a section for that metadata type will either be created or
-          replaced. If no values have been assigned to a given metadata type,
-          any exists section for that metadata type will be removed from the
-          image.
-          
-          @throw Error if the operation fails
-         */
         void writeMetadata();
-        /*!
-          @brief Assign new exif data. The new exif data is not written
-             to the image until the writeMetadata() method is called.
-          @param exifData An ExifData instance holding exif data to be copied
-         */
         void setExifData(const ExifData& exifData);
         void clearExifData();
         void setIptcData(const IptcData& iptcData);
@@ -111,11 +145,11 @@ namespace Exiv2 {
         IptcData& iptcData() { return iptcData_; }
         std::string comment() const { return comment_; }
         BasicIo& io() const { return *io_; }
-        //@}        
+        //@}
     protected:
         //! @name Creators
         //@{
-        /*! 
+        /*!
           @brief Constructor that can either open an existing image or create
               a new image from scratch. If a new image is to be created, any
               existing data is overwritten.
@@ -141,7 +175,7 @@ namespace Exiv2 {
           @brief Writes the image header (aka signature) to the BasicIo instance.
           @param oIo BasicIo instance that the header is written to.
           @return 0 if successful;<BR>
-                 4 if the output file can not be written to;<BR>
+                  4 if the output file can not be written to
          */
         virtual int writeHeader(BasicIo& oIo) const =0;
         //@}
@@ -155,16 +189,16 @@ namespace Exiv2 {
           moved (see below). This applies only if the type matches and the
           function returns true. If the type does not match, the stream
           position is not changed. However, if reading from the stream fails,
-          the stream position is undefined. Consult the stream state to obtain 
+          the stream position is undefined. Consult the stream state to obtain
           more information in this case.
-          
+
           @param iIo BasicIo instance to read from.
           @param advance Flag indicating whether the position of the io
               should be advanced by the number of characters read to
               analyse the data (true) or left at its original
               position (false). This applies only if the type matches.
           @return  true  if the data matches the type of this class;<BR>
-                   false if the data does not match;<BR>
+                   false if the data does not match
          */
         virtual bool isThisType(BasicIo& iIo, bool advance) const =0;
         //@}
@@ -178,9 +212,6 @@ namespace Exiv2 {
         static const byte com_;                 //!< JPEG Comment marker
         static const char exifId_[];            //!< Exif identifier
         static const char jfifId_[];            //!< JFIF identifier
-        static const char ps3Id_[];             //!< Photoshop marker
-        static const char bimId_[];             //!< Photoshop marker
-        static const uint16_t iptc_;              //!< Photoshop Iptc marker
 
     private:
         // DATA
@@ -196,32 +227,9 @@ namespace Exiv2 {
               when the BasicIo instance is positioned one byte past the end of a
               Jpeg segment.
           @return the next Jpeg segment marker if successful;<BR>
-                 -1 if a maker was not found before EOF;<BR>
+                 -1 if a maker was not found before EOF
          */
         int advanceToMarker() const;
-        /*!
-          @brief Locates Photoshop formated Iptc data in a memory buffer.
-              Operates on raw data to simplify reuse.
-          @param pPsData Pointer to buffer containing entire payload of 
-              Photoshop formated APP13 Jpeg segment.
-          @param sizePsData Size in bytes of pPsData.
-          @param record Output value that is set to the start of the Iptc
-              data block within pPsData (may not be null).
-          @param sizeHdr Output value that is set to the size of the header
-              within the Iptc data block pointed to by record (may not
-              be null).
-          @param sizeIptc Output value that is set to the size of the actual
-              Iptc data within the Iptc data block pointed to by record
-              (may not be null).
-          @return 0 if successful;<BR>
-                  3 if no Iptc data was found in pPsData;<BR>
-                 -2 if the pPsData buffer does not contain valid data.
-         */
-        int locateIptcData(const byte *pPsData, 
-                           long sizePsData,
-                           const byte **record, 
-                           uint16_t *const sizeHdr,
-                           uint16_t *const sizeIptc) const;
         /*!
           @brief Initialize the image with the provided data.
           @param initData Data to be written to the associated BasicIo
@@ -231,8 +239,8 @@ namespace Exiv2 {
          */
         int initImage(const byte initData[], long dataSize);
         /*!
-          @brief Provides the main implementation of writeMetadata() by 
-                writing all buffered metadata to the provided BasicIo. 
+          @brief Provides the main implementation of writeMetadata() by
+                writing all buffered metadata to the provided BasicIo.
           @param oIo BasicIo instance to write to (a temporary location).
 
           @return 4 if opening or writing to the associated BasicIo fails
@@ -248,7 +256,7 @@ namespace Exiv2 {
         JpegBase& operator=(const JpegBase& rhs);
     }; // class JpegBase
 
-    /*! 
+    /*!
       @brief Class to access JPEG images
      */
     class JpegImage : public JpegBase {
@@ -256,7 +264,7 @@ namespace Exiv2 {
     public:
         //! @name Creators
         //@{
-        /*! 
+        /*!
           @brief Constructor that can either open an existing Jpeg image or create
               a new image from scratch. If a new image is to be created, any
               existing data is overwritten. Since the constructor can not return
@@ -275,27 +283,9 @@ namespace Exiv2 {
         //! Destructor
         ~JpegImage() {}
         //@}
-        
-        //! @cond IGNORE
-        // Public only so that we can create a static instance
-        struct JpegRegister{
-            JpegRegister();
-        };
-        //! @endcond
     protected:
         //! @name Accessors
         //@{
-        /*!
-          @brief Determine if the content of the BasicIo instance is a Jpeg image.
-              See base class for more details.
-          @param iIo BasicIo instance to read from.
-          @param advance Flag indicating whether the position of the io
-              should be advanced by the number of characters read to
-              analyse the data (true) or left at its original
-              position (false). This applies only if the type matches.
-          @return  true  if the data matches a Jpeg image;<BR>
-                   false if the data does not match;<BR>
-         */
         bool isThisType(BasicIo& iIo, bool advance) const;
         //@}
         //! @name Manipulators
@@ -306,7 +296,7 @@ namespace Exiv2 {
           @return 0 if successful;<BR>
                  2 if the input image is invalid or can not be read;<BR>
                  4 if the temporary image can not be written to;<BR>
-                -3 other temporary errors;<BR>
+                -3 other temporary errors
          */
         int writeHeader(BasicIo& oIo) const;
         //@}
@@ -324,15 +314,13 @@ namespace Exiv2 {
         JpegImage& operator=(const JpegImage& rhs);
     }; // class JpegImage
 
-    static JpegImage::JpegRegister jpegReg;
-
     //! Helper class to access %Exiv2 files
     class ExvImage : public JpegBase {
         friend bool isExvType(BasicIo& iIo, bool advance);
     public:
         //! @name Creators
         //@{
-        /*! 
+        /*!
           @brief Constructor that can either open an existing Exv image or create
               a new image from scratch. If a new image is to be created, any
               existing data is overwritten. Since the constructor can not return
@@ -351,37 +339,13 @@ namespace Exiv2 {
         //! Destructor
         ~ExvImage() {}
         //@}
-        
-        //! @cond IGNORE
-        // Public only so that we can create a static instance
-        struct ExvRegister{
-            ExvRegister();
-        };
-        //! @endcond
     protected:
         //! @name Accessors
         //@{
-        /*!
-          @brief Determine if the content of the BasicIo instance is an Exv
-              image. See base class for more details.
-          @param iIo BasicIo instance to read from.
-          @param advance Flag indicating whether the position of the io
-              should be advanced by the number of characters read to
-              analyse the data (true) or left at its original
-              position (false). This applies only if the type matches.
-          @return  true  if the data matches a Jpeg image;<BR>
-                   false if the data does not match;<BR>
-         */
         virtual bool isThisType(BasicIo& iIo, bool advance) const;
         //@}
         //! @name Manipulators
         //@{
-        /*!
-          @brief Writes an Exv header (aka signature) to the BasicIo instance.
-          @param oIo BasicIo instance that the header is written to.
-          @return 0 if successful;<BR>
-                  4 if the output file can not be written to;<BR>
-         */
         int writeHeader(BasicIo& oIo) const;
         //@}
     private:
@@ -398,8 +362,28 @@ namespace Exiv2 {
         ExvImage& operator=(const ExvImage& rhs);
     }; // class ExvImage
 
-    static ExvImage::ExvRegister exvReg;
+// *****************************************************************************
+// template, inline and free functions
+
+    // These could be static private functions on Image subclasses but then
+    // ImageFactory needs to be made a friend.
+    /*!
+      @brief Create a new JpegImage instance and return an auto-pointer to it.
+             Caller owns the returned object and the auto-pointer ensures that
+             it will be deleted.
+     */
+    Image::AutoPtr newJpegInstance(BasicIo::AutoPtr io, bool create);
+    //! Check if the file iIo is a JPEG image.
+    bool isJpegType(BasicIo& iIo, bool advance);
+    /*!
+      @brief Create a new ExvImage instance and return an auto-pointer to it.
+             Caller owns the returned object and the auto-pointer ensures that
+             it will be deleted.
+     */
+    Image::AutoPtr newExvInstance(BasicIo::AutoPtr io, bool create);
+    //! Check if the file iIo is an EXV file
+    bool isExvType(BasicIo& iIo, bool advance);
+
 }                                       // namespace Exiv2
 
-
 #endif                                  // #ifndef JPGIMAGE_HPP_
