@@ -1,35 +1,35 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
- * 
+ * Copyright (C) 2004, 2005, 2006 Andreas Huggel <ahuggel@gmx.net>
+ *
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA.
  */
 /*
   File:      makernote.cpp
-  Version:   $Rev: 579 $
+  Version:   $Rev: 669 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   18-Feb-04, ahu: created
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: makernote.cpp 579 2005-06-11 04:11:23Z ahuggel $");
+EXIV2_RCSID("@(#) $Id: makernote.cpp 669 2006-01-17 14:53:57Z ahuggel $");
 
 // Define DEBUG_* to output debug information to std::cerr, e.g, by calling
-// make like this: make DEFS=-DDEBUG_MAKERNOTE makernote.o 
+// make like this: make DEFS=-DDEBUG_MAKERNOTE makernote.o
 //#define DEBUG_MAKERNOTE
 //#define DEBUG_REGISTRY
 
@@ -49,7 +49,7 @@ EXIV2_RCSID("@(#) $Id: makernote.cpp 579 2005-06-11 04:11:23Z ahuggel $");
 // class member definitions
 namespace Exiv2 {
 
-    MakerNote::MakerNote(bool alloc) 
+    MakerNote::MakerNote(bool alloc)
         : alloc_(alloc), offset_(0), byteOrder_(invalidByteOrder)
     {
     }
@@ -65,51 +65,51 @@ namespace Exiv2 {
     }
 
     IfdMakerNote::IfdMakerNote(IfdId ifdId, bool alloc, bool hasNext)
-        : MakerNote(alloc), 
-          absOffset_(true), adjOffset_(0), ifd_(ifdId, 0, alloc, hasNext)
+        : MakerNote(alloc),
+          absShift_(true), shift_(0), start_(0), ifd_(ifdId, 0, alloc, hasNext)
     {
     }
 
     IfdMakerNote::IfdMakerNote(const IfdMakerNote& rhs)
-        : MakerNote(rhs), absOffset_(rhs.absOffset_), adjOffset_(rhs.adjOffset_),
-          header_(rhs.header_.size_), ifd_(rhs.ifd_)
+        : MakerNote(rhs), absShift_(rhs.absShift_), shift_(rhs.shift_),
+          start_(rhs.start_), header_(rhs.header_.size_), ifd_(rhs.ifd_)
     {
         memcpy(header_.pData_, rhs.header_.pData_, header_.size_);
     }
 
     int IfdMakerNote::read(const byte* buf,
-                           long len, 
-                           ByteOrder byteOrder, 
-                           long offset)
+                           long len,
+                           long start,
+                           ByteOrder byteOrder,
+                           long shift)
     {
         // Remember the offset
-        offset_ = offset;
+        offset_ = start - shift;
         // Set byte order if none is set yet
         if (byteOrder_ == invalidByteOrder) byteOrder_ = byteOrder;
         // Read and check the header (and set offset adjustment)
-        int rc = readHeader(buf, len, byteOrder);
+        int rc = readHeader(buf + start, len - start, byteOrder);
         if (rc == 0) {
             rc = checkHeader();
         }
-        // Adjust the offset
-        offset = absOffset_ ? offset + adjOffset_ : adjOffset_;
+        // Adjust shift
+        long newShift = absShift_ ? shift + shift_ : start + shift_;
         // Read the makernote IFD
         if (rc == 0) {
-            rc = ifd_.read(buf + headerSize(), 
-                           len - headerSize(),
-                           byteOrder_,
-                           offset);
+            rc = ifd_.read(buf, len, start + start_, byteOrder_, newShift);
         }
         if (rc == 0) {
             // IfdMakerNote currently does not support multiple IFDs
             if (ifd_.next() != 0) {
+#ifndef SUPPRESS_WARNINGS
                 std::cerr << "Warning: Makernote IFD has a next pointer != 0 ("
                           << ifd_.next()
                           << "). Ignored.\n";
+#endif
             }
         }
 #ifdef DEBUG_MAKERNOTE
-        hexdump(std::cerr, buf, len, offset);
+        hexdump(std::cerr, buf + start, len - start);
         if (rc == 0) ifd_.print(std::cerr);
 #endif
 
@@ -123,7 +123,7 @@ namespace Exiv2 {
         // Set byte order if none is set yet
         if (byteOrder_ == invalidByteOrder) byteOrder_ = byteOrder;
         // Adjust the offset
-        offset = absOffset_ ? offset + adjOffset_ : adjOffset_;
+        offset = absShift_ ? offset + start_ - shift_ : start_ - shift_;
 
         long len = 0;
         len += copyHeader(buf);
@@ -132,7 +132,7 @@ namespace Exiv2 {
         return len;
     } // IfdMakerNote::copy
 
-    int IfdMakerNote::readHeader(const byte* buf, 
+    int IfdMakerNote::readHeader(const byte* buf,
                                  long len,
                                  ByteOrder byteOrder)
     {
@@ -141,8 +141,8 @@ namespace Exiv2 {
     }
 
     void IfdMakerNote::updateBase(byte* pNewBase)
-    { 
-        if (absOffset_) {
+    {
+        if (absShift_) {
             ifd_.updateBase(pNewBase);
         }
     }
@@ -164,9 +164,9 @@ namespace Exiv2 {
         return header_.size_;
     }
 
-    Entries::const_iterator IfdMakerNote::findIdx(int idx) const 
+    Entries::const_iterator IfdMakerNote::findIdx(int idx) const
     {
-        return ifd_.findIdx(idx); 
+        return ifd_.findIdx(idx);
     }
 
     long IfdMakerNote::size() const
@@ -184,9 +184,42 @@ namespace Exiv2 {
         return AutoPtr(clone_());
     }
 
+    int MakerNoteFactory::Init::count = 0;
+
+    MakerNoteFactory::Init::Init()
+    {
+        ++count;
+    }
+
+    MakerNoteFactory::Init::~Init()
+    {
+        if (--count == 0) {
+            Exiv2::MakerNoteFactory::cleanup();
+        }
+    }
+
     MakerNoteFactory::Registry* MakerNoteFactory::pRegistry_ = 0;
     MakerNoteFactory::IfdIdRegistry* MakerNoteFactory::pIfdIdRegistry_ = 0;
 
+    void MakerNoteFactory::cleanup()
+    {
+        if (pRegistry_ != 0) {
+            Registry::iterator e = pRegistry_->end();
+            for (Registry::iterator i = pRegistry_->begin(); i != e; ++i) {
+                delete i->second;
+            }
+            delete pRegistry_;
+        }
+
+        if (pIfdIdRegistry_ != 0) {
+            IfdIdRegistry::iterator e = pIfdIdRegistry_->end();
+            for (IfdIdRegistry::iterator i = pIfdIdRegistry_->begin(); i != e; ++i) {
+                delete i->second;
+            }
+            delete pIfdIdRegistry_;
+        }
+    }
+
     void MakerNoteFactory::init()
     {
         if (0 == pRegistry_) {
@@ -203,6 +236,11 @@ namespace Exiv2 {
         init();
         MakerNote* pMakerNote = makerNote.release();
         assert(pMakerNote);
+        IfdIdRegistry::iterator pos = pIfdIdRegistry_->find(ifdId);
+        if (pos != pIfdIdRegistry_->end()) {
+            delete pos->second;
+            pos->second = 0;
+        }
         (*pIfdIdRegistry_)[ifdId] = pMakerNote;
     } // MakerNoteFactory::registerMakerNote
 
@@ -215,12 +253,12 @@ namespace Exiv2 {
         return i->second->create(alloc);
     } // MakerNoteFactory::create
 
-    void MakerNoteFactory::registerMakerNote(const std::string& make, 
-                                             const std::string& model, 
+    void MakerNoteFactory::registerMakerNote(const std::string& make,
+                                             const std::string& model,
                                              CreateFct createMakerNote)
     {
 #ifdef DEBUG_REGISTRY
-        std::cerr << "Registering MakerNote create function for \"" 
+        std::cerr << "Registering MakerNote create function for \""
                   << make << "\" and \"" << model << "\".\n";
 #endif
         init();
@@ -255,16 +293,16 @@ namespace Exiv2 {
         }
     } // MakerNoteFactory::registerMakerNote
 
-    MakerNote::AutoPtr MakerNoteFactory::create(const std::string& make, 
+    MakerNote::AutoPtr MakerNoteFactory::create(const std::string& make,
                                                 const std::string& model,
                                                 bool alloc,
-                                                const byte* buf, 
-                                                long len, 
-                                                ByteOrder byteOrder, 
+                                                const byte* buf,
+                                                long len,
+                                                ByteOrder byteOrder,
                                                 long offset)
     {
 #ifdef DEBUG_REGISTRY
-        std::cerr << "Entering MakerNoteFactory::create(\"" 
+        std::cerr << "Entering MakerNoteFactory::create(\""
                   << make << "\", \"" << model << "\", "
                   << (alloc == true ? "true" : "false") << ")\n";
 #endif
@@ -273,7 +311,7 @@ namespace Exiv2 {
         ModelRegistry* pModelRegistry = 0;
 #ifdef DEBUG_REGISTRY
         std::string makeMatch;
-        std::cerr << "Searching make registry...\n"; 
+        std::cerr << "Searching make registry...\n";
 #endif
         assert(pRegistry_ != 0);
         Registry::const_iterator end1 = pRegistry_->end();
@@ -324,7 +362,7 @@ namespace Exiv2 {
                                 const std::string& key)
     {
 #ifdef DEBUG_REGISTRY
-        std::cerr << "   Matching registry entry \"" << regEntry << "\" (" 
+        std::cerr << "   Matching registry entry \"" << regEntry << "\" ("
                   << (int)regEntry.size() << ") with key \"" << key << "\" ("
                   << (int)key.size() << "): ";
 #endif
@@ -360,11 +398,11 @@ namespace Exiv2 {
                 }
 
                 bool found = false;
-                // Find the substr ss in the key starting from index ki. 
+                // Find the substr ss in the key starting from index ki.
                 // Take care of the special cases
                 //   + where the substr must match the key from beg to end,
                 //   + from beg,
-                //   + to end 
+                //   + to end
                 //   + and where it can be anywhere in the key.
                 // If found, ki is adjusted to the position in the key after ss.
                 if (ei == 0 && pos == std::string::npos) { // ei == 0 => ki == 0
@@ -380,7 +418,7 @@ namespace Exiv2 {
                     }
                 }
                 else if (pos == std::string::npos) {
-                    if (   ss.size() <= uKey.size() 
+                    if (   ss.size() <= uKey.size()
                         && ki <= uKey.size() - ss.size()) {
                         if (0 == uKey.compare(
                                 uKey.size() - ss.size(), ss.size(), ss)) {
@@ -390,7 +428,7 @@ namespace Exiv2 {
                     }
                 }
                 else {
-                    std::string::size_type idx = uKey.find(ss, ki); 
+                    std::string::size_type idx = uKey.find(ss, ki);
                     if (idx != std::string::npos) {
                         found = true;
                         ki = idx + ss.size();
@@ -416,7 +454,7 @@ namespace Exiv2 {
         std::cerr << "Match (score: " << count + 1 << ")\n";
 #endif
         return count + 1;
-        
+
     } // MakerNoteFactory::match
 
 }                                       // namespace Exiv2
