@@ -1,26 +1,26 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
- * 
+ * Copyright (C) 2004, 2005, 2006 Andreas Huggel <ahuggel@gmx.net>
+ *
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA.
  */
 /*
   File:      metadatum.cpp
-  Version:   $Rev: 538 $
+  Version:   $Rev: 669 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
              Brad Schick (brad) <brad@robotbattle.com>
   History:   26-Jan-04, ahu: created
@@ -28,7 +28,7 @@
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: metadatum.cpp 538 2005-03-12 16:43:06Z ahuggel $");
+EXIV2_RCSID("@(#) $Id: metadatum.cpp 669 2006-01-17 14:53:57Z ahuggel $");
 
 // *****************************************************************************
 // included header files
@@ -42,21 +42,43 @@ EXIV2_RCSID("@(#) $Id: metadatum.cpp 538 2005-03-12 16:43:06Z ahuggel $");
 // *****************************************************************************
 // class member definitions
 namespace Exiv2 {
-    
+
     Key::AutoPtr Key::clone() const
     {
         return AutoPtr(clone_());
     }
 
+    Key& Key::operator=(const Key& /*rhs*/)
+    {
+        return *this;
+    }
+
+    Metadatum::Metadatum()
+    {
+    }
+
+    Metadatum::Metadatum(const Metadatum& /*rhs*/)
+    {
+    }
+
+    Metadatum::~Metadatum()
+    {
+    }
+
+    Metadatum& Metadatum::operator=(const Metadatum& /*rhs*/)
+    {
+        return *this;
+    }
+
     std::ostream& operator<<(std::ostream& os, const Metadatum& md)
     {
         os << "0x" << std::setw(4) << std::setfill('0') << std::right
-                  << std::hex << md.tag() << " " 
+                  << std::hex << md.tag() << " "
                   << std::setw(40) << std::setfill(' ') << std::left
                   << md.key() << " "
                   << std::setw(9) << std::setfill(' ') << std::left
                   << md.typeName() << " "
-                  << std::dec << md.value() 
+                  << std::dec << md.value()
                   << "\n";
         return os;
     }
