@@ -1,44 +1,44 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
- * 
+ * Copyright (C) 2004, 2005, 2006 Andreas Huggel <ahuggel@gmx.net>
+ *
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA.
  */
 /*!
   @file    nikonmn.hpp
   @brief   Nikon MakerNote formats.
 
   The Nikon MakerNote formats are implemented according to the following references<BR>
-  Format 1: 
+  Format 1:
   <ul>
   <li><a href="http://www.tawbaware.com/990exif.htm">MakerNote EXIF Tag of the Nikon 990</a> by Max Lyons</li></ul>
-  Format 2: 
-  <ul><li>"Appendix 2: Makernote of Nikon" of the document 
+  Format 2:
+  <ul><li>"Appendix 2: Makernote of Nikon" of the document
   <a href="http://park2.wakwak.com/%7Etsuruzoh/Computer/Digicams/exif-e.html">
   Exif file format</a> by TsuruZoh Tachibanaya</li></ul>
-  Format 3: 
+  Format 3:
   <ul><li>"EXIFutils Field Reference Guide"</li>
-  <li><a href="http://www.ozhiker.com/electronics/pjmt/jpeg_info/nikon_mn.html#Nikon_Type_3_Tags">Nikon Type 3 Makernote Tags Definition</a> 
+  <li><a href="http://www.ozhiker.com/electronics/pjmt/jpeg_info/nikon_mn.html#Nikon_Type_3_Tags">Nikon Type 3 Makernote Tags Definition</a>
   of the PHP JPEG Metadata Toolkit by Evan Hunter</li>
   <li>Nikon tag information from <a href="http://www.sno.phy.queensu.ca/~phil/exiftool/">ExifTool</a> by Phil Harvey</li>
-  <li>Email communication with Robert Rottmerhusen</li>
+  <li>Email communication with <a href="http://www.rottmerhusen.com">Robert Rottmerhusen</a></li>
   </ul>
 
-  @version $Rev: 588 $
+  @version $Rev: 669 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    17-May-04, ahu: created<BR>
@@ -74,27 +74,27 @@ namespace Exiv2 {
              initialized to operate in the memory management model indicated.
              The caller owns this copy and the auto-pointer ensures that it
              will be deleted.
-      
+
       @param alloc Memory management model for the new MakerNote. Determines if
              memory required to store data should be allocated and deallocated
              (true) or not (false). If false, only pointers to the buffer
              provided to read() will be kept. See Ifd for more background on
              this concept.
-      @param buf Pointer to the makernote character buffer (not used). 
-      @param len Length of the makernote character buffer (not used). 
-      @param byteOrder Byte order in which the Exif data (and possibly the 
+      @param buf Pointer to the makernote character buffer (not used).
+      @param len Length of the makernote character buffer (not used).
+      @param byteOrder Byte order in which the Exif data (and possibly the
              makernote) is encoded (not used).
       @param offset Offset from the start of the TIFF header of the makernote
              buffer (not used).
-      
+
       @return An auto-pointer to a newly created empty MakerNote. The caller
              owns this copy and the auto-pointer ensures that it will be
              deleted.
      */
     MakerNote::AutoPtr createNikonMakerNote(bool alloc,
-                                            const byte* buf, 
-                                            long len, 
-                                            ByteOrder byteOrder, 
+                                            const byte* buf,
+                                            long len,
+                                            ByteOrder byteOrder,
                                             long offset);
 
 // *****************************************************************************
@@ -125,7 +125,7 @@ namespace Exiv2 {
         AutoPtr clone() const;
         //@}
 
-        //! @name Print functions for Nikon1 %MakerNote tags 
+        //! @name Print functions for Nikon1 %MakerNote tags
         //@{
         //! Print ISO setting
         static std::ostream& print0x0002(std::ostream& os, const Value& value);
@@ -160,9 +160,9 @@ namespace Exiv2 {
     static Nikon1MakerNote::RegisterMn registerNikon1MakerNote;
 
     /*!
-      @brief A second MakerNote format used by Nikon cameras, including the 
+      @brief A second MakerNote format used by Nikon cameras, including the
              E700, E800, E900, E900S, E910, E950
-     */ 
+     */
     class Nikon2MakerNote : public IfdMakerNote {
     public:
         //! Shortcut for a %Nikon2MakerNote auto pointer.
@@ -182,8 +182,8 @@ namespace Exiv2 {
         //@}
 
         //! @name Manipulators
-        //@{        
-        int readHeader(const byte* buf, 
+        //@{
+        int readHeader(const byte* buf,
                        long len,
                        ByteOrder byteOrder);
         //@}
@@ -195,7 +195,7 @@ namespace Exiv2 {
         AutoPtr clone() const;
         //@}
 
-        //! @name Print functions for Nikon2 %MakerNote tags 
+        //! @name Print functions for Nikon2 %MakerNote tags
         //@{
         //! Print quality setting
         static std::ostream& print0x0003(std::ostream& os, const Value& value);
@@ -252,7 +252,7 @@ namespace Exiv2 {
 
         //! @name Manipulators
         //@{
-        int readHeader(const byte* buf, 
+        int readHeader(const byte* buf,
                        long len,
                        ByteOrder byteOrder);
         //@}
@@ -264,7 +264,7 @@ namespace Exiv2 {
         AutoPtr clone() const;
         //@}
 
-        //! @name Print functions for Nikon3 %MakerNote tags 
+        //! @name Print functions for Nikon3 %MakerNote tags
         //@{
         //! Print ISO setting
         static std::ostream& print0x0002(std::ostream& os, const Value& value);
