@@ -1,26 +1,26 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
- * 
+ * Copyright (C) 2004, 2005, 2006 Andreas Huggel <ahuggel@gmx.net>
+ *
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA.
  */
 /*
   File:      sigmamn.cpp
-  Version:   $Rev: 569 $
+  Version:   $Rev: 669 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   02-Apr-04, ahu: created
   Credits:   Sigma and Foveon MakerNote implemented according to the specification
@@ -29,7 +29,7 @@
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: sigmamn.cpp 569 2005-05-28 05:48:43Z ahuggel $");
+EXIV2_RCSID("@(#) $Id: sigmamn.cpp 669 2006-01-17 14:53:57Z ahuggel $");
 
 // *****************************************************************************
 // included header files
@@ -44,9 +44,6 @@ EXIV2_RCSID("@(#) $Id: sigmamn.cpp 569 2005-05-28 05:48:43Z ahuggel $");
 #include <iomanip>
 #include <cassert>
 
-// Define DEBUG_MAKERNOTE to output debug information to std::cerr
-#undef DEBUG_MAKERNOTE
-
 // *****************************************************************************
 // class member definitions
 namespace Exiv2 {
@@ -54,7 +51,7 @@ namespace Exiv2 {
     //! @cond IGNORE
     SigmaMakerNote::RegisterMn::RegisterMn()
     {
-        MakerNoteFactory::registerMakerNote("SIGMA", "*", createSigmaMakerNote); 
+        MakerNoteFactory::registerMakerNote("SIGMA", "*", createSigmaMakerNote);
         MakerNoteFactory::registerMakerNote("FOVEON", "*", createSigmaMakerNote);
         MakerNoteFactory::registerMakerNote(
             sigmaIfdId, MakerNote::AutoPtr(new SigmaMakerNote));
@@ -65,31 +62,31 @@ namespace Exiv2 {
 
     // Sigma (Foveon) MakerNote Tag Info
     const TagInfo SigmaMakerNote::tagInfo_[] = {
-        TagInfo(0x0002, "SerialNumber", "Camera serial number", sigmaIfdId, makerTags, asciiString, printValue),
-        TagInfo(0x0003, "DriveMode", "Drive Mode", sigmaIfdId, makerTags, asciiString, printValue),
-        TagInfo(0x0004, "ResolutionMode", "Resolution Mode", sigmaIfdId, makerTags, asciiString, printValue),
-        TagInfo(0x0005, "AutofocusMode", "Autofocus mode", sigmaIfdId, makerTags, asciiString, printValue),
-        TagInfo(0x0006, "FocusSetting", "Focus setting", sigmaIfdId, makerTags, asciiString, printValue),
-        TagInfo(0x0007, "WhiteBalance", "White balance", sigmaIfdId, makerTags, asciiString, printValue),
-        TagInfo(0x0008, "ExposureMode", "Exposure mode", sigmaIfdId, makerTags, asciiString, print0x0008),
-        TagInfo(0x0009, "MeteringMode", "Metering mode", sigmaIfdId, makerTags, asciiString, print0x0009),
-        TagInfo(0x000a, "LensRange", "Lens focal length range", sigmaIfdId, makerTags, asciiString, printValue),
-        TagInfo(0x000b, "ColorSpace", "Color space", sigmaIfdId, makerTags, asciiString, printValue),
-        TagInfo(0x000c, "Exposure", "Exposure", sigmaIfdId, makerTags, asciiString, printStripLabel),
-        TagInfo(0x000d, "Contrast", "Contrast", sigmaIfdId, makerTags, asciiString, printStripLabel),
-        TagInfo(0x000e, "Shadow", "Shadow", sigmaIfdId, makerTags, asciiString, printStripLabel),
-        TagInfo(0x000f, "Highlight", "Highlight", sigmaIfdId, makerTags, asciiString, printStripLabel),
-        TagInfo(0x0010, "Saturation", "Saturation", sigmaIfdId, makerTags, asciiString, printStripLabel),
-        TagInfo(0x0011, "Sharpness", "Sharpness", sigmaIfdId, makerTags, asciiString, printStripLabel),
-        TagInfo(0x0012, "FillLight", "X3 Fill light", sigmaIfdId, makerTags, asciiString, printStripLabel),
-        TagInfo(0x0014, "ColorAdjustment", "Color adjustment", sigmaIfdId, makerTags, asciiString, printStripLabel),
-        TagInfo(0x0015, "AdjustmentMode", "Adjustment mode", sigmaIfdId, makerTags, asciiString, printValue),
-        TagInfo(0x0016, "Quality", "Quality", sigmaIfdId, makerTags, asciiString, printStripLabel),
-        TagInfo(0x0017, "Firmware", "Firmware", sigmaIfdId, makerTags, asciiString, printValue),
-        TagInfo(0x0018, "Software", "Software", sigmaIfdId, makerTags, asciiString, printValue),
-        TagInfo(0x0019, "AutoBracket", "Auto bracket", sigmaIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x0002, "SerialNumber", "SerialNumber", "Camera serial number", sigmaIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x0003, "DriveMode", "DriveMode", "Drive Mode", sigmaIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x0004, "ResolutionMode", "ResolutionMode", "Resolution Mode", sigmaIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x0005, "AutofocusMode", "AutofocusMode", "Autofocus mode", sigmaIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x0006, "FocusSetting", "FocusSetting", "Focus setting", sigmaIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x0007, "WhiteBalance", "WhiteBalance", "White balance", sigmaIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x0008, "ExposureMode", "ExposureMode", "Exposure mode", sigmaIfdId, makerTags, asciiString, print0x0008),
+        TagInfo(0x0009, "MeteringMode", "MeteringMode", "Metering mode", sigmaIfdId, makerTags, asciiString, print0x0009),
+        TagInfo(0x000a, "LensRange", "LensRange", "Lens focal length range", sigmaIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x000b, "ColorSpace", "ColorSpace", "Color space", sigmaIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x000c, "Exposure", "Exposure", "Exposure", sigmaIfdId, makerTags, asciiString, printStripLabel),
+        TagInfo(0x000d, "Contrast", "Contrast", "Contrast", sigmaIfdId, makerTags, asciiString, printStripLabel),
+        TagInfo(0x000e, "Shadow", "Shadow", "Shadow", sigmaIfdId, makerTags, asciiString, printStripLabel),
+        TagInfo(0x000f, "Highlight", "Highlight", "Highlight", sigmaIfdId, makerTags, asciiString, printStripLabel),
+        TagInfo(0x0010, "Saturation", "Saturation", "Saturation", sigmaIfdId, makerTags, asciiString, printStripLabel),
+        TagInfo(0x0011, "Sharpness", "Sharpness", "Sharpness", sigmaIfdId, makerTags, asciiString, printStripLabel),
+        TagInfo(0x0012, "FillLight", "FillLight", "X3 Fill light", sigmaIfdId, makerTags, asciiString, printStripLabel),
+        TagInfo(0x0014, "ColorAdjustment", "ColorAdjustment", "Color adjustment", sigmaIfdId, makerTags, asciiString, printStripLabel),
+        TagInfo(0x0015, "AdjustmentMode", "AdjustmentMode", "Adjustment mode", sigmaIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x0016, "Quality", "Quality", "Quality", sigmaIfdId, makerTags, asciiString, printStripLabel),
+        TagInfo(0x0017, "Firmware", "Firmware", "Firmware", sigmaIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x0018, "Software", "Software", "Software", sigmaIfdId, makerTags, asciiString, printValue),
+        TagInfo(0x0019, "AutoBracket", "AutoBracket", "Auto bracket", sigmaIfdId, makerTags, asciiString, printValue),
         // End of list marker
-        TagInfo(0xffff, "(UnknownSigmaMakerNoteTag)", "Unknown SigmaMakerNote tag", sigmaIfdId, makerTags, invalidTypeId, printValue)
+        TagInfo(0xffff, "(UnknownSigmaMakerNoteTag)", "(UnknownSigmaMakerNoteTag)", "Unknown SigmaMakerNote tag", sigmaIfdId, makerTags, invalidTypeId, printValue)
     };
 
     SigmaMakerNote::SigmaMakerNote(bool alloc)
@@ -107,7 +104,7 @@ namespace Exiv2 {
     }
 
     int SigmaMakerNote::readHeader(const byte* buf,
-                                   long len, 
+                                   long len,
                                    ByteOrder byteOrder)
     {
         if (len < 10) return 1;
@@ -118,7 +115,7 @@ namespace Exiv2 {
         header_.alloc(10);
         memcpy(header_.pData_, buf, header_.size_);
         // Adjust the offset of the IFD for the prefix
-        adjOffset_ = 10;
+        start_ = 10;
         return 0;
     }
 
@@ -127,9 +124,9 @@ namespace Exiv2 {
         int rc = 0;
         // Check the SIGMA or FOVEON prefix
         if (   header_.size_ < 10
-            || std::string(reinterpret_cast<char*>(header_.pData_), 8) 
+            || std::string(reinterpret_cast<char*>(header_.pData_), 8)
                         != std::string("SIGMA\0\0\0", 8)
-            && std::string(reinterpret_cast<char*>(header_.pData_), 8) 
+            && std::string(reinterpret_cast<char*>(header_.pData_), 8)
                         != std::string("FOVEON\0\0", 8)) {
             rc = 2;
         }
@@ -200,9 +197,9 @@ namespace Exiv2 {
 // free functions
 
     MakerNote::AutoPtr createSigmaMakerNote(bool alloc,
-                                            const byte* buf, 
-                                            long len, 
-                                            ByteOrder byteOrder, 
+                                            const byte* buf,
+                                            long len,
+                                            ByteOrder byteOrder,
                                             long offset)
     {
         return MakerNote::AutoPtr(new SigmaMakerNote(alloc));
