@@ -1,32 +1,32 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
- * 
+ * Copyright (C) 2004, 2005, 2006 Andreas Huggel <ahuggel@gmx.net>
+ *
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA.
  */
 /*
   File:      sonymn.cpp
-  Version:   $Rev: 569 $
+  Version:   $Rev: 669 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   18-Apr-05, ahu: created
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: sonymn.cpp 569 2005-05-28 05:48:43Z ahuggel $");
+EXIV2_RCSID("@(#) $Id: sonymn.cpp 669 2006-01-17 14:53:57Z ahuggel $");
 
 // *****************************************************************************
 // included header files
@@ -41,9 +41,6 @@ EXIV2_RCSID("@(#) $Id: sonymn.cpp 569 2005-05-28 05:48:43Z ahuggel $");
 #include <iomanip>
 #include <cassert>
 
-// Define DEBUG_MAKERNOTE to output debug information to std::cerr
-#undef DEBUG_MAKERNOTE
-
 // *****************************************************************************
 // class member definitions
 namespace Exiv2 {
@@ -51,27 +48,27 @@ namespace Exiv2 {
     //! @cond IGNORE
     SonyMakerNote::RegisterMn::RegisterMn()
     {
-        MakerNoteFactory::registerMakerNote("SONY", "*", createSonyMakerNote); 
+        MakerNoteFactory::registerMakerNote("SONY", "*", createSonyMakerNote);
         MakerNoteFactory::registerMakerNote(
             sonyIfdId, MakerNote::AutoPtr(new SonyMakerNote));
-        
+
         ExifTags::registerMakerTagInfo(sonyIfdId, tagInfo_);
     }
     //! @endcond
 
     // Sony MakerNote Tag Info
     const TagInfo SonyMakerNote::tagInfo_[] = {
-        TagInfo(0x2000, "0x2000", "Unknown", sonyIfdId, makerTags, undefined, printValue),
-        TagInfo(0x9001, "0x9001", "Unknown", sonyIfdId, makerTags, undefined, printValue),
-        TagInfo(0x9002, "0x9002", "Unknown", sonyIfdId, makerTags, undefined, printValue),
-        TagInfo(0x9003, "0x9003", "Unknown", sonyIfdId, makerTags, undefined, printValue),
-        TagInfo(0x9004, "0x9004", "Unknown", sonyIfdId, makerTags, undefined, printValue),
-        TagInfo(0x9005, "0x9005", "Unknown", sonyIfdId, makerTags, undefined, printValue),
-        TagInfo(0x9006, "0x9006", "Unknown", sonyIfdId, makerTags, undefined, printValue),
-        TagInfo(0x9007, "0x9007", "Unknown", sonyIfdId, makerTags, undefined, printValue),
-        TagInfo(0x9008, "0x9008", "Unknown", sonyIfdId, makerTags, undefined, printValue),
+        TagInfo(0x2000, "0x2000", "0x2000", "Unknown", sonyIfdId, makerTags, undefined, printValue),
+        TagInfo(0x9001, "0x9001", "0x9001", "Unknown", sonyIfdId, makerTags, undefined, printValue),
+        TagInfo(0x9002, "0x9002", "0x9002", "Unknown", sonyIfdId, makerTags, undefined, printValue),
+        TagInfo(0x9003, "0x9003", "0x9003", "Unknown", sonyIfdId, makerTags, undefined, printValue),
+        TagInfo(0x9004, "0x9004", "0x9004", "Unknown", sonyIfdId, makerTags, undefined, printValue),
+        TagInfo(0x9005, "0x9005", "0x9005", "Unknown", sonyIfdId, makerTags, undefined, printValue),
+        TagInfo(0x9006, "0x9006", "0x9006", "Unknown", sonyIfdId, makerTags, undefined, printValue),
+        TagInfo(0x9007, "0x9007", "0x9007", "Unknown", sonyIfdId, makerTags, undefined, printValue),
+        TagInfo(0x9008, "0x9008", "0x9008", "Unknown", sonyIfdId, makerTags, undefined, printValue),
         // End of list marker
-        TagInfo(0xffff, "(UnknownSonyMakerNoteTag)", "Unknown SonyMakerNote tag", sonyIfdId, makerTags, invalidTypeId, printValue)
+        TagInfo(0xffff, "(UnknownSonyMakerNoteTag)", "Unknown SonyMakerNote tag", "Unknown SonyMakerNote tag", sonyIfdId, makerTags, invalidTypeId, printValue)
     };
 
     SonyMakerNote::SonyMakerNote(bool alloc)
@@ -89,14 +86,14 @@ namespace Exiv2 {
     }
 
     int SonyMakerNote::readHeader(const byte* buf,
-                                   long len, 
+                                   long len,
                                    ByteOrder byteOrder)
     {
         if (len < 12) return 1;
         header_.alloc(12);
         memcpy(header_.pData_, buf, header_.size_);
         // Adjust the offset of the IFD for the prefix
-        adjOffset_ = 12;
+        start_ = 12;
         return 0;
     }
 
@@ -105,7 +102,7 @@ namespace Exiv2 {
         int rc = 0;
         // Check the SONY prefix
         if (   header_.size_ < 12
-            || std::string(reinterpret_cast<char*>(header_.pData_), 12) 
+            || std::string(reinterpret_cast<char*>(header_.pData_), 12)
                != std::string("SONY DSC \0\0\0", 12)) {
             rc = 2;
         }
@@ -139,9 +136,9 @@ namespace Exiv2 {
 // free functions
 
     MakerNote::AutoPtr createSonyMakerNote(bool alloc,
-                                            const byte* buf, 
-                                            long len, 
-                                            ByteOrder byteOrder, 
+                                            const byte* buf,
+                                            long len,
+                                            ByteOrder byteOrder,
                                             long offset)
     {
         return MakerNote::AutoPtr(new SonyMakerNote(alloc));
