@@ -1,33 +1,33 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
- * 
+ * Copyright (C) 2004, 2005, 2006 Andreas Huggel <ahuggel@gmx.net>
+ *
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA.
  */
 /*
   File:      types.cpp
-  Version:   $Rev: 578 $
+  Version:   $Rev: 671 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   26-Jan-04, ahu: created
              11-Feb-04, ahu: isolated as a component
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: types.cpp 578 2005-06-07 15:01:11Z ahuggel $");
+EXIV2_RCSID("@(#) $Id: types.cpp 671 2006-01-18 11:34:45Z ahuggel $");
 
 // *****************************************************************************
 // included header files
@@ -40,6 +40,9 @@ EXIV2_RCSID("@(#) $Id: types.cpp 578 2005-06-07 15:01:11Z ahuggel $");
 #include <sstream>
 #include <utility>
 #include <cctype>
+#include <ctime>
+#include <cstdio>
+#include <cassert>
 
 // *****************************************************************************
 // class member definitions
@@ -67,6 +70,7 @@ namespace Exiv2 {
         TypeInfoTable(date,             "Date",        8),
         TypeInfoTable(time,             "Time",        11),
         TypeInfoTable(comment,          "Comment",     1),
+        TypeInfoTable(directory,        "Directory",   1),
         // End of list marker
         TypeInfoTable(lastTypeId,       "(Unknown)",   0)
     };
@@ -89,14 +93,14 @@ namespace Exiv2 {
     {
         return typeInfoTable_[ typeId < lastTypeId ? typeId : 0 ].size_;
     }
-    
+
     DataBuf::DataBuf(DataBuf& rhs)
         : pData_(rhs.pData_), size_(rhs.size_)
     {
         rhs.release();
     }
 
-    DataBuf::DataBuf(byte* pData, long size) 
+    DataBuf::DataBuf(const byte* pData, long size)
         : pData_(0), size_(0)
     {
         if (size > 0) {
@@ -114,12 +118,12 @@ namespace Exiv2 {
     }
 
     void DataBuf::alloc(long size)
-    { 
+    {
         if (size > size_) {
-            delete[] pData_; 
-            size_ = size; 
+            delete[] pData_;
+            size_ = size;
             pData_ = new byte[size];
-        } 
+        }
     }
 
     std::pair<byte*, long> DataBuf::release()
@@ -155,11 +159,11 @@ namespace Exiv2 {
     uint32_t getULong(const byte* buf, ByteOrder byteOrder)
     {
         if (byteOrder == littleEndian) {
-            return   (byte)buf[3] << 24 | (byte)buf[2] << 16 
+            return   (byte)buf[3] << 24 | (byte)buf[2] << 16
                    | (byte)buf[1] <<  8 | (byte)buf[0];
         }
         else {
-            return   (byte)buf[0] << 24 | (byte)buf[1] << 16 
+            return   (byte)buf[0] << 24 | (byte)buf[1] << 16
                    | (byte)buf[2] <<  8 | (byte)buf[3];
         }
     }
@@ -184,11 +188,11 @@ namespace Exiv2 {
     int32_t getLong(const byte* buf, ByteOrder byteOrder)
     {
         if (byteOrder == littleEndian) {
-            return   (byte)buf[3] << 24 | (byte)buf[2] << 16 
+            return   (byte)buf[3] << 24 | (byte)buf[2] << 16
                    | (byte)buf[1] <<  8 | (byte)buf[0];
         }
         else {
-            return   (byte)buf[0] << 24 | (byte)buf[1] << 16 
+            return   (byte)buf[0] << 24 | (byte)buf[1] << 16
                    | (byte)buf[2] <<  8 | (byte)buf[3];
         }
     }
@@ -276,13 +280,13 @@ namespace Exiv2 {
 
     void hexdump(std::ostream& os, const byte* buf, long len, long offset)
     {
-        const std::string::size_type pos = 8 + 16 * 3 + 2; 
-        const std::string align(pos, ' '); 
+        const std::string::size_type pos = 8 + 16 * 3 + 2;
+        const std::string align(pos, ' ');
 
         long i = 0;
         while (i < len) {
-            os << "  " 
-               << std::setw(4) << std::setfill('0') << std::hex 
+            os << "  "
+               << std::setw(4) << std::setfill('0') << std::hex
                << i + offset << "  ";
             std::ostringstream ss;
             do {
@@ -297,47 +301,37 @@ namespace Exiv2 {
         os << std::dec << std::setfill(' ');
     } // hexdump
 
-    int gcd(int a, int b)
-    {
-        int temp;
-        if (a < b) {
-            temp = a;
-            a = b; 
-            b = temp; 
-        }
-        while ((temp = a % b) != 0) {
-            a = b;
-            b = temp;
-        }
-        return b;
-    } // gcd
-
-    long lgcd(long a, long b)
-    {
-        long temp;
-        if (a < b) {
-            temp = a;
-            a = b; 
-            b = temp; 
-        }
-        while ((temp = a % b) != 0) {
-            a = b;
-            b = temp;
-        }
-        return b;
-    } // lgcd
-
     bool isHex(const std::string& str, size_t size, const std::string& prefix)
     {
-        if (   str.size() <= prefix.size() 
+        if (   str.size() <= prefix.size()
             || str.substr(0, prefix.size()) != prefix) return false;
         if (   size > 0
             && str.size() != size + prefix.size()) return false;
-        
+
         for (size_t i = prefix.size(); i < str.size(); ++i) {
             if (!isxdigit(str[i])) return false;
         }
         return true;
     } // isHex
 
+    int exifTime(const char* buf, struct tm* tm)
+    {
+        assert(buf != 0);
+        assert(tm != 0);
+        int rc = 1;
+        int year, mon, mday, hour, min, sec;
+        int scanned = sscanf(buf, "%4d:%2d:%2d %2d:%2d:%2d",
+                             &year, &mon, &mday, &hour, &min, &sec);
+        if (scanned == 6) {
+            tm->tm_year = year - 1900;
+            tm->tm_mon  = mon - 1;
+            tm->tm_mday = mday;
+            tm->tm_hour = hour;
+            tm->tm_min  = min;
+            tm->tm_sec  = sec;
+            rc = 0;
+        }
+        return rc;
+    } // exifTime
+
 }                                       // namespace Exiv2
