@@ -1,27 +1,27 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
- * 
+ * Copyright (C) 2004, 2005, 2006 Andreas Huggel <ahuggel@gmx.net>
+ *
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA.
  */
 /*!
   @file    types.hpp
   @brief   Type definitions for %Exiv2 and related functionality
-  @version $Rev: 581 $
+  @version $Rev: 808 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @date    09-Jan-04, ahu: created<BR>
@@ -39,12 +39,14 @@
 # include "exv_conf.h"
 #endif
 
+#include "exiv2_version.h"
+
 // + standard includes
 #include <string>
 #include <iosfwd>
 #include <utility>
+#include <algorithm>
 #include <sstream>
-#include <cstdio>
 #ifdef EXV_HAVE_STDINT_H
 # include <stdint.h>
 #endif
@@ -58,6 +60,17 @@ typedef __int16          int16_t;
 typedef __int32          int32_t;
 #endif
 
+/*!
+  @brief Macro to make calls to member functions through a pointer more readable.
+         See the C++ FAQ LITE, item
+         <a href="http://www.parashift.com/c++-faq-lite/pointers-to-members.html#faq-33.5" title="[33.5] How can I avoid syntax errors when calling a member function using a pointer-to-member-function?">[33.5] How can I avoid syntax errors when calling a member function using a pointer-to-member-function?</a>.
+ */
+#define EXV_CALL_MEMBER_FN(object,ptrToMember)  ((object).*(ptrToMember))
+
+// *****************************************************************************
+// forward declarations
+struct tm;
+
 // *****************************************************************************
 // namespace extensions
 namespace Exiv2 {
@@ -76,23 +89,35 @@ namespace Exiv2 {
     //! Type to express the byte order (little or big endian)
     enum ByteOrder { invalidByteOrder, littleEndian, bigEndian };
 
+    //! An identifier for each type of metadata
+    enum MetadataId { mdExif=1, mdIptc=2, mdComment=4 };
+
     //! Type identifiers for IFD format types
-    enum TypeId { invalidTypeId, unsignedByte, asciiString, unsignedShort, 
-                  unsignedLong, unsignedRational, invalid6, undefined, 
-                  signedShort, signedLong, signedRational, 
-                  string, date, time, 
+    enum TypeId { invalidTypeId, unsignedByte, asciiString, unsignedShort,
+                  unsignedLong, unsignedRational, invalid6, undefined,
+                  signedShort, signedLong, signedRational,
+                  string, date, time,
                   comment,
+                  directory,
                   lastTypeId };
 
     // Todo: decentralize IfdId, so that new ids can be defined elsewhere
     //! Type to specify the IFD to which a metadata belongs
-    enum IfdId { ifdIdNotSet, 
-                 ifd0Id, exifIfdId, gpsIfdId, iopIfdId, ifd1Id, 
+    enum IfdId { ifdIdNotSet,
+                 ifd0Id, exifIfdId, gpsIfdId, iopIfdId, ifd1Id,
                  canonIfdId, canonCs1IfdId, canonCs2IfdId, canonCfIfdId,
-                 fujiIfdId, nikon1IfdId, nikon2IfdId, nikon3IfdId, 
+                 fujiIfdId,
+                 minoltaIfdId, minoltaCs5DIfdId, minoltaCs7DIfdId,
+                 minoltaCsOldIfdId, minoltaCsNewIfdId,
+                 nikon1IfdId, nikon2IfdId, nikon3IfdId,
                  olympusIfdId, panasonicIfdId, sigmaIfdId, sonyIfdId,
                  lastIfdId };
 
+    //! Type to identify where the data is stored in a directory
+    enum DataLocId { invalidDataLocId,
+                     valueData, directoryData,
+                     lastDataLocId };
+
 // *****************************************************************************
 // class definitions
 
@@ -102,7 +127,7 @@ namespace Exiv2 {
         TypeInfoTable(TypeId typeId, const char* name, long size);
         TypeId typeId_;                         //!< Type id
         const char* name_;                      //!< Name of the type
-        long size_;                             //!< Bytes per data entry 
+        long size_;                             //!< Bytes per data entry
     }; // struct TypeInfoTable
 
     //! Type information lookup functions. Implemented as a static class.
@@ -151,12 +176,12 @@ namespace Exiv2 {
         //@{
         //! Default constructor
         DataBuf() : pData_(0), size_(0) {}
-        //! Constructor with an initial buffer size 
+        //! Constructor with an initial buffer size
         explicit DataBuf(long size) : pData_(new byte[size]), size_(size) {}
         //! Constructor, copies an existing buffer
-        DataBuf(byte* pData, long size);
-        /*! 
-          @brief Copy constructor. Transfers the buffer to the newly created 
+        DataBuf(const byte* pData, long size);
+        /*!
+          @brief Copy constructor. Transfers the buffer to the newly created
                  object similar to std::auto_ptr, i.e., the original object is
                  modified.
          */
@@ -169,26 +194,26 @@ namespace Exiv2 {
         //@{
         /*!
           @brief Assignment operator. Transfers the buffer and releases the
-                 buffer at the original object similar to std::auto_ptr, i.e., 
+                 buffer at the original object similar to std::auto_ptr, i.e.,
                  the original object is modified.
          */
         DataBuf& operator=(DataBuf& rhs);
         //! Allocate a data buffer of the given size
         void alloc(long size);
         /*!
-          @brief Release ownership of the buffer to the caller. Returns the 
+          @brief Release ownership of the buffer to the caller. Returns the
                  buffer as a data pointer and size pair, resets the internal
                  buffer.
          */
         std::pair<byte*, long> release();
         //! Reset value
-        void reset(std::pair<byte*, long> =std::make_pair(0,0));
+        void reset(std::pair<byte*, long> =std::make_pair((byte*)(0),long(0)));
         //@}
 
         /*!
           @name Conversions
 
-          Special conversions with auxiliary type to enable copies 
+          Special conversions with auxiliary type to enable copies
           and assignments, similar to those used for std::auto_ptr.
           See http://www.josuttis.com/libbook/auto_ptr.html for a discussion.
          */
@@ -202,7 +227,7 @@ namespace Exiv2 {
         //! Pointer to the buffer, 0 if none has been allocated
         byte* pData_;
         //! The current size of the buffer
-        long size_; 
+        long size_;
     }; // class DataBuf
 
 
@@ -232,7 +257,7 @@ namespace Exiv2 {
     std::istream& operator>>(std::istream& is, URational& r);
 
     /*!
-      @brief Convert an unsigned short to data, write the data to the buffer, 
+      @brief Convert an unsigned short to data, write the data to the buffer,
              return number of bytes written.
      */
     long us2Data(byte* buf, uint16_t s, ByteOrder byteOrder);
@@ -247,7 +272,7 @@ namespace Exiv2 {
      */
     long ur2Data(byte* buf, URational l, ByteOrder byteOrder);
     /*!
-      @brief Convert a signed short to data, write the data to the buffer, 
+      @brief Convert a signed short to data, write the data to the buffer,
              return number of bytes written.
      */
     long s2Data(byte* buf, int16_t s, ByteOrder byteOrder);
@@ -269,32 +294,83 @@ namespace Exiv2 {
      */
     void hexdump(std::ostream& os, const byte* buf, long len, long offset =0);
 
-    /*!
-      @brief Return the greatest common denominator of integers a and b.
-             Both parameters must be greater than 0.
-     */
-    int gcd(int a, int b);
-
-    /*!
-      @brief Return the greatest common denominator of long values a and b. 
-             Both parameters must be greater than 0.
-     */
-    long lgcd(long a, long b);
-
     /*!
       @brief Return true if str is a hex number starting with prefix followed
-             by size hex digits, false otherwise. If size is 0, any number of 
+             by size hex digits, false otherwise. If size is 0, any number of
              digits is allowed and all are checked.
      */
-    bool isHex(const std::string& str, 
+    bool isHex(const std::string& str,
                size_t size =0,
                const std::string& prefix ="");
 
+    /*!
+      @brief Converts a string in the form "%Y:%m:%d %H:%M:%S", e.g.,
+             "2007:05:24 12:31:55" to broken down time format,
+             returns 0 if successful, else 1.
+     */
+    int exifTime(const char* buf, struct tm* tm);
+
 // *****************************************************************************
 // template and inline definitions
 
+    /*!
+      @brief Find an element that matches \em key in the array \em src.
+
+      Designed to be used with lookup tables as shown in the example below.
+      Requires a %Key structure (ideally in the array) and a comparison operator
+      to compare a key with an array element.  The size of the array is
+      determined automagically. Thanks to Stephan Broennimann for this nifty
+      implementation.
+
+      @code
+      struct Bar {
+          int i;
+          int k;
+          const char* data;
+
+          struct Key;
+          bool operator==(const Bar::Key& rhs) const;
+      };
+
+      struct Bar::Key {
+          Key(int a, int b) : i(a), k(b) {}
+          int i;
+          int k;
+      };
+
+      bool Bar::operator==(const Bar::Key& key) const // definition
+      {
+          return i == key.i && k == key.k;
+      }
+
+      const Bar bars[] = {
+          { 1, 1, "bar data 1" },
+          { 1, 2, "bar data 2" },
+          { 1, 3, "bar data 3" }
+      };
+
+      int main ( void ) {
+          const Bar* bar = find(bars, Bar::Key(1, 3));
+          if (bar) std::cout << bar->data << "\n";
+          else std::cout << "Key not found.\n";
+          return 0;
+      }
+      @endcode
+    */
+    template<typename T, typename K, int N>
+    const T* find(T (&src)[N], const K& key)
+    {
+        const T* rc = std::find(src, src + N, key);
+        return rc == src + N ? 0 : rc;
+    }
+
+    //! Template used in the COUNTOF macro to determine the size of an array
+    template <typename T, int N> char (&sizer(T (&)[N]))[N];
+//! Macro to determine the size of an array
+#define EXV_COUNTOF(a) (sizeof(sizer(a)))
+
     //! Utility function to convert the argument of any type to a string
-    template<typename T> 
+    template<typename T>
     std::string toString(const T& arg)
     {
         std::ostringstream os;
@@ -302,6 +378,47 @@ namespace Exiv2 {
         return os.str();
     }
 
+    /*!
+      @brief Return the greatest common denominator of n and m.
+             (implementation from Boost rational.hpp)
+
+      @note We use n and m as temporaries in this function, so there is no
+            value in using const IntType& as we would only need to make a copy
+            anyway...
+     */
+    template <typename IntType>
+    IntType gcd(IntType n, IntType m)
+    {
+        // Avoid repeated construction
+        IntType zero(0);
+
+        // This is abs() - given the existence of broken compilers with Koenig
+        // lookup issues and other problems, I code this explicitly. (Remember,
+        // IntType may be a user-defined type).
+#ifdef _MSC_VER
+#pragma warning( disable : 4146 )
+#endif
+        if (n < zero)
+            n = -n;
+        if (m < zero)
+            m = -m;
+#ifdef _MSC_VER
+#pragma warning( default : 4146 )
+#endif
+
+        // As n and m are now positive, we can be sure that %= returns a
+        // positive value (the standard guarantees this for built-in types,
+        // and we require it of user-defined types).
+        for(;;) {
+            if(m == zero)
+                return n;
+            n %= m;
+            if(n == zero)
+                return m;
+            m %= n;
+        }
+    }
+
 }                                       // namespace Exiv2
 
 #endif                                  // #ifndef TYPES_HPP_
