@@ -1,32 +1,32 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
- * 
+ * Copyright (C) 2004, 2005, 2006 Andreas Huggel <ahuggel@gmx.net>
+ *
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA.
  */
 /*
   File:      utils.cpp
-  Version:   $Rev: 560 $
+  Version:   $Rev: 669 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   08-Dec-03, ahu: created
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: utils.cpp 560 2005-04-17 11:51:32Z ahuggel $");
+EXIV2_RCSID("@(#) $Id: utils.cpp 669 2006-01-17 14:53:57Z ahuggel $");
 
 // *****************************************************************************
 // included header files
@@ -60,20 +60,25 @@ namespace Util {
 
 // *****************************************************************************
 // class Getopt
-int Getopt::getopt(int argc, char* const argv[], const std::string& optstring)
-{
-    progname_ = Util::basename(argv[0]);
+    int Getopt::getopt(int argc, char* const argv[], const std::string& optstring)
+    {
+        progname_ = Util::basename(argv[0]);
 
-    for (;;) {
-        int c = ::getopt(argc, argv, optstring.c_str());
-        if (c == -1) break;
-        errcnt_ += option(c, ::optarg == 0 ? "" : ::optarg, ::optopt);
+        for (;;) {
+            int c = ::getopt(argc, argv, optstring.c_str());
+            if (c == -1) break;
+            errcnt_ += option(c, ::optarg == 0 ? "" : ::optarg, ::optopt);
+        }
+        for (int i = ::optind; i < argc; i++) {
+            errcnt_ += nonoption(argv[i]);
+        }
+        return errcnt_;
     }
-    for (int i = ::optind; i < argc; i++) {
-        errcnt_ += nonoption(argv[i]);
+
+    int Getopt::nonoption(const std::string& /*argv*/)
+    {
+        return 0;
     }
-    return errcnt_; 
-}
 
 // *****************************************************************************
 // free functions
@@ -83,7 +88,7 @@ int Getopt::getopt(int argc, char* const argv[], const std::string& optstring)
         if (path == "") return ".";
         // Strip trailing slashes or backslashes
         std::string p = path;
-        while (   p.length() > 1 
+        while (   p.length() > 1
                && (p[p.length()-1] == '\\' || p[p.length()-1] == '/')) {
             p = p.substr(0, p.length()-1);
         }
@@ -93,7 +98,7 @@ int Getopt::getopt(int argc, char* const argv[], const std::string& optstring)
         if (idx == std::string::npos) return ".";
         if (idx == 1 && p[0] == '\\' && p[1] == '\\') return p; // For Windows paths
         p = p.substr(0, idx == 0 ? 1 : idx);
-        while (   p.length() > 1 
+        while (   p.length() > 1
                && (p[p.length()-1] == '\\' || p[p.length()-1] == '/')) {
             p = p.substr(0, p.length()-1);
         }
@@ -105,7 +110,7 @@ int Getopt::getopt(int argc, char* const argv[], const std::string& optstring)
         if (path == "") return ".";
         // Strip trailing slashes or backslashes
         std::string p = path;
-        while (   p.length() > 1 
+        while (   p.length() > 1
                && (p[p.length()-1] == '\\' || p[p.length()-1] == '/')) {
             p = p.substr(0, p.length()-1);
         }
@@ -119,7 +124,7 @@ int Getopt::getopt(int argc, char* const argv[], const std::string& optstring)
 
     std::string suffix(const std::string& path)
     {
-        std::string b = basename(path);    
+        std::string b = basename(path);
         std::string::size_type idx = b.rfind('.');
         if (idx == std::string::npos || idx == 0 || idx == b.length()-1) {
             return "";
