@@ -1,26 +1,26 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004, 2005 Andreas Huggel <ahuggel@gmx.net>
- * 
+ * Copyright (C) 2004, 2005, 2006 Andreas Huggel <ahuggel@gmx.net>
+ *
  * This program is part of the Exiv2 distribution.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301 USA.
  */
 /*
   File:      value.cpp
-  Version:   $Rev: 560 $
+  Version:   $Rev: 702 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   26-Jan-04, ahu: created
              11-Feb-04, ahu: isolated as a component
@@ -28,7 +28,7 @@
  */
 // *****************************************************************************
 #include "rcsid.hpp"
-EXIV2_RCSID("@(#) $Id: value.cpp 560 2005-04-17 11:51:32Z ahuggel $");
+EXIV2_RCSID("@(#) $Id: value.cpp 702 2006-03-13 03:55:26Z ahuggel $");
 
 // *****************************************************************************
 // included header files
@@ -41,6 +41,7 @@ EXIV2_RCSID("@(#) $Id: value.cpp 560 2005-04-17 11:51:32Z ahuggel $");
 #include <iomanip>
 #include <sstream>
 #include <cassert>
+#include <cstring>
 #include <ctime>
 
 // *****************************************************************************
@@ -110,6 +111,11 @@ namespace Exiv2 {
         return value;
     } // Value::create
 
+    int Value::setDataArea(const byte* /*buf*/, long /*len*/)
+    {
+        return -1;
+    }
+
     std::string Value::toString() const
     {
         std::ostringstream os;
@@ -125,13 +131,14 @@ namespace Exiv2 {
         return *this;
     }
 
-    void DataValue::read(const byte* buf, long len, ByteOrder byteOrder)
+    int DataValue::read(const byte* buf, long len, ByteOrder byteOrder)
     {
         // byteOrder not needed
         value_.assign(buf, buf + len);
+        return 0;
     }
 
-    void DataValue::read(const std::string& buf)
+    int DataValue::read(const std::string& buf)
     {
         std::istringstream is(buf);
         int tmp;
@@ -139,6 +146,7 @@ namespace Exiv2 {
         while (is >> tmp) {
             value_.push_back(static_cast<byte>(tmp));
         }
+        return 0;
     }
 
     long DataValue::copy(byte* buf, ByteOrder byteOrder) const
@@ -176,20 +184,23 @@ namespace Exiv2 {
         return *this;
     }
 
-    void StringValueBase::read(const std::string& buf)
+    int StringValueBase::read(const std::string& buf)
     {
         value_ = buf;
+        return 0;
     }
 
-    void StringValueBase::read(const byte* buf, long len, ByteOrder byteOrder)
+    int StringValueBase::read(const byte* buf, long len, ByteOrder byteOrder)
     {
-        // byteOrder not needed 
-        value_ = std::string(reinterpret_cast<const char*>(buf), len);
+        // byteOrder not needed
+        if (buf) value_ = std::string(reinterpret_cast<const char*>(buf), len);
+        return 0;
     }
 
     long StringValueBase::copy(byte* buf, ByteOrder byteOrder) const
     {
         // byteOrder not needed
+        assert(buf != 0);
         return static_cast<long>(
             value_.copy(reinterpret_cast<char*>(buf), value_.size())
             );
@@ -224,10 +235,11 @@ namespace Exiv2 {
         return *this;
     }
 
-    void AsciiValue::read(const std::string& buf)
+    int AsciiValue::read(const std::string& buf)
     {
         value_ = buf;
-        if (value_[value_.size()-1] != '\0') value_ += '\0';
+        if (value_.size() > 0 && value_[value_.size()-1] != '\0') value_ += '\0';
+        return 0;
     }
 
     AsciiValue* AsciiValue::clone_() const
@@ -243,7 +255,7 @@ namespace Exiv2 {
     }
 
     CommentValue::CharsetTable::CharsetTable(CharsetId charsetId,
-                                             const char* name, 
+                                             const char* name,
                                              const char* code)
         : charsetId_(charsetId), name_(name), code_(code)
     {
@@ -302,23 +314,28 @@ namespace Exiv2 {
         return *this;
     }
 
-    void CommentValue::read(const std::string& comment)
+    int CommentValue::read(const std::string& comment)
     {
         std::string c = comment;
         CharsetId charsetId = undefined;
         if (comment.length() > 8 && comment.substr(0, 8) == "charset=") {
             std::string::size_type pos = comment.find_first_of(' ');
             std::string name = comment.substr(8, pos-8);
-            // Strip quotes (so you can also to specify the charset without quotes)
+            // Strip quotes (so you can also specify the charset without quotes)
             if (name[0] == '"') name = name.substr(1);
             if (name[name.length()-1] == '"') name = name.substr(0, name.length()-1);
             charsetId = CharsetInfo::charsetIdByName(name);
-            if (charsetId == invalidCharsetId) throw Error(28, name);
+            if (charsetId == invalidCharsetId) {
+#ifndef SUPPRESS_WARNINGS
+                std::cerr << Error(28, name) << "\n";
+#endif
+                return 1;
+            }
             c.clear();
             if (pos != std::string::npos) c = comment.substr(pos+1);
         }
         const std::string code(CharsetInfo::code(charsetId), 8);
-        StringValueBase::read(code + c);
+        return StringValueBase::read(code + c);
     }
 
     std::ostream& CommentValue::write(std::ostream& os) const
@@ -343,15 +360,15 @@ namespace Exiv2 {
             const std::string code = value_.substr(0, 8);
             charsetId = CharsetInfo::charsetIdByCode(code);
         }
-        return charsetId;        
+        return charsetId;
     }
 
     CommentValue* CommentValue::clone_() const
     {
-        return new CommentValue(*this);        
+        return new CommentValue(*this);
     }
 
-    DateValue::DateValue(int year, int month, int day) 
+    DateValue::DateValue(int year, int month, int day)
         : Value(date)
     {
         date_.year = year;
@@ -369,43 +386,63 @@ namespace Exiv2 {
         return *this;
     }
 
-    void DateValue::read(const byte* buf, long len, ByteOrder byteOrder)
+    int DateValue::read(const byte* buf, long len, ByteOrder /*byteOrder*/)
     {
-        // byteOrder not needed 
         // Hard coded to read Iptc style dates
-        if (len != 8) throw Error(29);
-        int scanned = sscanf(reinterpret_cast<const char*>(buf), 
-                   "%4d%2d%2d", 
-                   &date_.year, &date_.month, &date_.day);
-        if (scanned != 3) throw Error(29);
+        if (len != 8) {
+#ifndef SUPPRESS_WARNINGS
+            std::cerr << Error(29) << "\n";
+#endif
+            return 1;
+        }
+        // Make the buffer a 0 terminated C-string for sscanf
+        char b[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0 };
+        memcpy(b, reinterpret_cast<const char*>(buf), 8);
+        int scanned = sscanf(b, "%4d%2d%2d",
+                             &date_.year, &date_.month, &date_.day);
+        if (scanned != 3) {
+#ifndef SUPPRESS_WARNINGS
+            std::cerr << Error(29) << "\n";
+#endif
+            return 1;
+        }
+        return 0;
     }
 
-    void DateValue::read(const std::string& buf)
+    int DateValue::read(const std::string& buf)
     {
-        // byteOrder not needed 
         // Hard coded to read Iptc style dates
-        if (buf.length() < 8) throw Error(29);
-        int scanned = sscanf(buf.data(), 
-                   "%4d-%d-%d", 
-                   &date_.year, &date_.month, &date_.day);
-        if (scanned != 3) throw Error(29);
+        if (buf.length() < 8) {
+#ifndef SUPPRESS_WARNINGS
+            std::cerr << Error(29) << "\n";
+#endif
+            return 1;
+        }
+        int scanned = sscanf(buf.c_str(), "%4d-%d-%d",
+                             &date_.year, &date_.month, &date_.day);
+        if (scanned != 3) {
+#ifndef SUPPRESS_WARNINGS
+            std::cerr << Error(29) << "\n";
+#endif
+            return 1;
+        }
+        return 0;
     }
 
-    void DateValue::setDate( const Date& src )
+    void DateValue::setDate(const Date& src)
     {
         date_.year = src.year;
         date_.month = src.month;
         date_.day = src.day;
     }
-    
-    long DateValue::copy(byte* buf, ByteOrder byteOrder) const
+
+    long DateValue::copy(byte* buf, ByteOrder /*byteOrder*/) const
     {
-        // byteOrder not needed
         // sprintf wants to add the null terminator, so use oversized buffer
         char temp[9];
 
-        int wrote = sprintf( temp, "%04d%02d%02d", 
-                           date_.year, date_.month, date_.day);
+        int wrote = sprintf(temp, "%04d%02d%02d",
+                            date_.year, date_.month, date_.day);
         assert(wrote == 8);
         memcpy(buf, temp, 8);
         return 8;
@@ -428,7 +465,7 @@ namespace Exiv2 {
                << std::setw(2) << std::setfill('0') << date_.day;
     }
 
-    long DateValue::toLong(long n) const 
+    long DateValue::toLong(long n) const
     {
         // Range of tm struct is limited to about 1970 to 2038
         // This will return -1 if outside that range
@@ -440,8 +477,8 @@ namespace Exiv2 {
         return static_cast<long>(std::mktime(&tms));
     }
 
-    TimeValue::TimeValue(int hour, int minute, 
-                         int second, int tzHour, 
+    TimeValue::TimeValue(int hour, int minute,
+                         int second, int tzHour,
                          int tzMinute)
         : Value(date)
     {
@@ -460,58 +497,100 @@ namespace Exiv2 {
         return *this;
     }
 
-    void TimeValue::read(const byte* buf, long len, ByteOrder byteOrder)
+    int TimeValue::read(const byte* buf, long len, ByteOrder /*byteOrder*/)
     {
-        // byteOrder not needed 
-        // Hard coded to read Iptc style times
-        if (len != 11) throw Error(30);
-        char plusMinus;
-        int scanned = sscanf(reinterpret_cast<const char*>(buf), 
-                   "%2d%2d%2d%1c%2d%2d", 
-                   &time_.hour, &time_.minute, &time_.second, 
-                   &plusMinus, &time_.tzHour, &time_.tzMinute );
-
-        if (scanned != 6) throw Error(30);
-        if (plusMinus == '-') {
-            time_.tzHour *= -1;
-            time_.tzMinute *= -1;
+        // Make the buffer a 0 terminated C-string for scanTime[36]
+        char b[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
+        memcpy(b, reinterpret_cast<const char*>(buf), (len < 12 ? len : 11));
+        // Hard coded to read HHMMSS or Iptc style times
+        int rc = 1;
+        if (len == 6) {
+            // Try to read (non-standard) HHMMSS format
+            rc = scanTime3(b, "%2d%2d%2d");
         }
+        if (len == 11) {
+            rc = scanTime6(b, "%2d%2d%2d%1c%2d%2d");
+        }
+#ifndef SUPPRESS_WARNINGS
+        if (rc) {
+            std::cerr << Error(30) << "\n";
+        }
+#endif
+        return rc;
     }
 
-    void TimeValue::read(const std::string& buf)
+    int TimeValue::read(const std::string& buf)
     {
-        // byteOrder not needed 
-        // Hard coded to read Iptc style times
-        if (buf.length() < 9) throw Error(30);
-        char plusMinus;
-        int scanned = sscanf(buf.data(),
-                   "%d:%d:%d%1c%d:%d", 
-                   &time_.hour, &time_.minute, &time_.second, 
-                   &plusMinus, &time_.tzHour, &time_.tzMinute );
-
-        if (scanned != 6) throw Error(30);
-        if (plusMinus == '-') {
-            time_.tzHour *= -1;
-            time_.tzMinute *= -1;
+        // Hard coded to read H:M:S or Iptc style times
+        int rc = 1;
+        if (buf.length() < 9) {
+            // Try to read (non-standard) H:M:S format
+            rc = scanTime3(buf.c_str(), "%d:%d:%d");
         }
+        else {
+            rc = scanTime6(buf.c_str(), "%d:%d:%d%1c%d:%d");
+        }
+#ifndef SUPPRESS_WARNINGS
+        if (rc) {
+            std::cerr << Error(30) << "\n";
+        }
+#endif
+        return rc;
+    }
+
+    int TimeValue::scanTime3(const char* buf, const char* format)
+    {
+        int rc = 1;
+        Time t;
+        int scanned = sscanf(buf, format, &t.hour, &t.minute, &t.second);
+        if (   scanned  == 3
+            && t.hour   >= 0 && t.hour   < 24
+            && t.minute >= 0 && t.minute < 60
+            && t.second >= 0 && t.second < 60) {
+            time_ = t;
+            rc = 0;
+        }
+        return rc;
+    }
+
+    int TimeValue::scanTime6(const char* buf, const char* format)
+    {
+        int rc = 1;
+        Time t;
+        char plusMinus;
+        int scanned = sscanf(buf, format, &t.hour, &t.minute, &t.second,
+                             &plusMinus, &t.tzHour, &t.tzMinute);
+        if (   scanned    == 6
+            && t.hour     >= 0 && t.hour     < 24
+            && t.minute   >= 0 && t.minute   < 60
+            && t.second   >= 0 && t.second   < 60
+            && t.tzHour   >= 0 && t.tzHour   < 24
+            && t.tzMinute >= 0 && t.tzMinute < 60) {
+            time_ = t;
+            if (plusMinus == '-') {
+                time_.tzHour *= -1;
+                time_.tzMinute *= -1;
+            }
+            rc = 0;
+        }
+        return rc;
     }
 
     void TimeValue::setTime( const Time& src )
     {
         memcpy(&time_, &src, sizeof(time_));
     }
-    
-    long TimeValue::copy(byte* buf, ByteOrder byteOrder) const
+
+    long TimeValue::copy(byte* buf, ByteOrder /*byteOrder*/) const
     {
-        // byteOrder not needed
         // sprintf wants to add the null terminator, so use oversized buffer
         char temp[12];
         char plusMinus = '+';
         if (time_.tzHour < 0 || time_.tzMinute < 0) plusMinus = '-';
 
-        int wrote = sprintf(temp, 
-                   "%02d%02d%02d%1c%02d%02d", 
-                   time_.hour, time_.minute, time_.second, 
+        int wrote = sprintf(temp,
+                   "%02d%02d%02d%1c%02d%02d",
+                   time_.hour, time_.minute, time_.second,
                    plusMinus, abs(time_.tzHour), abs(time_.tzMinute));
 
         assert(wrote == 11);
@@ -533,7 +612,7 @@ namespace Exiv2 {
     {
         char plusMinus = '+';
         if (time_.tzHour < 0 || time_.tzMinute < 0) plusMinus = '-';
-        
+
         return os << std::right
            << std::setw(2) << std::setfill('0') << time_.hour << ':'
            << std::setw(2) << std::setfill('0') << time_.minute << ':'
@@ -542,9 +621,9 @@ namespace Exiv2 {
            << std::setw(2) << std::setfill('0') << abs(time_.tzMinute);
     }
 
-    long TimeValue::toLong(long n) const 
+    long TimeValue::toLong(long n) const
     {
-        // Returns number of seconds in the day in UTC. 
+        // Returns number of seconds in the day in UTC.
         long result = (time_.hour - time_.tzHour) * 60 * 60;
         result += (time_.minute - time_.tzMinute) * 60;
         result += time_.second;
