@@ -3,13 +3,13 @@
   Abstract : ExifData write unit tests
 
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
-  Version  : $Rev: 560 $
+  Version  : $Rev: 650 $
 
-  Test procedure: 
-   $ rm -f test.jpg thumb.jpg iii ttt; 
+  Test procedure:
+   $ rm -f test.jpg thumb.jpg iii ttt;
    $ ./exifprint ../test/img_1771.jpg > iii;
-   $ cp ../test/img_1771.jpg ./test.jpg; 
-   $ ./makernote-test2 ../test/img_1771.jpg > ttt; 
+   $ cp ../test/img_1771.jpg ./test.jpg;
+   $ ./makernote-test2 ../test/img_1771.jpg > ttt;
    $ diff iii ttt
 
  */
@@ -60,8 +60,8 @@ try {
     case 1:
         std::cerr << "Case 1: ";
         std::cerr << "Non-intrusive change to the standard Exif metadata\n";
-        testCase(testFile, "test1.jpg", "thumb1", 
-                 "Exif.Photo.DateTimeOriginal", 
+        testCase(testFile, "test1.jpg", "thumb1",
+                 "Exif.Photo.DateTimeOriginal",
                  "1999:11:22 00:11:22");
         break;
     case 2:
@@ -75,14 +75,14 @@ try {
         std::cerr << "Case 3: ";
         std::cerr << "Non-intrusive change to the Exif metadata (w/o makernote)\n";
         testCase(testFile, "test3.jpg", "thumb3",
-                 "Exif.Photo.DateTimeOriginal", 
+                 "Exif.Photo.DateTimeOriginal",
                  "1999:11:22 00:11:22");
         break;
     case 4:
         std::cerr << "Case 4: ";
         std::cerr << "Intrusive change to the standard Exif metadata\n";
         testCase(testFile, "test4.jpg", "thumb4",
-                 "Exif.Photo.DateTimeOriginal", 
+                 "Exif.Photo.DateTimeOriginal",
                  "1999:11:22 00:11:22 and twenty seconds");
         break;
     case 5:
@@ -96,7 +96,7 @@ try {
         std::cerr << "Case 6: ";
         std::cerr << "Intrusive change to the Exif metadata (w/o makernote)\n";
         testCase(testFile, "test6.jpg", "thumb6",
-                 "Exif.Photo.DateTimeOriginal", 
+                 "Exif.Photo.DateTimeOriginal",
                  "1999:11:22 00:11:22 and twenty seconds");
         break;
     case 7:
@@ -204,11 +204,16 @@ void exifPrint(const ExifData& exifData)
 {
     ExifData::const_iterator i = exifData.begin();
     for (; i != exifData.end(); ++i) {
-        std::cout << std::setw(53) << std::setfill(' ') << std::left
+        std::cout << std::setw(44) << std::setfill(' ') << std::left
                   << i->key() << " "
                   << "0x" << std::setw(4) << std::setfill('0') << std::right
-                  << std::hex << i->tag() << "  " 
-                  << std::dec << i->value() 
+                  << std::hex << i->tag() << " "
+                  << std::setw(9) << std::setfill(' ') << std::left
+                  << i->typeName() << " "
+                  << std::dec << std::setw(3)
+                  << std::setfill(' ') << std::right
+                  << i->count() << "  "
+                  << std::dec << i->value()
                   << "\n";
     }
 }
